SystemOrganization addCategory: #'iStoaXP-Core'!
SystemOrganization addCategory: 'iStoaXP-Core-Board'!
SystemOrganization addCategory: 'iStoaXP-Core-Game'!
SystemOrganization addCategory: 'iStoaXP-Core-Match'!
SystemOrganization addCategory: 'iStoaXP-Core-Widget'!

Trait named: #TMatchByBlock
	uses: {}
	category: 'iStoaXP-Core-Match'!

!TMatchByBlock commentStamp: 'HilaireFernandes 7/25/2014 19:04' prior: 0!
Behavior for Match games (association)  where the validity of the user answers is checked with blocks of code.!

!TMatchByBlock methodsFor: 'initialize' stamp: 'HilaireFernandes 8/5/2014 13:16'!
basket: itemsCollection targets: targetsCollection answers: answersCollection
	"Instanciate the targets"
	self targets: (targetsCollection collect: [:targetAttribute |
		targetAttribute first asMatchTargetBlock validityBlock: targetAttribute second]).
	self rightAnswers: answersCollection.
	self installBasket: itemsCollection.
	self installTargets.! !

!TMatchByBlock methodsFor: 'initialize' stamp: 'HilaireFernandes 7/26/2014 23:01'!
basketInstance
	^ MatchBasketBlockMorph new.! !

!TMatchByBlock methodsFor: 'initialize' stamp: 'HilaireFernandes 7/25/2014 19:28'!
installBasket: itemsCollection
	self basket 
		addAllMorphs: (itemsCollection shuffled collect: [:item | item asMatchItemBlock]);
		wrapDirection: #none; 
		wrapDirection: #topToBottom.
! !

!TMatchByBlock methodsFor: 'accessing' stamp: 'HilaireFernandes 7/25/2014 19:28'!
numberOfQuestions
"Each target is a question, we are not more accurate now as guess per target sub question may not be easy"
	^ self targets size! !

!TMatchByBlock methodsFor: 'helpers' stamp: 'HilaireFernandes 7/25/2014 19:28'!
showErrors
	self targets do: [ :aTarget | aTarget markAsRightOrWrong  ].! !

!TMatchByBlock methodsFor: 'helpers' stamp: 'HilaireFernandes 8/5/2014 13:17'!
showSolution
	self resetTargets.
	self targets withIndexDo: [ :target :index | 
		(self rightAnswers at: index) do: [ :anAnswerType | | validItem |
			validItem := self basket giveMe: anAnswerType.
			target gotItem: validItem]].
	self showErrors! !

!TMatchByBlock methodsFor: 'tests' stamp: 'HilaireFernandes 7/25/2014 19:28'!
testSuccessAnswer
	^ self targets allSatisfy:  [:aTarget | aTarget isRight]! !

!TMatchByBlock methodsFor: 'accessing' stamp: 'HilaireFernandes 7/28/2014 00:05'!
userRightAnswers 
"Collect the targets with the correct answers"
	^ self targets select: [ :aTarget | aTarget isRight ]! !

Trait named: #TMatchByCollection
	uses: {}
	category: 'iStoaXP-Core-Match'!

!TMatchByCollection commentStamp: 'HilaireFernandes 7/25/2014 19:06' prior: 0!
Behavior for Match games (association)  where the validity of the user answers is checked by comparing to a collection of  answers.!

!TMatchByCollection methodsFor: 'initialize' stamp: 'HilaireFernandes 7/26/2014 23:01'!
basketInstance
	^ MatchBasketMorph new.! !

!TMatchByCollection methodsFor: 'initialize' stamp: 'HilaireFernandes 7/25/2014 18:34'!
installBasket: itemsCollection
	|allAnswers|
	allAnswers := OrderedCollection new.
	itemsCollection do: [:targetAnswers| allAnswers addAll: targetAnswers].
	self basket 
		addAllMorphs: allAnswers shuffled;
		wrapDirection: #none; 
		wrapDirection: #topToBottom
! !

!TMatchByCollection methodsFor: 'accessing' stamp: 'HilaireFernandes 8/5/2014 13:18'!
numberOfQuestions
	^ self rightAnswers inject: 0 into: [:sum :answers | sum + answers size]! !

!TMatchByCollection methodsFor: 'helpers' stamp: 'HilaireFernandes 8/5/2014 13:18'!
showErrors
	"Clear out all, including the answer without matching target"
	self rightAnswers do: [:someAnswers |
		someAnswers do: [:aMatchItem | aMatchItem markAsRight]].
	"Mark the items wrong or right "
	self targets do: [:aTarget | 
		aTarget items do: [:aMatchItem | aMatchItem markAsRightOrWrong: aTarget]].
	"Mark wrong the item in the basket that should be associated to a target"
	self basket submorphsDo: [ :aMatchItem |
		aMatchItem targets ifNotEmpty: [ aMatchItem markAsWrong]]! !

!TMatchByCollection methodsFor: 'helpers' stamp: 'HilaireFernandes 7/25/2014 18:36'!
showSolution
	self resetTargets.
	self basket submorphsDo: [ :aMatchItem |
		aMatchItem targets do: [:aTarget |	aTarget gotItem: aMatchItem]].
	self showErrors "to clear out items marked wrong"! !

!TMatchByCollection methodsFor: 'initialize' stamp: 'HilaireFernandes 8/5/2014 13:18'!
targets: targetsCollection answers: answersCollection
	answersCollection doWithIndex: [:someAnswers :answerIndex | 
		| theTarget targetAnswers | 
		"Instanciate the target only for valid answers (remember we may have invalid answers)"
		theTarget := answerIndex <= targetsCollection size 
			ifTrue: [self targets add: (targetsCollection at: answerIndex) asMatchTarget].
		self rightAnswers add: (targetAnswers := OrderedCollection new).
		"Associate theTarget to these answers, associate nil (theTarget value) if none"
		someAnswers do: [:anItem | 
			(targetAnswers add: anItem asMatchItem) addTarget: theTarget]].
	self installBasket: self rightAnswers.
	self installTargets! !

!TMatchByCollection methodsFor: 'tests' stamp: 'HilaireFernandes 8/5/2014 13:18'!
testSuccessAnswer
	self userAnswers doWithIndex:  [:userAnswer :index | 
		userAnswer asBag = (self rightAnswers at: index) asBag ifFalse: [^ false]].
	^ true! !

!TMatchByCollection methodsFor: 'accessing' stamp: 'HilaireFernandes 7/28/2014 00:07'!
userRightAnswers
"Collect the items correctly matched with the targets or the basket"
	| rightAnswers |
	rightAnswers := OrderedCollection new.
	self targets do: [:aTarget |
		aTarget items do: [:item | (item isMatching: aTarget) ifTrue: [rightAnswers add: item]]].
	self basket submorphsDo: [:item| item targets ifEmpty: [ rightAnswers add: item]].
	^ rightAnswers! !

Morph subclass: #BoardMorph
	instanceVariableNames: 'random question activity mascotQuestion illustration'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'iStoaXP-Core-Board'!

!BoardMorph commentStamp: '<historical>' prior: 0!
A BoardMorph is both a model and a view for a board game.
 - as a model it defines the logic of the game, 
 - as a a view it is the place where to layout the board game components (morphs).

GOLDEN RULE: no media should be loaded from a BoardMorph. The needed resources must be installed from the Etayage.

By default, it is a table layout, in a top to bottom column direction, with centered cells.

- random, our random generator!

BoardMorph subclass: #BoardBlockMatch
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'iStoaXP-Core-Board'!

!BoardBlockMatch commentStamp: 'HilaireFernandes 8/5/2014 12:50' prior: 0!
An association activity, it comes with:
- a basked with a collection of draggable items 
- one or more line, each with an illustration and a collection of  basket to drop items.
- targets testing is done with block

The targets are arranged in rows, one per row. 
Alternatively, the targets are integrated in a composite morph at arbitrary position.
!

!BoardBlockMatch class methodsFor: 'new board - row' stamp: 'HilaireFernandes 7/23/2014 07:26'!
question: aString basket: items targets: targets answers: answers
	^ self question: aString 
		illustration: nil 
		basket: items 
		targets: targets 
		answers: answers
		listDirection: #topToBottom! !

!BoardBlockMatch class methodsFor: 'new board - row' stamp: 'HilaireFernandes 7/23/2014 07:29'!
question: aString basket: items targets: targets answers: answers listDirection: symbol
	^ self question: aString 
		illustration: nil 
		basket: items 
		targets: targets 
		answers: answers
		listDirection: symbol! !

!BoardBlockMatch class methodsFor: 'new board - composite' stamp: 'HilaireFernandes 8/5/2014 23:34'!
question: aString composite: composite basket: items targets: targets answers: answers
	^ self question: aString composite: composite basket: items targets: targets answers: answers listDirection: #topToBottom! !

!BoardBlockMatch class methodsFor: 'new board - composite' stamp: 'HilaireFernandes 8/5/2014 23:32'!
question: aString composite: composite basket: items targets: targets answers: answers listDirection: symbol
	| board |
	board := self basicNew question: aString;
		composite: composite basket: items targets: targets answers: answers;
		initialize;
		yourself.
	board activity listDirection: symbol.
	^ board! !

!BoardBlockMatch class methodsFor: 'new board - row' stamp: 'HilaireFernandes 7/23/2014 07:28'!
question: aString illustration: anIllustration basket: items targets: targets answers: answers
	^ self question: aString 
		illustration: anIllustration 
		basket: items 
		targets: targets 
		answers: answers
		listDirection: #topToBottom! !

!BoardBlockMatch class methodsFor: 'new board - row' stamp: 'HilaireFernandes 7/23/2014 12:10'!
question: aString illustration: anIllustration basket: items targets: targets answers: answers listDirection: symbol
	| board |
	board := self basicNew question: aString;
		illustration: anIllustration ;
		basket: items targets: targets answers: answers;
		initialize;
		yourself.
	board activity listDirection: symbol.
	^ board! !

!BoardBlockMatch methodsFor: 'initialize' stamp: 'HilaireFernandes 7/25/2014 18:33'!
basket: items targets: targets answers: answers
	activity := MatchRowBlockMorph basket: items targets: targets answers: answers! !

!BoardBlockMatch methodsFor: 'initialize' stamp: 'HilaireFernandes 8/5/2014 23:24'!
composite: composite basket: items targets: targets answers: answers
	activity := MatchCompositeBlockMorph composite: composite basket: items targets: targets answers: answers! !

BoardMorph subclass: #BoardMatch
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'iStoaXP-Core-Board'!

!BoardMatch commentStamp: 'HilaireFernandes 8/5/2014 12:50' prior: 0!
An association activity, it comes with:
- a collection of targets
- a collection of answers, right and wrong 
- one or more line, each with an illustration and a collection of  basket to drop items.

The targets are arranged in rows, one per row. 
Alternatively, the targets are integrated in a composite morph at arbitrary position.!

!BoardMatch class methodsFor: 'new board - composite' stamp: 'HilaireFernandes 7/26/2014 13:09'!
question: aString composite: composite targets: targets answers: answers
	^ self question: aString composite: composite targets: targets answers: answers listDirection: #topToBottom
! !

!BoardMatch class methodsFor: 'new board - composite' stamp: 'HilaireFernandes 7/26/2014 13:11'!
question: aString composite: composite targets: targets answers: answers listDirection: symbol
	| board |
	board := self basicNew question: aString;
		composite: composite targets: targets answers: answers;
		initialize;
		yourself.
	board activity listDirection: symbol.
	^ board! !

!BoardMatch class methodsFor: 'new board - row' stamp: 'HilaireFernandes 7/23/2014 07:33'!
question: aString illustration: anIllustration targets: targets answers: answers
	^ self question: aString illustration: anIllustration targets: targets answers: answers listDirection: #topToBottom
! !

!BoardMatch class methodsFor: 'new board - row' stamp: 'HilaireFernandes 7/23/2014 07:30'!
question: aString illustration: anIllustration targets: targets answers: answers listDirection: symbol
	| board |
	board := self basicNew question: aString;
		illustration: anIllustration;
		targets: targets answers: answers;
		initialize;
		yourself.
	board activity listDirection: symbol.
	^ board! !

!BoardMatch class methodsFor: 'new board - row' stamp: 'HilaireFernandes 7/23/2014 07:32'!
question: aString targets: targets answers: answers
	^ self question: aString illustration: nil targets: targets answers: answers listDirection: #topToBottom! !

!BoardMatch class methodsFor: 'new board - row' stamp: 'HilaireFernandes 7/23/2014 07:33'!
question: aString targets: targets answers: answers listDirection: symbol
	^ self question: aString illustration: nil targets: targets answers: answers listDirection: symbol! !

!BoardMatch methodsFor: 'initialize' stamp: 'HilaireFernandes 7/26/2014 13:12'!
composite: composite targets: targets answers: answers
	activity := MatchCompositeMorph composite: composite targets: targets answers: answers! !

!BoardMatch methodsFor: 'initialize' stamp: 'HilaireFernandes 7/25/2014 18:33'!
targets: targets answers: answers
	activity := MatchRowMorph targets: targets answers: answers! !

!BoardMorph methodsFor: 'accessing' stamp: 'HilaireFernandes 7/23/2014 07:24'!
activity
	^ activity ! !

!BoardMorph methodsFor: 'accessing' stamp: 'HilaireFernandes 7/24/2014 13:33'!
activityPlug
"Do we have an illustration to add along the activity?"
	illustration ifNotNil: [ | morph |
		morph := Morph new color: Color transparent;
			layoutPolicy: TableLayout new;
			listDirection: #leftToRight;
			listCentering: #center;
			wrapCentering: #center;
			cellInset: 5@0;
			hResizing: #spaceFill;
			vResizing: #spaceFill;
			addMorphBack: illustration;
			addMorphBack: activity.
		^ morph].
	^ activity ! !

!BoardMorph methodsFor: 'board manipulation' stamp: 'HilaireFernandes 4/24/2014 10:37'!
dialog: aMascotMessage
"replace the board information area with the given message"
	self replaceSubmorph: self firstSubmorph by: aMascotMessage.! !

!BoardMorph methodsFor: 'board manipulation' stamp: 'HilaireFernandes 4/23/2014 09:39'!
getBackQuestion
"get back the mascot question"
	self replaceSubmorph: self firstSubmorph  by: self mascotQuestion .! !

!BoardMorph methodsFor: 'accessing' stamp: 'HilaireFernandes 6/8/2014 12:03'!
illustration: aMorph
	illustration := aMorph ! !

!BoardMorph methodsFor: 'initialize' stamp: 'HilaireFernandes 6/7/2014 17:48'!
initialize 
	super initialize.
	self color: (Color white alpha: 0.8).
	self layoutPolicy: TableLayout new;
		listDirection: #topToBottom;
		cellPositioning: #center;
		listCentering: #center;
		wrapCentering: #center;
		cellInset: 0@20;
		layoutInset: 0@20;
		clipSubmorphs: true.
	self addMorphBack: (mascotQuestion := Stoa mascotSays: self question);
		addMorphBack: Stoa vfill;
		addMorphBack: self activityPlug;
		addMorphBack: Stoa vfill.
		! !

!BoardMorph methodsFor: 'testing' stamp: 'HilaireFernandes 1/15/2014 15:37'!
isSuccess
	^ activity testSuccessAnswer! !

!BoardMorph methodsFor: 'accessing' stamp: 'HilaireFernandes 4/24/2014 10:31'!
mascotQuestion
	^ mascotQuestion! !

!BoardMorph methodsFor: 'user achievement' stamp: 'HilaireFernandes 7/27/2014 23:45'!
numberOfQuestions
	^ activity numberOfQuestions! !

!BoardMorph methodsFor: 'user achievement' stamp: 'HilaireFernandes 7/27/2014 23:47'!
numberOfRightAnswers
	^ activity numberOfRightAnswers! !

!BoardMorph methodsFor: 'user achievement' stamp: 'HilaireFernandes 7/27/2014 23:51'!
numberOfWrongAnswers
	^ self numberOfQuestions - self numberOfRightAnswers ! !

!BoardMorph methodsFor: 'board manipulation' stamp: 'HilaireFernandes 12/10/2013 21:10'!
pause! !

!BoardMorph methodsFor: 'accessing' stamp: 'HilaireFernandes 12/11/2013 20:32'!
question
	^ question! !

!BoardMorph methodsFor: 'accessing' stamp: 'HilaireFernandes 12/11/2013 20:32'!
question: aString
	question := aString! !

!BoardMorph methodsFor: 'accessing' stamp: 'HilaireFernandes 12/10/2013 21:10'!
randomGenerator
	^ random! !

!BoardMorph methodsFor: 'user achievement' stamp: 'HilaireFernandes 7/27/2014 23:52'!
ratioSuccess
	^ self numberOfRightAnswers / self numberOfQuestions! !

!BoardMorph methodsFor: 'user achievement' stamp: 'HilaireFernandes 7/27/2014 23:41'!
rightAnswers
"a collection of the right answers to the test"
	self subclassResponsibility ! !

!BoardMorph methodsFor: 'accessing' stamp: 'HilaireFernandes 12/10/2013 21:10'!
seed: anInteger 
	random := Random seed: anInteger.! !

!BoardMorph methodsFor: 'helpers' stamp: 'HilaireFernandes 1/15/2014 15:36'!
showErrors
	activity showErrors ! !

!BoardMorph methodsFor: 'helpers' stamp: 'HilaireFernandes 1/15/2014 15:36'!
showSolution
	activity showSolution ! !

!BoardMorph methodsFor: 'board manipulation' stamp: 'HilaireFernandes 12/10/2013 21:10'!
start
! !

!BoardMorph methodsFor: 'testing' stamp: 'HilaireFernandes 12/13/2013 18:58'!
testers
	^self class allSelectors 
		select: [:each | each beginsWith: #testSuccess].
! !

!BoardMorph methodsFor: 'board manipulation' stamp: 'HilaireFernandes 12/10/2013 21:10'!
unpause! !

!BoardMorph methodsFor: 'user achievement' stamp: 'HilaireFernandes 7/27/2014 23:42'!
userRightAnswers
"A collection of the right answers given by the user in the activity"
	^ self subclassResponsibility ! !

BoardMorph subclass: #PluggableBoard
	instanceVariableNames: 'showErrors showSolution isSuccess'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'iStoaXP-Core-Board'!

!PluggableBoard commentStamp: 'HilaireFernandes 8/7/2014 10:39' prior: 0!
A plugged board is a kind of generic board to defined different kind of board game. No subclassing should be needed.

- activity: the sole morph to represent the game
- showErrors: bloc of code
- showSolution:  bloc
- isSuccess: bloc or array of blocs!

!PluggableBoard class methodsFor: 'instance creation' stamp: 'HilaireFernandes 1/22/2014 20:25'!
question: aString activity: aMorph isSuccess: blocSuccess showErrors: blocErrors showSolution: blocSolution
	^ self question: aString
		illustration: nil
		activity: aMorph
		isSuccess: blocSuccess
		showErrors: blocErrors
		showSolution: blocSolution
! !

!PluggableBoard class methodsFor: 'instance creation' stamp: 'HilaireFernandes 1/22/2014 20:37'!
question: aString illustration: anIllustration activity: aMorph isSuccess: blocSuccess showErrors: blocErrors showSolution: blocSolution
	^ self basicNew question: aString;
		activity: aMorph;
		illustration: anIllustration;
		isSuccess: blocSuccess;
		showErrors: blocErrors;
		showSolution: blocSolution;
		initialize;
		yourself! !

!PluggableBoard class methodsFor: 'instance creation' stamp: 'HilaireFernandes 5/27/2014 20:56'!
testQuestion: aString illustration: anIllustration activity: aMorph isSuccess: blocSuccess showErrors: blocErrors showSolution: blocSolution
	^ self basicNew question: aString;
		activity: aMorph;
		illustration: anIllustration;
		isSuccess: blocSuccess;
		showErrors: blocErrors;
		showSolution: blocSolution;
		yourself! !

!PluggableBoard methodsFor: 'accessing' stamp: 'HilaireFernandes 6/8/2014 12:03'!
activity: aMorph
	activity := aMorph ! !

!PluggableBoard methodsFor: 'testing' stamp: 'HilaireFernandes 5/27/2014 21:04'!
isSuccess
	^ isSuccess isClosure 
		ifTrue: [isSuccess value]
		ifFalse: [ isSuccess allSatisfy: [:aSuccessBlock | aSuccessBlock value]]! !

!PluggableBoard methodsFor: 'accessing' stamp: 'HilaireFernandes 5/27/2014 20:31'!
isSuccess: blocOrArray
	isSuccess := blocOrArray! !

!PluggableBoard methodsFor: 'user achievement' stamp: 'HilaireFernandes 7/28/2014 01:02'!
numberOfQuestions
	(activity respondsTo: #numberOfQuestions) ifTrue: [ ^ activity numberOfQuestions].
	^ isSuccess isClosure 
		ifTrue: [ 1 ]
		ifFalse: [ isSuccess size ]! !

!PluggableBoard methodsFor: 'user achievement' stamp: 'HilaireFernandes 7/28/2014 01:02'!
numberOfRightAnswers
	(activity respondsTo: #numberOfRightAnswers) ifTrue: [ ^ activity numberOfRightAnswers].
	^ isSuccess isClosure 
		ifTrue: [ isSuccess value asBit]
		ifFalse: [ isSuccess inject: 0 into: [:successes : next | successes + next value asBit] ]! !

!PluggableBoard methodsFor: 'helpers' stamp: 'HilaireFernandes 1/18/2014 18:46'!
showErrors
	showErrors value! !

!PluggableBoard methodsFor: 'accessing' stamp: 'HilaireFernandes 1/18/2014 18:44'!
showErrors: bloc
	showErrors := bloc! !

!PluggableBoard methodsFor: 'helpers' stamp: 'HilaireFernandes 2/2/2014 11:22'!
showSolution
	showSolution value.
	self showErrors "To wipe out the red marked area"! !

!PluggableBoard methodsFor: 'accessing' stamp: 'HilaireFernandes 1/18/2014 18:45'!
showSolution: bloc
	showSolution := bloc! !

Morph subclass: #IllustratedTextMorph
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'iStoaXP-Core-Widget'!

!IllustratedTextMorph commentStamp: 'HilaireFernandes 5/27/2014 15:59' prior: 0!
An illustration morph with a text attached bellow.

(IllustratedTextMorph 
	illustration:  ThemeIcons current volumeIcon asMorph 
	of: 10) openInWorld!

!IllustratedTextMorph class methodsFor: 'as yet unclassified' stamp: 'HilaireFernandes 1/18/2014 21:04'!
illustration: aMorph of: aString
	^ self new
		addMorph: aMorph;
		addMorphBack:  (aString asMorph font: Stoa textFont);
		yourself! !

!IllustratedTextMorph methodsFor: 'accessing' stamp: 'HilaireFernandes 1/24/2014 21:31'!
illustration
	^ self firstSubmorph ! !

!IllustratedTextMorph methodsFor: 'initialize' stamp: 'HilaireFernandes 1/5/2014 17:03'!
initialize 
	super initialize.
	self color: Color transparent.
	self layoutPolicy: TableLayout new;
		borderWidth: 1;
		borderColor: Color black;
		listDirection: #topToBottom;
		vResizing: #shrinkWrap;
		hResizing: #shrinkWrap;
		wrapCentering: #center;
		cellPositioning: #topCenter;
		layoutInset: 2.

! !

!IllustratedTextMorph methodsFor: 'accessing' stamp: 'HilaireFernandes 1/5/2014 15:51'!
stringMorph
	^ self lastSubmorph ! !

Morph subclass: #MatchAbstract
	instanceVariableNames: 'basket targets rightAnswers'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'iStoaXP-Core-Match'!

!MatchAbstract commentStamp: 'HilaireFernandes 7/25/2014 20:17' prior: 0!
An association game: it comes with a basket populated with draggable items and targets.
The user drag the items to the matching targets.
The subclass represent different arangments of the layout and validity controller throught references to collections of right answers or throught block of code to check user answers. In the later case, several right answers are possible.!

!MatchAbstract methodsFor: 'accessing' stamp: 'HilaireFernandes 7/26/2014 13:39'!
basket
	^ basket ifNil: [ basket := self basketInstance ]! !

!MatchAbstract methodsFor: 'accessing' stamp: 'HilaireFernandes 7/26/2014 13:37'!
basketInstance
	self subclassResponsibility! !

!MatchAbstract methodsFor: 'initialize' stamp: 'HilaireFernandes 8/5/2014 13:19'!
initialize 
	super initialize.
	self color: Color transparent;
		layoutPolicy: TableLayout new;
		listDirection: #topToBottom;
		hResizing: #spaceFill;
		vResizing: #spaceFill;
		clipSubmorphs: true;
		cellPositioning: #center;
		layoutInset: 10;
		cellInset: 10.
	self addMorph: self basket.
	targets := OrderedCollection new.
	rightAnswers := OrderedCollection new.! !

!MatchAbstract methodsFor: 'initialize' stamp: 'HilaireFernandes 1/12/2014 11:40'!
installBasket: itemsCollection
	self subclassResponsibility! !

!MatchAbstract methodsFor: 'initialize' stamp: 'HilaireFernandes 7/25/2014 20:06'!
installTargets
	self subclassResponsibility! !

!MatchAbstract methodsFor: 'layout-properties' stamp: 'HilaireFernandes 7/25/2014 22:43'!
listDirection: symbol
	super listDirection: symbol.
	basket ifNil: [ ^self ].
	(symbol == #topToBottom or: [ symbol == #bottomToTop ]) 
		ifTrue: [ basket horizontalOriented ] 
		ifFalse: [ basket verticalOriented  ]! !

!MatchAbstract methodsFor: 'accessing' stamp: 'HilaireFernandes 1/11/2014 20:59'!
numberOfQuestions
	self subclassResponsibility! !

!MatchAbstract methodsFor: 'accessing' stamp: 'HilaireFernandes 7/28/2014 00:22'!
numberOfRightAnswers
	^ self userRightAnswers size! !

!MatchAbstract methodsFor: 'helpers' stamp: 'HilaireFernandes 1/11/2014 21:02'!
resetTargets
"Put all the MatchItems in the targets back to the basket"
	targets do: [ :aTarget |
		aTarget items do: [ :aMatchItem |
			basket addMorphBack: aMatchItem.
			aTarget lostItem: aMatchItem] ]! !

!MatchAbstract methodsFor: 'accessing' stamp: 'HilaireFernandes 8/5/2014 13:19'!
rightAnswers
	^ rightAnswers ! !

!MatchAbstract methodsFor: 'accessing' stamp: 'HilaireFernandes 8/5/2014 13:19'!
rightAnswers: aCollection
	rightAnswers := aCollection ! !

!MatchAbstract methodsFor: 'helpers' stamp: 'HilaireFernandes 1/11/2014 20:59'!
showErrors
	self subclassResponsibility! !

!MatchAbstract methodsFor: 'helpers' stamp: 'HilaireFernandes 1/11/2014 20:59'!
showSolution
	self subclassResponsibility! !

!MatchAbstract methodsFor: 'accessing' stamp: 'HilaireFernandes 1/11/2014 21:02'!
targets
	^ targets ! !

!MatchAbstract methodsFor: 'accessing' stamp: 'HilaireFernandes 7/25/2014 20:09'!
targets: aCollection
	targets := aCollection! !

!MatchAbstract methodsFor: 'tests' stamp: 'HilaireFernandes 1/11/2014 20:59'!
testSuccessAnswer
	self subclassResponsibility! !

!MatchAbstract methodsFor: 'accessing' stamp: 'HilaireFernandes 1/11/2014 21:03'!
userAnswers
	^ targets collect: [ :target | target items]! !

MatchAbstract subclass: #MatchCompositeAbstract
	instanceVariableNames: 'composite'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'iStoaXP-Core-Match'!

!MatchCompositeAbstract commentStamp: 'HilaireFernandes 11/3/2014 10:20' prior: 0!
Abstract class for a Match - association - game organised in an arbitrary maner. Targets are at arbitrary positions in the composite, positions defined outside of the class

The view is organized freely: 
 - at the top/right basket with draggable items 
 - a collection of morphs freely composed with at least targets.!

!MatchCompositeAbstract class methodsFor: 'helpers' stamp: 'HilaireFernandes 7/26/2014 23:21'!
targetPlug
	^ self targetPlug: 1! !

!MatchCompositeAbstract class methodsFor: 'helpers' stamp: 'HilaireFernandes 11/3/2014 21:19'!
targetPlug: integer
	"a target plug in a composite, it is identified by a number, so the real target is plug into at a later time. "
	| targetPlug |
	targetPlug := Morph newBounds: (0@0 extent: (Stoa scale: 250@250)) color: Color transparent.
	targetPlug setProperty: #target toValue: integer;
		clipSubmorphs: true;
		layoutPolicy: TableLayout new;
		vResizing: #shrinkWrap; 
		hResizing: #shrinkWrap.
	^ targetPlug! !

!MatchCompositeAbstract methodsFor: 'acccessing' stamp: 'HilaireFernandes 7/26/2014 13:25'!
composite: aMorph
	composite := aMorph.
! !

!MatchCompositeAbstract methodsFor: 'initialize' stamp: 'HilaireFernandes 7/26/2014 13:25'!
installComposite
	self addMorphBack: composite! !

!MatchCompositeAbstract methodsFor: 'initialize' stamp: 'HilaireFernandes 11/3/2014 21:50'!
installTargets
"Need to plug the target morphs in the appropriate positions in the composite morph"
	| targetPlugs |
	targetPlugs := composite allMorphs select: [ :each | each hasProperty: #target].
	targetPlugs do: [ :each | | index aTarget |
		index := each valueOfProperty: #target.
		aTarget := self styleForTarget: (targets at: index).
		each addMorph: aTarget]! !

!MatchCompositeAbstract methodsFor: 'initialize' stamp: 'HilaireFernandes 11/3/2014 22:03'!
styleForTarget: aTarget
"Apply the default style for a composite target"
	aTarget vResizing: #shrinkWrap; hResizing: #shrinkWrap. "when target is in a compositte, we want it to shrink"
	aTarget color: (Color white alpha: 0.5);
		cellInset: 0;
		colorWith: (Color white alpha: 0.5).
	^ aTarget! !

MatchCompositeAbstract subclass: #MatchCompositeBlockMorph
	uses: TMatchByBlock
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'iStoaXP-Core-Match'!

!MatchCompositeBlockMorph commentStamp: 'HilaireFernandes 8/5/2014 14:42' prior: 0!
A MatchCompositeBlockMorph is an association game with target positionned in a composite morph and showErrors, showSolution, testSuccess done with blocks of code.
!

MatchCompositeBlockMorph class
	uses: TMatchByBlock classTrait
	instanceVariableNames: ''!

!MatchCompositeBlockMorph class methodsFor: 'instance creation' stamp: 'HilaireFernandes 8/5/2014 14:33'!
composite: composite basket: itemsCollection targets: targetsCollection answers: answersCollection
	^ self new 
		composite: composite;
		basket: itemsCollection targets: targetsCollection answers: answersCollection;
		installComposite! !

!MatchCompositeBlockMorph methodsFor: 'initialize' stamp: ''!
basketInstance
	^ MatchBasketBlockMorph new.! !

!MatchCompositeBlockMorph methodsFor: 'initialize' stamp: ''!
installBasket: itemsCollection
	self basket 
		addAllMorphs: (itemsCollection shuffled collect: [:item | item asMatchItemBlock]);
		wrapDirection: #none; 
		wrapDirection: #topToBottom.
! !

!MatchCompositeBlockMorph methodsFor: 'accessing' stamp: ''!
numberOfQuestions
"Each target is a question, we are not more accurate now as guess per target sub question may not be easy"
	^ self targets size! !

!MatchCompositeBlockMorph methodsFor: 'helpers' stamp: ''!
showErrors
	self targets do: [ :aTarget | aTarget markAsRightOrWrong  ].! !

!MatchCompositeBlockMorph methodsFor: 'helpers' stamp: ''!
showSolution
	self resetTargets.
	self targets withIndexDo: [ :target :index | 
		(self rightAnswers at: index) do: [ :anAnswerType | | validItem |
			validItem := self basket giveMe: anAnswerType.
			target gotItem: validItem]].
	self showErrors! !

!MatchCompositeBlockMorph methodsFor: 'tests' stamp: ''!
testSuccessAnswer
	^ self targets allSatisfy:  [:aTarget | aTarget isRight]! !

!MatchCompositeBlockMorph methodsFor: 'accessing' stamp: ''!
userRightAnswers 
"Collect the targets with the correct answers"
	^ self targets select: [ :aTarget | aTarget isRight ]! !

MatchCompositeAbstract subclass: #MatchCompositeMorph
	uses: TMatchByCollection
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'iStoaXP-Core-Match'!

!MatchCompositeMorph commentStamp: 'HilaireFernandes 8/6/2014 00:29' prior: 0!
I am an association game between targets and draggable-items. In such game there is only one right answers.

rightAnswers is a collection of rigth answers for each target. It may contain more collections than targets size. The last superfluous collection contains items to remain in the basket.
!

MatchCompositeMorph class
	uses: TMatchByCollection classTrait
	instanceVariableNames: ''!

!MatchCompositeMorph class methodsFor: 'instance creation' stamp: 'HilaireFernandes 7/26/2014 13:24'!
composite: composite targets: targetsCollection answers: answersCollection
	^ self new 
		composite: composite;
		targets: targetsCollection answers: answersCollection;
		installComposite! !

!MatchCompositeMorph class methodsFor: 'as yet unclassified' stamp: 'HilaireFernandes 11/3/2014 21:28'!
example1
"
	MatchCompositeMorph example1 openInWorld
"
	| composite morph |
	composite := Morph newBounds: (0@0 extent: 300@300).
	composite addMorph: (self targetPlug: 1);
		addMorph: ((self targetPlug: 2) position:  200@50);
		addMorph: ((self targetPlug: 3) position: 50@200).
	morph := self 
		composite: composite 
		targets:  #('4 x 3' '' '3 x 3' )
		answers: #(
				#('12') 
				#('8') 
				#('9') 
				#('xxx')
			).
	morph extent: 400@400.
	^ morph! !

!MatchCompositeMorph methodsFor: 'initialize' stamp: ''!
basketInstance
	^ MatchBasketMorph new.! !

!MatchCompositeMorph methodsFor: 'initialize' stamp: ''!
installBasket: itemsCollection
	|allAnswers|
	allAnswers := OrderedCollection new.
	itemsCollection do: [:targetAnswers| allAnswers addAll: targetAnswers].
	self basket 
		addAllMorphs: allAnswers shuffled;
		wrapDirection: #none; 
		wrapDirection: #topToBottom
! !

!MatchCompositeMorph methodsFor: 'accessing' stamp: ''!
numberOfQuestions
	^ self rightAnswers inject: 0 into: [:sum :answers | sum + answers size]! !

!MatchCompositeMorph methodsFor: 'helpers' stamp: ''!
showErrors
	"Clear out all, including the answer without matching target"
	self rightAnswers do: [:someAnswers |
		someAnswers do: [:aMatchItem | aMatchItem markAsRight]].
	"Mark the items wrong or right "
	self targets do: [:aTarget | 
		aTarget items do: [:aMatchItem | aMatchItem markAsRightOrWrong: aTarget]].
	"Mark wrong the item in the basket that should be associated to a target"
	self basket submorphsDo: [ :aMatchItem |
		aMatchItem targets ifNotEmpty: [ aMatchItem markAsWrong]]! !

!MatchCompositeMorph methodsFor: 'helpers' stamp: ''!
showSolution
	self resetTargets.
	self basket submorphsDo: [ :aMatchItem |
		aMatchItem targets do: [:aTarget |	aTarget gotItem: aMatchItem]].
	self showErrors "to clear out items marked wrong"! !

!MatchCompositeMorph methodsFor: 'initialize' stamp: ''!
targets: targetsCollection answers: answersCollection
	answersCollection doWithIndex: [:someAnswers :answerIndex | 
		| theTarget targetAnswers | 
		"Instanciate the target only for valid answers (remember we may have invalid answers)"
		theTarget := answerIndex <= targetsCollection size 
			ifTrue: [self targets add: (targetsCollection at: answerIndex) asMatchTarget].
		self rightAnswers add: (targetAnswers := OrderedCollection new).
		"Associate theTarget to these answers, associate nil (theTarget value) if none"
		someAnswers do: [:anItem | 
			(targetAnswers add: anItem asMatchItem) addTarget: theTarget]].
	self installBasket: self rightAnswers.
	self installTargets! !

!MatchCompositeMorph methodsFor: 'tests' stamp: ''!
testSuccessAnswer
	self userAnswers doWithIndex:  [:userAnswer :index | 
		userAnswer asBag = (self rightAnswers at: index) asBag ifFalse: [^ false]].
	^ true! !

!MatchCompositeMorph methodsFor: 'accessing' stamp: ''!
userRightAnswers
"Collect the items correctly matched with the targets or the basket"
	| rightAnswers |
	rightAnswers := OrderedCollection new.
	self targets do: [:aTarget |
		aTarget items do: [:item | (item isMatching: aTarget) ifTrue: [rightAnswers add: item]]].
	self basket submorphsDo: [:item| item targets ifEmpty: [ rightAnswers add: item]].
	^ rightAnswers! !

MatchAbstract subclass: #MatchRowAbstract
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'iStoaXP-Core-Match'!

!MatchRowAbstract commentStamp: 'HilaireFernandes 8/6/2014 00:29' prior: 0!
Abstract class for a Match - association - game organised in row.

The view is organized in row, from the top to the bottom: 
 - basket with draggable items 
 - a collection of rows, each one composed with: 
	- at the left an illustration, 
	- at the right a target to drop the items

- basket: where are initialily installed the items (MathBasketXXXMorph),
- targets: a collection of target where to drop the items (MatchTargetXXXMorph))
- rightAnswers: a collection of collection of valid model for each target.
!

!MatchRowAbstract methodsFor: 'initialize' stamp: 'HilaireFernandes 7/25/2014 23:26'!
installTargets
"Add target morphs in row"
	targets do: [ :aTarget | self addMorphBack: aTarget ]! !

MatchRowAbstract subclass: #MatchRowBlockMorph
	uses: TMatchByBlock
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'iStoaXP-Core-Match'!

!MatchRowBlockMorph commentStamp: 'HilaireFernandes 8/6/2014 00:29' prior: 0!
I am an association game between targets and draggable-items.
Validities of the user answers are check through one block of code per target.
In this game there are several right answers, the model provides one right answers example (validAnswers instance variable).

- rightAnswers: it provides *a* possible valid answer. We may have multiple valid answers with this block match game, so we need one. Only block of code are used to dermine if user proposed answer is valid though.
It is a collection of symbol collections (#type in MatchItemBlockMorph). 
One collection per target.!

MatchRowBlockMorph class
	uses: TMatchByBlock classTrait
	instanceVariableNames: ''!

!MatchRowBlockMorph class methodsFor: 'instance creation' stamp: 'HilaireFernandes 1/12/2014 11:25'!
basket: itemsCollection targets: targetsCollection answers: answersCollection
	^ self new basket: itemsCollection  targets: targetsCollection answers: answersCollection! !

!MatchRowBlockMorph class methodsFor: 'instance creation' stamp: 'HilaireFernandes 7/23/2014 07:20'!
basket: itemsCollection targets: targetsCollection answers: answersCollection listDirection: symbol
	^ (self new basket: itemsCollection  targets: targetsCollection answers: answersCollection)
		listDirection: symbol;
		yourself.! !

!MatchRowBlockMorph class methodsFor: 'examples' stamp: 'HilaireFernandes 7/25/2014 19:35'!
example1
"
	MatchRowBlockMorph example1 openInWorld
"
	^ self 
		basket: { 
			'Apple' asMatchItemBlock type: #apple.  
			'Apple' asMatchItemBlock type: #apple. 
			'Apple' asMatchItemBlock type: #apple.
			'Orange' asMatchItemBlock type: #orange. 
			'Banana' asMatchItemBlock type: #banana.}
		targets: #( 
			#('I want #apple > 1' '[(self count: #apple) > 1]')
			#('I want 1 orange' '[(self count: #orange) = 1]'))
		answers:  #(
			#(#apple #apple)
			#(#orange))! !

!MatchRowBlockMorph class methodsFor: 'examples' stamp: 'HilaireFernandes 7/25/2014 19:35'!
example2
"
	(MatchRowBlockMorph example2 extent: 400@300) openInWorld; layoutChanged
"
	^ self 
		basket: { 
			Stoa apple asMatchItemBlock.
			Stoa apple asMatchItemBlock.
			Stoa apple asMatchItemBlock.
			Stoa orange asMatchItemBlock.
			Stoa banana asMatchItemBlock}
		targets: #( 
			#('I want #apple > 1' '[(self count: #apple) > 1]')
			#('I want 1 orange' '[(self count: #orange) = 1]'))
		answers:  #(
			#(#apple #apple)
			#(#orange))! !

!MatchRowBlockMorph methodsFor: 'initialize' stamp: ''!
basket: itemsCollection targets: targetsCollection answers: answersCollection
	"Instanciate the targets"
	self targets: (targetsCollection collect: [:targetAttribute |
		targetAttribute first asMatchTargetBlock validityBlock: targetAttribute second]).
	self rightAnswers: answersCollection.
	self installBasket: itemsCollection.
	self installTargets.! !

!MatchRowBlockMorph methodsFor: 'initialize' stamp: ''!
basketInstance
	^ MatchBasketBlockMorph new.! !

!MatchRowBlockMorph methodsFor: 'initialize' stamp: ''!
installBasket: itemsCollection
	self basket 
		addAllMorphs: (itemsCollection shuffled collect: [:item | item asMatchItemBlock]);
		wrapDirection: #none; 
		wrapDirection: #topToBottom.
! !

!MatchRowBlockMorph methodsFor: 'accessing' stamp: ''!
numberOfQuestions
"Each target is a question, we are not more accurate now as guess per target sub question may not be easy"
	^ self targets size! !

!MatchRowBlockMorph methodsFor: 'helpers' stamp: ''!
showErrors
	self targets do: [ :aTarget | aTarget markAsRightOrWrong  ].! !

!MatchRowBlockMorph methodsFor: 'helpers' stamp: ''!
showSolution
	self resetTargets.
	self targets withIndexDo: [ :target :index | 
		(self rightAnswers at: index) do: [ :anAnswerType | | validItem |
			validItem := self basket giveMe: anAnswerType.
			target gotItem: validItem]].
	self showErrors! !

!MatchRowBlockMorph methodsFor: 'tests' stamp: ''!
testSuccessAnswer
	^ self targets allSatisfy:  [:aTarget | aTarget isRight]! !

!MatchRowBlockMorph methodsFor: 'accessing' stamp: ''!
userRightAnswers 
"Collect the targets with the correct answers"
	^ self targets select: [ :aTarget | aTarget isRight ]! !

MatchRowAbstract subclass: #MatchRowMorph
	uses: TMatchByCollection
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'iStoaXP-Core-Match'!

!MatchRowMorph commentStamp: 'HilaireFernandes 8/6/2014 00:29' prior: 0!
I am an association game between targets and draggable-items. In such game there is only one right answers.

rightAnswers is a collection of rigth answers for each target. It may contain more collections than targets size. The last superfluous collection contains items to remain in the basket.!

MatchRowMorph class
	uses: TMatchByCollection classTrait
	instanceVariableNames: ''!

!MatchRowMorph class methodsFor: 'examples' stamp: 'HilaireFernandes 7/25/2014 19:34'!
example1
	"
	MatchRowMorph example1 openInWorld
	"
	^ self 
		targets: #('4 x 3' '2 x 4' '3 x 3' )
		answers: #(
				#('4 + 4 + 4' '12' '3 + 3 + 3 + 3' )
				#('8' '2 + 2 + 2 + 2' '4 + 4' )
				#('9' '3 + 3 + 3' )
				#('xxxx' 'yyyy' )
			)! !

!MatchRowMorph class methodsFor: 'examples' stamp: 'HilaireFernandes 7/25/2014 19:34'!
example2
	"
	MatchRowMorph example2 openInWorld
	"
	^ self 
		targets: {ThemeIcons volumeIcon asMorph . ThemeIcons warningIcon asMorph . ThemeIcons calendarIcon asMorph }
		answers: {  
				#('4 + 4 + 4' '12' '3 + 3 + 3 + 3' ) .
				#('8' '2 + 2 + 2 + 2' '4 + 4' ) .
				#('9' '3 + 3 + 3' ) .
				{ThemeIcons errorIcon asMorph . 'yyyy' } .}! !

!MatchRowMorph class methodsFor: 'instance creation' stamp: 'HilaireFernandes 1/8/2014 20:51'!
targets: targetsCollection answers: answersCollection
	^ self new targets: targetsCollection answers: answersCollection! !

!MatchRowMorph methodsFor: 'initialize' stamp: ''!
basketInstance
	^ MatchBasketMorph new.! !

!MatchRowMorph methodsFor: 'initialize' stamp: ''!
installBasket: itemsCollection
	|allAnswers|
	allAnswers := OrderedCollection new.
	itemsCollection do: [:targetAnswers| allAnswers addAll: targetAnswers].
	self basket 
		addAllMorphs: allAnswers shuffled;
		wrapDirection: #none; 
		wrapDirection: #topToBottom
! !

!MatchRowMorph methodsFor: 'accessing' stamp: ''!
numberOfQuestions
	^ self rightAnswers inject: 0 into: [:sum :answers | sum + answers size]! !

!MatchRowMorph methodsFor: 'helpers' stamp: ''!
showErrors
	"Clear out all, including the answer without matching target"
	self rightAnswers do: [:someAnswers |
		someAnswers do: [:aMatchItem | aMatchItem markAsRight]].
	"Mark the items wrong or right "
	self targets do: [:aTarget | 
		aTarget items do: [:aMatchItem | aMatchItem markAsRightOrWrong: aTarget]].
	"Mark wrong the item in the basket that should be associated to a target"
	self basket submorphsDo: [ :aMatchItem |
		aMatchItem targets ifNotEmpty: [ aMatchItem markAsWrong]]! !

!MatchRowMorph methodsFor: 'helpers' stamp: ''!
showSolution
	self resetTargets.
	self basket submorphsDo: [ :aMatchItem |
		aMatchItem targets do: [:aTarget |	aTarget gotItem: aMatchItem]].
	self showErrors "to clear out items marked wrong"! !

!MatchRowMorph methodsFor: 'initialize' stamp: ''!
targets: targetsCollection answers: answersCollection
	answersCollection doWithIndex: [:someAnswers :answerIndex | 
		| theTarget targetAnswers | 
		"Instanciate the target only for valid answers (remember we may have invalid answers)"
		theTarget := answerIndex <= targetsCollection size 
			ifTrue: [self targets add: (targetsCollection at: answerIndex) asMatchTarget].
		self rightAnswers add: (targetAnswers := OrderedCollection new).
		"Associate theTarget to these answers, associate nil (theTarget value) if none"
		someAnswers do: [:anItem | 
			(targetAnswers add: anItem asMatchItem) addTarget: theTarget]].
	self installBasket: self rightAnswers.
	self installTargets! !

!MatchRowMorph methodsFor: 'tests' stamp: ''!
testSuccessAnswer
	self userAnswers doWithIndex:  [:userAnswer :index | 
		userAnswer asBag = (self rightAnswers at: index) asBag ifFalse: [^ false]].
	^ true! !

!MatchRowMorph methodsFor: 'accessing' stamp: ''!
userRightAnswers
"Collect the items correctly matched with the targets or the basket"
	| rightAnswers |
	rightAnswers := OrderedCollection new.
	self targets do: [:aTarget |
		aTarget items do: [:item | (item isMatching: aTarget) ifTrue: [rightAnswers add: item]]].
	self basket submorphsDo: [:item| item targets ifEmpty: [ rightAnswers add: item]].
	^ rightAnswers! !

Morph subclass: #MatchBasketAbstract
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'iStoaXP-Core-Match'!

!MatchBasketAbstract commentStamp: '<historical>' prior: 0!
Abstract class to define the Basket in association game.!

!MatchBasketAbstract methodsFor: 'layout-properties' stamp: 'HilaireFernandes 7/23/2014 07:40'!
horizontalOriented
"the basket take maximum space horizontally"
	self	 listDirection: #leftToRight;
		hResizing: #spaceFill;
		vResizing: #shrinkWrap;
		wrapDirection: #topToBottom.! !

!MatchBasketAbstract methodsFor: 'initialize' stamp: 'HilaireFernandes 1/10/2014 21:36'!
initialize
	super initialize.
	self color: Color white;
		 borderWidth: 3;
		 setBorderStyle: #raised.
	self layoutPolicy: TableLayout new;
		 listDirection: #leftToRight;
		 hResizing: #spaceFill;
		 vResizing: #shrinkWrap;
		 layoutInset: 5;
		 cellInset: 5;
		wrapDirection: #topToBottom.! !

!MatchBasketAbstract methodsFor: 'accessing' stamp: 'HilaireFernandes 1/10/2014 21:37'!
matchItemClass
	self subclassResponsibility ! !

!MatchBasketAbstract methodsFor: 'layout-properties' stamp: 'HilaireFernandes 7/23/2014 07:41'!
verticalOriented
"the basket take maximum space vertically"
	self	 listDirection: #topToBottom;
		hResizing: #shrinkWrap;
		vResizing: #spaceFill;
		wrapDirection: #leftToRight.! !

!MatchBasketAbstract methodsFor: 'dropping/grabbing' stamp: 'HilaireFernandes 1/10/2014 21:37'!
wantsDroppedMorph: aMorph event: evt 
	^ aMorph class = self matchItemClass ! !

MatchBasketAbstract subclass: #MatchBasketBlockMorph
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'iStoaXP-Core-Match'!

!MatchBasketBlockMorph commentStamp: '<historical>' prior: 0!
I am a container for MatchItemBlockMorph.
!

!MatchBasketBlockMorph methodsFor: 'accessing' stamp: 'HilaireFernandes 1/12/2014 13:32'!
giveMe: itemType
	^ self submorphs 
		detect: [:item | item type = itemType]
		ifNone: [self error: 'There is no more items with type: ', itemType]! !

!MatchBasketBlockMorph methodsFor: 'accessing' stamp: 'HilaireFernandes 1/11/2014 16:56'!
matchItemClass
	^ MatchItemBlockMorph ! !

MatchBasketAbstract subclass: #MatchBasketMorph
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'iStoaXP-Core-Match'!

!MatchBasketMorph commentStamp: '<historical>' prior: 0!
I am a container for MatchItemMoprh the user can drag out and in.!

!MatchBasketMorph methodsFor: 'accessing' stamp: 'HilaireFernandes 1/10/2014 21:37'!
matchItemClass
	^ MatchItemMorph ! !

Morph subclass: #MatchItemAbstract
	instanceVariableNames: 'model'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'iStoaXP-Core-Match'!

!MatchItemAbstract commentStamp: '<historical>' prior: 0!
Abstract class to define the Item in association game.

- model: the object I represent!

!MatchItemAbstract class methodsFor: 'instance creation' stamp: 'HilaireFernandes 1/12/2014 12:56'!
from: anObject
	^ self new model: anObject! !

!MatchItemAbstract methodsFor: 'updating' stamp: 'HilaireFernandes 1/10/2014 21:44'!
colorWith: aColor
	self color: aColor ; borderColor: aColor darker! !

!MatchItemAbstract methodsFor: 'event handling' stamp: 'HilaireFernandes 1/10/2014 21:48'!
handlesMouseDown: anEvent 
	^ true! !

!MatchItemAbstract methodsFor: 'initialize' stamp: 'HilaireFernandes 1/18/2014 10:17'!
initialize
	super initialize.
	self markAsRight; borderWidth: 2; borderColor: Stoa rightColor darker; useRoundedCorners.
	self layoutPolicy: TableLayout new;
		 vResizing: #shrinkWrap;
		 hResizing: #shrinkWrap;
		 layoutInset: 5 @ 5.! !

!MatchItemAbstract methodsFor: 'testing' stamp: 'HilaireFernandes 1/10/2014 21:40'!
isMatching: aTarget
	self subclassResponsibility! !

!MatchItemAbstract methodsFor: 'dropping/grabbing' stamp: 'HilaireFernandes 1/10/2014 21:55'!
justDroppedInto: aMorph event: anEvent 
	aMorph changed.
	aMorph class = self matchTargetClass ifTrue: [aMorph gotItem: self]! !

!MatchItemAbstract methodsFor: 'dropping/grabbing' stamp: 'HilaireFernandes 1/10/2014 21:55'!
justGrabbedFrom: formerOwner 
	formerOwner changed.
	formerOwner class = self matchTargetClass 
		ifTrue: [formerOwner lostItem: self]! !

!MatchItemAbstract methodsFor: 'updating' stamp: 'HilaireFernandes 1/10/2014 21:44'!
markAsRight
	self colorWith: Stoa rightColor! !

!MatchItemAbstract methodsFor: 'updating' stamp: 'HilaireFernandes 1/10/2014 22:19'!
markAsRightOrWrong: aTarget 
	(self isMatching: aTarget)
		ifTrue: [self markAsRight]
		ifFalse: [self markAsWrong]! !

!MatchItemAbstract methodsFor: 'updating' stamp: 'HilaireFernandes 1/10/2014 21:44'!
markAsWrong
	self colorWith: Stoa wrongColor! !

!MatchItemAbstract methodsFor: 'accessing' stamp: 'HilaireFernandes 1/10/2014 21:54'!
matchBasketClass
	self subclassResponsibility ! !

!MatchItemAbstract methodsFor: 'accessing' stamp: 'HilaireFernandes 1/10/2014 21:53'!
matchTargetClass
	self subclassResponsibility ! !

!MatchItemAbstract methodsFor: 'accessing' stamp: 'HilaireFernandes 1/10/2014 22:17'!
model
	^ model! !

!MatchItemAbstract methodsFor: 'accessing' stamp: 'HilaireFernandes 1/15/2014 16:41'!
model: anObject
	model isString
		ifTrue: [ self addMorph: (model := anObject asStringMorph) ]
		ifFalse: [ self addMorph: (model := anObject asMorph)]! !

!MatchItemAbstract methodsFor: 'event handling' stamp: 'HilaireFernandes 1/10/2014 21:48'!
mouseDown: evt 
	evt hand grabMorph: self! !

!MatchItemAbstract methodsFor: 'dropping/grabbing' stamp: 'HilaireFernandes 1/10/2014 21:54'!
wantsToBeDroppedInto: aMorph 
	^ aMorph class = self matchTargetClass or: [aMorph class = self matchBasketClass]! !

MatchItemAbstract subclass: #MatchItemBlockMorph
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'iStoaXP-Core-Match'!

!MatchItemBlockMorph commentStamp: '<historical>' prior: 0!
I am an item the user drags out/in a MatchBasketBlockMorph or a MatchTargetBlockMorph.

- type: a symbol to indicate my nature (for example #apple)!

!MatchItemBlockMorph methodsFor: 'as yet unclassified' stamp: 'HilaireFernandes 1/15/2014 16:53'!
asMatchItemBlock
	^ self! !

!MatchItemBlockMorph methodsFor: 'accessing' stamp: 'HilaireFernandes 1/11/2014 17:08'!
isMatching: aTarget
	self shouldNotImplement ! !

!MatchItemBlockMorph methodsFor: 'accessing' stamp: 'HilaireFernandes 1/11/2014 17:03'!
matchBasketClass
	^ MatchBasketBlockMorph ! !

!MatchItemBlockMorph methodsFor: 'accessing' stamp: 'HilaireFernandes 1/11/2014 17:03'!
matchTargetClass
	^ MatchTargetBlockMorph ! !

!MatchItemBlockMorph methodsFor: 'accessing' stamp: 'HilaireFernandes 1/15/2014 16:36'!
type
"We may want to attach the type to the underneath morph (See Stoa glass for example)"
	^model valueOfProperty: #itemType ifAbsentPut: #none! !

!MatchItemBlockMorph methodsFor: 'accessing' stamp: 'HilaireFernandes 1/15/2014 16:37'!
type: aSymbol
	model setProperty: #itemType toValue: aSymbol! !

MatchItemAbstract subclass: #MatchItemMorph
	instanceVariableNames: 'targets'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'iStoaXP-Core-Match'!

!MatchItemMorph commentStamp: '<historical>' prior: 0!
I am an item the user drags out/in a MatchBasketMorph or a MatchTargetMorph.
- targets: a collection of MatchTargetMorph rightly associated to myself to check my validity state
!

!MatchItemMorph class methodsFor: 'instance creation' stamp: 'HilaireFernandes 1/8/2014 11:12'!
from: anObject
	^ self new model: anObject! !

!MatchItemMorph methodsFor: 'comparing' stamp: 'HilaireFernandes 1/12/2014 12:58'!
= aMatchItemMorph
	^ aMatchItemMorph 
		ifNotNil: [model = aMatchItemMorph model]
		ifNil: [ false ]! !

!MatchItemMorph methodsFor: 'adding' stamp: 'HilaireFernandes 1/7/2014 21:54'!
addTarget: aTarget 
	"set my target"
	aTarget ifNotNil: [targets add: aTarget].
	^ aTarget! !

!MatchItemMorph methodsFor: 'comparing' stamp: 'HilaireFernandes 1/12/2014 12:58'!
hash
	^ model hash! !

!MatchItemMorph methodsFor: 'initialize' stamp: 'HilaireFernandes 1/10/2014 21:57'!
initialize
	super initialize.
	targets := OrderedCollection new.! !

!MatchItemMorph methodsFor: 'testing' stamp: 'HilaireFernandes 1/10/2014 15:03'!
isMatching: aTarget 
	^ targets includes: aTarget
! !

!MatchItemMorph methodsFor: 'accessing' stamp: 'HilaireFernandes 1/10/2014 21:55'!
matchBasketClass
	^ MatchBasketMorph ! !

!MatchItemMorph methodsFor: 'accessing' stamp: 'HilaireFernandes 1/10/2014 21:55'!
matchTargetClass
	^ MatchTargetMorph ! !

!MatchItemMorph methodsFor: 'accessing' stamp: 'HilaireFernandes 1/7/2014 21:59'!
targets
	^ targets! !

Morph subclass: #MatchTargetAbstract
	instanceVariableNames: 'model targetBasket targetHead'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'iStoaXP-Core-Match'!

!MatchTargetAbstract commentStamp: '<historical>' prior: 0!
Abstract class to define the target in association game.
I am a target where user can drag in MatchItem.

I come with an illustration to differenciate me from other targets.

- model : the model of the target, I build my targetHead from it.
- tagetBasket: the place for my dropped in/out morphs
- targetHead: model view 
!

!MatchTargetAbstract class methodsFor: 'instance creation' stamp: 'HilaireFernandes 1/12/2014 12:56'!
from: anObject
	^ self new model: anObject! !

!MatchTargetAbstract methodsFor: 'layout' stamp: 'HilaireFernandes 1/10/2014 22:03'!
acceptDroppingMorph: aMorph event: evt
	"This message is sent when a morph is dropped onto me and I agreed 
	Add it to my basket"
	| layout |
	layout := targetBasket layoutPolicy.
	targetBasket 
		privateAddMorph: aMorph 
		atIndex: (layout indexForInserting: aMorph at: evt position in: targetBasket).! !

!MatchTargetAbstract methodsFor: 'updating' stamp: 'HilaireFernandes 1/12/2014 12:21'!
colorWith: aColor
	targetBasket  
		color: aColor; 
		borderColor: aColor darker! !

!MatchTargetAbstract methodsFor: 'accessing' stamp: 'HilaireFernandes 7/23/2014 11:36'!
firstItem
	^ self items first! !

!MatchTargetAbstract methodsFor: 'updating' stamp: 'HilaireFernandes 1/10/2014 22:02'!
gotItem: matchItem
	targetBasket addMorphBack: matchItem ! !

!MatchTargetAbstract methodsFor: 'initialize' stamp: 'HilaireFernandes 1/10/2014 22:04'!
initialize
	super initialize.
	self color: Stoa subAreaColor;
		layoutPolicy: TableLayout new;
		listDirection: #leftToRight;
		layoutInset: 5 @ 5;
		cellInset: 15 @ 5;
		hResizing: #spaceFill;
		vResizing: #spaceFill.
	self addMorph: self targetBasket ! !

!MatchTargetAbstract methodsFor: 'accessing' stamp: 'HilaireFernandes 1/10/2014 22:02'!
items
	^ targetBasket submorphs! !

!MatchTargetAbstract methodsFor: 'updating' stamp: 'HilaireFernandes 1/10/2014 22:02'!
lostItem: matchItem
	targetBasket removeMorph: matchItem ! !

!MatchTargetAbstract methodsFor: 'updating' stamp: 'HilaireFernandes 1/12/2014 12:21'!
markAsRight
	self colorWith: Stoa subAreaColor! !

!MatchTargetAbstract methodsFor: 'updating' stamp: 'HilaireFernandes 1/12/2014 12:21'!
markAsWrong
	self colorWith: Stoa wrongColor! !

!MatchTargetAbstract methodsFor: 'accessing' stamp: 'HilaireFernandes 1/10/2014 22:04'!
matchItemClass
	self subclassResponsibility ! !

!MatchTargetAbstract methodsFor: 'accessing' stamp: 'HilaireFernandes 1/10/2014 22:19'!
model
	^ model! !

!MatchTargetAbstract methodsFor: 'accessing' stamp: 'HilaireFernandes 7/26/2014 13:48'!
model: anObject
	model := anObject.
	targetHead := Morph new color: Stoa targetColor;
		borderWidth: 1;
		layoutPolicy: TableLayout new;
		vResizing: #shrinkWrap;
		hResizing: #shrinkWrap;
		layoutInset: 5.
	model isString
		ifTrue: [
			model ifEmpty: [^ self]. "No header for empty string" 
			targetHead addMorph: (model asStringMorph)]
		ifFalse: [targetHead addMorph: model asMorph].
	self addMorph: targetHead! !

!MatchTargetAbstract methodsFor: 'accessing' stamp: 'HilaireFernandes 1/12/2014 13:00'!
targetBasket
	^ targetBasket ifNil: [ 
		targetBasket := BorderedMorph new borderWidth: 2;
 			layoutPolicy: TableLayout new;
			listDirection: #leftToRight;
		 	wrapCentering: #center;
			wrapDirection: #topToBottom;
			layoutInset: 5;
			cellInset: 10 @ 5;
			hResizing: #spaceFill;
			vResizing: #spaceFill.
		self markAsRight.
		targetBasket]! !

!MatchTargetAbstract methodsFor: 'dropping/grabbing' stamp: 'HilaireFernandes 1/10/2014 22:04'!
wantsDroppedMorph: aMorph event: evt 
	^ aMorph class = self matchItemClass! !

MatchTargetAbstract subclass: #MatchTargetBlockMorph
	instanceVariableNames: 'validityBlock'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'iStoaXP-Core-Match'!

!MatchTargetBlockMorph commentStamp: '<historical>' prior: 0!
I am a target where user can drag in MatchItemBlock.

- validityBlock: validity block to validate or not the user answers. It takes no argument and it answers a boolean.

For example, a block like
	[ (self count: #apple) > 3  ]
will validate any answer containing more than 3 target item of type apple.!

!MatchTargetBlockMorph methodsFor: 'validity helpers' stamp: 'HilaireFernandes 7/23/2014 11:28'!
count
	^ self items size! !

!MatchTargetBlockMorph methodsFor: 'validity helpers' stamp: 'HilaireFernandes 1/11/2014 17:18'!
count: aType
	^ self items count: [ :item | item type = aType ]! !

!MatchTargetBlockMorph methodsFor: 'validity helpers' stamp: 'HilaireFernandes 7/23/2014 12:50'!
equals: typeCollection
	^ (self items collect: [ :item | item type ]) sorted = typeCollection sorted! !

!MatchTargetBlockMorph methodsFor: 'testing' stamp: 'HilaireFernandes 1/12/2014 11:59'!
isRight
	^ validityBlock value! !

!MatchTargetBlockMorph methodsFor: 'updating' stamp: 'HilaireFernandes 1/12/2014 12:25'!
markAsRightOrWrong
	self isRight 
		ifTrue: [self markAsRight]
		ifFalse: [self markAsWrong]! !

!MatchTargetBlockMorph methodsFor: 'accessing' stamp: 'HilaireFernandes 1/11/2014 17:09'!
matchItemClass
	^ MatchItemBlockMorph ! !

!MatchTargetBlockMorph methodsFor: 'accessing' stamp: 'HilaireFernandes 1/11/2014 21:53'!
validityBlock: aString
	validityBlock := Compiler evaluate: aString for: self logged: false.! !

MatchTargetAbstract subclass: #MatchTargetMorph
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'iStoaXP-Core-Match'!

!MatchTargetMorph commentStamp: '<historical>' prior: 0!
I am a target where user can drag in MatchItem.
!

!MatchTargetMorph class methodsFor: 'instance creation' stamp: 'HilaireFernandes 1/8/2014 11:12'!
from: anObject
	^ self new model: anObject! !

!MatchTargetMorph methodsFor: 'comparing' stamp: 'HilaireFernandes 1/12/2014 12:59'!
= aTargetMorph
	^ aTargetMorph 
		ifNotNil: [model = aTargetMorph model]
		ifNil: [ false ]! !

!MatchTargetMorph methodsFor: 'comparing' stamp: 'HilaireFernandes 1/12/2014 12:59'!
hash
	^ model hash! !

!MatchTargetMorph methodsFor: 'accessing' stamp: 'HilaireFernandes 1/10/2014 22:04'!
matchItemClass
	^ MatchItemMorph ! !

!Morph methodsFor: '*iStoaXP-Core-Widget' stamp: 'HilaireFernandes 7/24/2014 16:29'!
asStateMorph
	^ ImageStateMorph new form: self imageForm! !

!Morph methodsFor: '*iStoaXP-Core-Widget' stamp: 'HilaireFernandes 10/26/2014 09:47'!
bottomCenter: aPoint
	self position: (aPoint x - (self extent x // 2)) @  (aPoint y - self extent y)! !

Morph subclass: #QuizMorph
	instanceVariableNames: 'proposals'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'iStoaXP-Core-Game'!

!QuizMorph commentStamp: '<historical>' prior: 0!
A QuizMorph is a quiz game. It comes with: 

- illustration: an optional morph to described visualy the quiz
- proposals: a collection of couples of morph and boolean!

!QuizMorph class methodsFor: 'documentation' stamp: 'HilaireFernandes 1/5/2014 15:14'!
example1
	" QuizMorph example1 openInWorld"
	^ self proposals: 
		(Array 
			with: {ThemeIcons volumeIcon asMorph. true} 
			with: {ThemeIcons warningIcon asMorph. false} 
			with: {ThemeIcons calendarIcon asMorph. true})! !

!QuizMorph class methodsFor: 'documentation' stamp: 'HilaireFernandes 1/5/2014 15:14'!
example2
	" QuizMorph example2 openInWorld"
	^ self 
		illustration: ThemeIcons errorIcon asMorph
		proposals: 
			(Array 
				with: {ThemeIcons volumeIcon asMorph. true} 
				with: {ThemeIcons warningIcon asMorph. false} 
				with: {ThemeIcons calendarIcon asMorph. true})! !

!QuizMorph class methodsFor: 'instance creation' stamp: 'HilaireFernandes 12/8/2013 11:10'!
illustration: aMorph proposals: aProposalCollection 
	"aProposalCollection est une collection de couple {aTextOrMorph . Boolean}"
	^ self new illustration: aMorph proposals: aProposalCollection! !

!QuizMorph class methodsFor: 'instance creation' stamp: 'HilaireFernandes 12/8/2013 11:10'!
proposals: aProposalCollection 
	^ self illustration: nil proposals: aProposalCollection! !

!QuizMorph methodsFor: 'buttons' stamp: 'HilaireFernandes 12/9/2013 21:06'!
clicked: event in: buttonMorph
	buttonMorph setProperty: #selected toValue: (buttonMorph valueOfProperty: #selected) not.
	self setVisualState: buttonMorph! !

!QuizMorph methodsFor: 'initialize' stamp: 'HilaireFernandes 1/8/2014 10:34'!
illustration: aMorph proposals: aCollection
	aMorph ifNotNil: [
		aMorph hasDropShadow: true; shadowOffset: 0@0; shadowColor: Stoa illustrationColor;
			borderWidth: 4; borderColor: Stoa illustrationColor.
		self select: aMorph.
		self addMorph: aMorph].
	proposals := aCollection shuffled.
	proposals do: [:aProposal |
		self addMorphBack: (self setButtonFor: aProposal first)]
	! !

!QuizMorph methodsFor: 'initialize' stamp: 'HilaireFernandes 5/17/2014 16:56'!
initialize 
	super initialize.
	self color: Color transparent;
		layoutPolicy: TableLayout new;
		 listDirection: #leftToRight;
		 hResizing: #spaceFill;
		 vResizing: #spaceFill;
		 clipSubmorphs: true;
		 cellPositioning: #center;
		 layoutInset: 5;
		 cellInset: 10.! !

!QuizMorph methodsFor: 'accessing' stamp: 'HilaireFernandes 12/13/2013 20:25'!
numberOfQuestions
	^ proposals size! !

!QuizMorph methodsFor: 'accessing' stamp: 'HilaireFernandes 7/27/2014 23:50'!
numberOfRightAnswers
	| userAnswers quantity|
	userAnswers := self userAnswers.
	quantity := 0.
	self rightAnswers withIndexDo: [:answer :index | 
		answer = (userAnswers at: index) ifTrue: [quantity := quantity + 1]].
	^ quantity! !

!QuizMorph methodsFor: 'accessing' stamp: 'HilaireFernandes 1/10/2014 14:12'!
proposals
	^ proposals! !

!QuizMorph methodsFor: 'accessing' stamp: 'HilaireFernandes 12/9/2013 21:44'!
rightAnswers
	^ proposals collect: [ :aProposal | aProposal second]! !

!QuizMorph methodsFor: 'buttons' stamp: 'HilaireFernandes 1/8/2014 10:37'!
select: buttonMorph
"Highligh this button as selected"
	buttonMorph borderColor: Stoa selectColor; color: Stoa selectColor! !

!QuizMorph methodsFor: 'buttons' stamp: 'HilaireFernandes 12/9/2013 21:06'!
setButtonFor: aMorph
	aMorph borderWidth: 4; color: Color transparent.
	aMorph on: #click send: #clicked:in: to: self.
	aMorph setProperty: #selected toValue: false.
	aMorph borderWidth: 4.
	self setVisualState: aMorph.
	^ aMorph! !

!QuizMorph methodsFor: 'buttons' stamp: 'HilaireFernandes 12/27/2013 17:08'!
setVisualState: buttonMorph
"Highligh this button as selected or unselected according to its selected property"
	(buttonMorph valueOfProperty: #selected)
		ifTrue: [self select: buttonMorph]
		ifFalse: [self unselect: buttonMorph]! !

!QuizMorph methodsFor: 'helpers' stamp: 'HilaireFernandes 1/8/2014 10:36'!
showErrors
	proposals do: [ :aProposal | | button |
		button := aProposal first.
		aProposal second = (button valueOfProperty: #selected)
			ifTrue: [button hasDropShadow: false]
			ifFalse: [ button hasDropShadow: true; shadowOffset: 0@0; shadowColor: Stoa wrongColor]].
	self changed! !

!QuizMorph methodsFor: 'helpers' stamp: 'HilaireFernandes 12/9/2013 21:55'!
showSolution
	proposals do: [ :aProposal |
		aProposal first setProperty: #selected toValue: aProposal second.
		self setVisualState: aProposal first].
	self showErrors ! !

!QuizMorph methodsFor: 'testing' stamp: 'HilaireFernandes 12/9/2013 21:45'!
testSuccessAnswer
	^ self rightAnswers = self userAnswers! !

!QuizMorph methodsFor: 'buttons' stamp: 'HilaireFernandes 12/27/2013 17:07'!
unselect: buttonMorph
"Highligh this button as unselected"
	buttonMorph borderColor: Color transparent; color: Color black! !

!QuizMorph methodsFor: 'accessing' stamp: 'HilaireFernandes 12/9/2013 21:44'!
userAnswers
	^ proposals collect: [ :aProposal | aProposal first valueOfProperty: #selected]! !

Morph subclass: #ScoreGainMorph
	instanceVariableNames: 'maxGain gain awardForm displayedAward gap shadowForm'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'iStoaXP-Core-Widget'!

!ScoreGainMorph commentStamp: '<historical>' prior: 0!
An animating Morph to show the gains in the score.!

!ScoreGainMorph class methodsFor: 'as yet unclassified' stamp: 'HilaireFernandes 12/26/2013 12:39'!
gain: aGain maxGain: aMaxGain
	^ self basicNew gain: aGain maxGain: aMaxGain; 
		initialize; 
		yourself! !

!ScoreGainMorph methodsFor: 'drawing' stamp: 'HilaireFernandes 1/31/2014 21:04'!
drawOn: aCanvas
	| position awardOffset|
	super drawOn: aCanvas.
	position := bounds topLeft + gap.
	awardOffset := gap + awardForm width @ 0.
	displayedAward timesRepeat: [ 
		aCanvas translucentImage: awardForm at: position.
		position := position + awardOffset].
	maxGain - displayedAward timesRepeat: [
		aCanvas translucentImage: shadowForm at: position.
		position := position + awardOffset]
	! !

!ScoreGainMorph methodsFor: 'accessing' stamp: 'HilaireFernandes 12/26/2013 12:40'!
gain: aGain maxGain: aMaxGain
	gain := aGain.
	maxGain := aMaxGain ! !

!ScoreGainMorph methodsFor: 'initialize' stamp: 'HilaireFernandes 1/31/2014 20:57'!
initialize
	super initialize.
	self color: (Color white alpha: 0.8).
	self useRoundedCorners. 
	displayedAward := -1.
	awardForm := Stoa getClipart: Path * 'awards' / 'bronzeMedal.png'.
	shadowForm := awardForm darker: 1.
	gap := Stoa scale: 10.
	self extent: (awardForm width + gap * maxGain + gap ) @ (2 * gap + awardForm height).! !

!ScoreGainMorph methodsFor: 'stepping and presenter' stamp: 'HilaireFernandes 1/31/2014 20:57'!
step
	displayedAward >= gain 
		ifTrue: [self stopStepping ] 
		ifFalse: [displayedAward := displayedAward + 1. self changed ]! !

!ScoreGainMorph methodsFor: 'testing' stamp: 'HilaireFernandes 12/26/2013 15:49'!
stepTime
	^ 500! !

Morph subclass: #ScoreLineMorph
	instanceVariableNames: 'awardForm rank radix awardQty shadowForm displayedAward gap'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'iStoaXP-Core-Widget'!

!ScoreLineMorph commentStamp: '<historical>' prior: 0!
I show the gain given the quantity of one specific award (crown, cup, medals).

- awardForm: a from representing the award
- rank: the rank of this award in the score composition, in [0 ; 5[
- radix: the base of our score system (max number of time we can repeat a given award)
- awardQty: the quantity of this award, in [0 ; radix[ 
		
!

!ScoreLineMorph class methodsFor: 'examples' stamp: 'HilaireFernandes 2/1/2014 11:26'!
example
"
	ScoreLineMorph example openInWorld
"

	^ self new awardQty: 3;
		rank: 1;
		yourself! !

!ScoreLineMorph methodsFor: 'accessing' stamp: 'HilaireFernandes 2/1/2014 14:34'!
awardQty: anObject
	awardQty := anObject.
	displayedAward := 0.! !

!ScoreLineMorph methodsFor: 'drawing' stamp: 'HilaireFernandes 4/27/2014 12:35'!
drawOn: aCanvas
	| position awardOffset myWidth scoreText|
	super drawOn: aCanvas.
	myWidth := self width.
	position := self topLeft + gap.
	awardOffset := gap + awardForm width @ 0.
	displayedAward timesRepeat: [ 
		aCanvas translucentImage: awardForm at: position.
		position := position + awardOffset].
	radix -1 - displayedAward timesRepeat: [
		aCanvas translucentImage: shadowForm at: position.
		position := position + awardOffset].
	
	position := self topLeft + (0 @ (awardForm height - Stoa largeTextFont height // 2)).
"	aCanvas 
		drawString: '=' 
		at: position + ((0.70 * myWidth) rounded @ 0)
		font: Stoa largeTextFont 
		color: Color black."
	
	scoreText := self score printString.
	aCanvas 
		drawString: scoreText 
		at: position + (myWidth - (Stoa largeTextFont widthOfString: scoreText)@0)
		font: Stoa largeTextFont 
		color: Color black
! !

!ScoreLineMorph methodsFor: 'initialize' stamp: 'HilaireFernandes 2/1/2014 16:44'!
initialize 
	super initialize.
	self color: Color transparent.
	displayedAward := -1.
	gap := Stoa scale: 10.
	radix := 5! !

!ScoreLineMorph methodsFor: 'stepping and presenter' stamp: 'HilaireFernandes 2/1/2014 17:10'!
isAnimationDone
	^ displayedAward >= awardQty 
! !

!ScoreLineMorph methodsFor: 'accessing' stamp: 'HilaireFernandes 1/30/2014 09:23'!
radix: anObject
	radix := anObject! !

!ScoreLineMorph methodsFor: 'accessing' stamp: 'HilaireFernandes 4/27/2014 14:35'!
rank: anObject
	| awardFile |
	rank := anObject.
	awardFile := #('bronzeMedal.png' 'silverMedal.png' 'goldMedal.png' 'goldCup.png' 'crown.png') at: (rank + 1).
	awardForm := Stoa getClipart: Path * 'awards' / awardFile.
	shadowForm := awardForm dimmed: 0.4.
	self minHeight: awardForm height.! !

!ScoreLineMorph methodsFor: 'accessing' stamp: 'HilaireFernandes 2/1/2014 14:44'!
score
	^  (radix raisedTo: rank) * displayedAward * 100! !

!ScoreLineMorph methodsFor: 'stepping and presenter' stamp: 'HilaireFernandes 2/1/2014 15:44'!
step
	displayedAward >= awardQty 
		ifTrue: [ self stopStepping ]
		ifFalse: [ displayedAward := displayedAward +1.
			self changed ]! !

!ScoreLineMorph methodsFor: 'stepping and presenter' stamp: 'HilaireFernandes 2/1/2014 17:15'!
stepTime 
	^ 500! !

Morph subclass: #ScoreMiniMorph
	instanceVariableNames: 'awardForms height gap'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'iStoaXP-Core-Widget'!

!ScoreMiniMorph commentStamp: '<historical>' prior: 0!
A MiniScoreMorph to display the current scrore in the game set.
!

!ScoreMiniMorph methodsFor: 'accessing' stamp: 'HilaireFernandes 2/1/2014 14:40'!
awards
"The quantity of different awards we have to built our score system:
crown, cup, gold, silver, bronze medals"
	^5! !

!ScoreMiniMorph methodsFor: 'drawing' stamp: 'HilaireFernandes 1/29/2014 20:48'!
drawOn: aCanvas
	| position awards |
	super drawOn: aCanvas.
	awards := self scoreAsAwards.
	position := bounds topLeft.
	1 to: self awards do: [:awardIndex | | form |
		(awards at: awardIndex) digitValue timesRepeat: [ 
			form := awardForms at: awardIndex.
			aCanvas translucentImage: form at: position.
			position := position + ((awardForms at: awardIndex) width + gap @ 0)].
		(awards at: awardIndex)  ~= $0 ifTrue: [
			position := position + (gap @ 0)]].! !

!ScoreMiniMorph methodsFor: 'initialize' stamp: 'HilaireFernandes 1/18/2014 10:10'!
initialize 
	super initialize.
	self color: Color transparent.
	awardForms := Array new: self awards.
	#('crownSmall.png' 'goldCupSmall.png' 'goldMedalSmall.png'  'silverMedalSmall.png'  'bronzeMedalSmall.png') 
		doWithIndex: [:award :index | awardForms at: index put: (Stoa getClipart: Path * 'awards' / award) ].
	height := 0.
	gap := Stoa scale: 5.
	awardForms do: [ :aForm| height := height max: aForm height].! !

!ScoreMiniMorph methodsFor: 'drawing' stamp: 'HilaireFernandes 2/20/2014 11:12'!
minExtent
	| minWidth awards |
	awards := self scoreAsAwards.
	minWidth := 0.
	1 to: self awards do: [:awardIndex | 
		(awards at: awardIndex) digitValue timesRepeat: [ 
			minWidth := minWidth + (awardForms at: awardIndex) width + gap.
		(awards at: awardIndex)  ~= $0 ifTrue: [
			minWidth := minWidth + gap]]].
	^ minWidth @ height ! !

!ScoreMiniMorph methodsFor: 'accessing' stamp: 'HilaireFernandes 12/30/2013 12:29'!
score
"the user score"
	^ Stoa score! !

!ScoreMiniMorph methodsFor: 'accessing' stamp: 'HilaireFernandes 12/23/2013 12:00'!
scoreAsAwards
"Damned, we even have a method for that ;-)"
	^ self score printPaddedWith: $0 to: self awards base: self scoreRadix ! !

!ScoreMiniMorph methodsFor: 'accessing' stamp: 'HilaireFernandes 2/1/2014 14:41'!
scoreRadix
"it is the number of time we can repeat a given award + 1
Understand: it is the radix system our score is converted to to give its awards representation"
	^ 5! !

Morph subclass: #ScrollNumberMorph
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'iStoaXP-Core-Widget'!

!ScrollNumberMorph commentStamp: '<historical>' prior: 0!
I am a horizontal scroll of number. My scroll is composed by a collection of decorated number slider morph.!

ScrollNumberMorph subclass: #ProgressionMorph
	instanceVariableNames: 'originalProgression answers'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'iStoaXP-Core-Game'!

!ProgressionMorph commentStamp: 'HilaireFernandes 5/27/2014 15:41' prior: 0!
A ProgressionMorph is a game with a partial (arithmetic or anything else) progression to be completed.

I mainly add gameplay functionnality to my parent.
!

!ProgressionMorph class methodsFor: 'instance creation' stamp: 'HilaireFernandes 1/22/2014 14:42'!
progression: aCollection min: min max: max shape: symbolOrColor answers: answers
	^ (self values: aCollection min: min max: max shape: symbolOrColor)
		answers: answers;
		yourself! !

!ProgressionMorph methodsFor: 'accessing' stamp: 'HilaireFernandes 1/22/2014 14:24'!
answers: aCollection
	answers := aCollection! !

!ProgressionMorph methodsFor: 'accessing' stamp: 'HilaireFernandes 1/22/2014 14:22'!
numberOfQuestions
	^ originalProgression count: [ :each | each isNil ]! !

!ProgressionMorph methodsFor: 'accessing' stamp: 'HilaireFernandes 7/27/2014 23:50'!
numberOfRightAnswers
	| userAnswers quantity|
	userAnswers := self userAnswers.
	quantity := 0.
	self rightAnswers withIndexDo: [:answer :index | 
		(originalProgression at: index) ifNil: 
			[answer = (userAnswers at: index) ifTrue: [quantity := quantity + 1]]].
	^ quantity! !

!ProgressionMorph methodsFor: 'accessing' stamp: 'HilaireFernandes 1/22/2014 14:24'!
rightAnswers
	^ answers! !

!ProgressionMorph methodsFor: 'helpers' stamp: 'HilaireFernandes 1/22/2014 14:57'!
showErrors
	self userAnswers withIndexDo: [ :anAnswer :index |
		anAnswer = (self rightAnswers at: index)
			ifTrue: [(self sliders at: index) numberSlider color: Color transparent ]
			ifFalse: [(self sliders at: index) numberSlider color: Stoa wrongColor] 
		]! !

!ProgressionMorph methodsFor: 'helpers' stamp: 'HilaireFernandes 1/22/2014 14:59'!
showSolution
	self rightAnswers withIndexDo: [ :answer : index|
		(self sliders at: index) value: answer].
	self showErrors! !

!ProgressionMorph methodsFor: 'tests' stamp: 'HilaireFernandes 1/22/2014 14:39'!
testSuccessAnswer
	^ self rightAnswers = self userAnswers ! !

!ProgressionMorph methodsFor: 'accessing' stamp: 'HilaireFernandes 1/22/2014 14:25'!
userAnswers
	^ self values! !

!ProgressionMorph methodsFor: 'initialize' stamp: 'HilaireFernandes 1/22/2014 14:20'!
values: aCollection min: aMin max: aMax shape: aShape
	super values: aCollection min: aMin max: aMax shape: aShape.
	originalProgression := aCollection! !

!ScrollNumberMorph class methodsFor: 'examples' stamp: 'HilaireFernandes 1/22/2014 10:15'!
example1
"
	ScrollNumberMorph example1 openInWorld
"

	^ self 
		values: #(1 nil 3 nil nil) 
		min: 1 
		max: 5 
		shape:  Color green! !

!ScrollNumberMorph class methodsFor: 'examples' stamp: 'HilaireFernandes 1/22/2014 10:37'!
example2
"
	ScrollNumberMorph example2 openInWorld
"

	^ self 
		values: #(1 nil 3 nil nil) 
		min: 1 
		max: 5 
		shape:  #note! !

!ScrollNumberMorph class methodsFor: 'examples' stamp: 'HilaireFernandes 1/22/2014 10:37'!
example3
"
	ScrollNumberMorph example3 openInWorld
"

	^ self 
		values: #(1 nil 3 nil nil) 
		min: 1 
		max: 5 
		shape:  #memo! !

!ScrollNumberMorph class methodsFor: 'initialize' stamp: 'HilaireFernandes 1/21/2014 21:11'!
values: aCollection min: min max: max shape: aSymbolOrColor
	^ self new 
		values: aCollection min: min max: max
		shape: aSymbolOrColor ! !

!ScrollNumberMorph methodsFor: 'initialize' stamp: 'HilaireFernandes 1/22/2014 15:32'!
initialize
	super initialize.
	self color: Color transparent.
	self layoutPolicy: TableLayout new;
		listDirection: #leftToRight;
		hResizing: #shrinkWrap;
		vResizing: #shrinkWrap;
		layoutInset: 0;
		cellInset: 0! !

!ScrollNumberMorph methodsFor: 'as yet unclassified' stamp: 'HilaireFeranndes 3/23/2014 09:59'!
slider: symbolOrColor lighter: boolean value: value min: min max: max
"Give me a number slider"
 	| slider fx|
	
	fx := boolean ifTrue: [ #darker ] ifFalse: [ #lighter ].
	symbolOrColor isSymbol 
		ifTrue: [
			symbolOrColor == #memo
				ifTrue: [slider := Stoa memoValue: value min: min max: max].
			symbolOrColor == #note
				ifTrue: [slider := Stoa noteValue: value min: min max: max].
			slider ifNil: [self error: 'Unknow symbol, ', symbolOrColor].
			fx == #darker ifTrue: [ 	slider form: (slider form darker: 0.08)]] 
		ifFalse: [ 
			slider := Stoa 
				color: (symbolOrColor perform: fx)
				value:  value
				min: min
				max: max ].
	^ slider! !

!ScrollNumberMorph methodsFor: 'accessing' stamp: 'HilaireFernandes 1/22/2014 11:14'!
sliders
	^ submorphs! !

!ScrollNumberMorph methodsFor: 'accessing' stamp: 'HilaireFernandes 1/22/2014 11:10'!
values
	^ submorphs collect: [ :sliderMorph | sliderMorph value ]! !

!ScrollNumberMorph methodsFor: 'accessing' stamp: 'HilaireFernandes 1/22/2014 11:13'!
values: aCollection
	aCollection size  ~= submorphs size ifTrue: [ self error: 'Collections size not matching'].
	submorphs withIndexDo: [:slider :index  |  
		slider value: (aCollection at: index)]! !

!ScrollNumberMorph methodsFor: 'initialize' stamp: 'HilaireFernandes 1/22/2014 10:18'!
values: aCollection min: min max: max shape: symbolOrColor
	1 to: aCollection size do: [ :index | | slider value isButtonHidden |
		value := aCollection at: index.
		isButtonHidden := value isNil not.
		value ifNil: [value := (min to: max) atRandom].
		slider := self 
			slider: symbolOrColor 
			lighter: index odd 
			value: value min: min max: max.
		isButtonHidden ifTrue: [ slider hideButtons].
		self addMorphBack: slider ].! !

Morph subclass: #SimpleCellMorph
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'iStoaXP-Core-Widget'!

!SimpleCellMorph commentStamp: 'HilaireFernandes 5/22/2014 09:02' prior: 0!
SimpleCellMorph populate a SimpleTableMorph. Its unique submorph is its content.
!

!SimpleCellMorph methodsFor: 'accessing' stamp: 'HilaireFernandes 5/22/2014 09:04'!
content
	^ self firstSubmorph! !

!SimpleCellMorph methodsFor: 'accessing' stamp: 'HilaireFernandes 5/27/2014 14:32'!
content: aMorph
	self removeAllMorphs.
	self addMorph: aMorph! !

!SimpleCellMorph methodsFor: 'as yet unclassified' stamp: 'HilaireFernandes 5/27/2014 14:38'!
initialize
	super initialize.
	self color: Color white;
		layoutPolicy: TableLayout new;
		hResizing: #rigid;
		vResizing: #rigid;
		wrapCentering: #center;
		listCentering: #center.! !

Morph subclass: #SimpleTableMorph
	instanceVariableNames: 'rows columns rowsArray cellGap'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'iStoaXP-Core-Widget'!

!SimpleTableMorph commentStamp: 'HilaireFernandes 5/27/2014 14:14' prior: 0!
A SimpleTableMorph to present submorphs in a table.

(SimpleTableMorph for: { 
	{StringMorph contents: 'Hello, I''am happty now!!'. 
	Morph new color: Color red.  
	Morph new color: Color blue }.
	{Morph new color: Color yellow.
	StringMorph contents: 'Are you happty now?'.
	 Morph new color: Color orange}}) openInWorld!

!SimpleTableMorph class methodsFor: 'instance creation' stamp: 'HilaireFernandes 5/22/2014 09:17'!
for: rowsArray
	"layout in a table the morphs in this array of rows"
	^ self basicNew for: rowsArray;
		initialize;
		yourself! !

!SimpleTableMorph methodsFor: 'adding' stamp: 'HilaireFernandes 5/24/2014 00:10'!
addRow: array
	| rowMorph |
	rowMorph := self newRow.
	array do: [ :each | rowMorph addMorphBack: (self newCell: each)].
	self addMorphBack: rowMorph! !

!SimpleTableMorph methodsFor: 'accessing' stamp: 'HilaireFernandes 5/26/2014 20:33'!
border: integer
	self cellInset: integer;
		layoutInset: integer.
	self submorphsDo: [ :aRowMorph |
		aRowMorph cellInset: integer ]! !

!SimpleTableMorph methodsFor: 'accessing' stamp: 'HilaireFernandes 5/26/2014 20:36'!
borderColor: aColor
	self color: aColor ! !

!SimpleTableMorph methodsFor: 'accessing' stamp: 'HilaireFernandes 5/27/2014 14:42'!
cellAt: aPoint
	^ (self column: aPoint x) at: aPoint y! !

!SimpleTableMorph methodsFor: 'accessing' stamp: 'HilaireFernandes 5/27/2014 14:43'!
cellColor: aColor at: aPoint
	(self cellAt: aPoint) color: aColor ! !

!SimpleTableMorph methodsFor: 'accessing' stamp: 'HilaireFernandes 5/27/2014 14:55'!
cellGap: integer
	cellGap := integer.
	self updateLayout ! !

!SimpleTableMorph methodsFor: 'accessing' stamp: 'HilaireFernandes 5/27/2014 14:57'!
cellsColor: aColor
	1 to: columns do: [:i | 
		(self column: i) do: [:aCell | aCell color: aColor]].
! !

!SimpleTableMorph methodsFor: 'accessing' stamp: 'HilaireFernandes 5/24/2014 00:15'!
column: integer
"Return the cells in this column"
	^ self submorphs collect: [ :rowMorph | rowMorph submorphs at: integer ]! !

!SimpleTableMorph methodsFor: 'compute' stamp: 'HilaireFernandes 5/27/2014 14:30'!
columnMinWidth: integer
"compute its minimum width"
	| width |
	width := 0.
	(self column: integer) do: [:cell | width := width max: cell firstSubmorph width].
	^ width ! !

!SimpleTableMorph methodsFor: 'accessing' stamp: 'HilaireFernandes 5/27/2014 14:15'!
columns
	^ columns! !

!SimpleTableMorph methodsFor: 'accessing' stamp: 'HilaireFernandes 6/7/2014 15:53'!
contentAt: aPoint
"return the morph in the cell at aPoint coordinates"
	^ (self cellAt: aPoint) content! !

!SimpleTableMorph methodsFor: 'initialization' stamp: 'HilaireFernandes 5/24/2014 00:21'!
for: arrays
	rowsArray := arrays.
	rows := rowsArray size.
	columns := rowsArray first size.
	rowsArray 
		detect: [:array | array size ~= columns] 
		ifFound: [self error: 'This is not a table'].
! !

!SimpleTableMorph methodsFor: 'initialization' stamp: 'HilaireFernandes 5/27/2014 14:55'!
initialize
	super initialize.
	self color: Color black.
	self layoutPolicy: TableLayout new;
		listDirection: #topToBottom;
		hResizing: #shrinkWrap;
		vResizing: #shrinkWrap.
	rowsArray do: [ :aRow | self addRow: aRow ].
	self border: 1.
	cellGap := 1.
	self updateLayout! !

!SimpleTableMorph methodsFor: 'instance creation' stamp: 'HilaireFernandes 8/5/2014 12:33'!
newCell: aContent
	^ SimpleCellMorph new content: (aContent ifNil: [ Morph new color: Color transparent ])! !

!SimpleTableMorph methodsFor: 'instance creation' stamp: 'HilaireFernandes 5/24/2014 00:17'!
newRow
"a transparent morph to layout the cell of a raw"
	^ Morph new color: Color transparent;
		layoutPolicy: TableLayout new;
		listDirection: #leftToRight;
		hResizing: #shrinkWrap;
		vResizing: #shrinkWrap;
		yourself! !

!SimpleTableMorph methodsFor: 'accessing' stamp: 'HilaireFernandes 5/24/2014 00:13'!
row: integer
"Return the cells in this row"
	^ (self submorphs at: integer) submorphs! !

!SimpleTableMorph methodsFor: 'compute' stamp: 'HilaireFernandes 5/27/2014 14:31'!
rowMinHeight: integer
"compute its minimum height"
	| height |
	height := 0.
	(self row: integer) do: [:cell | height := height max: cell firstSubmorph height].
	^ height! !

!SimpleTableMorph methodsFor: 'accessing' stamp: 'HilaireFernandes 5/27/2014 14:15'!
rows
	^ rows! !

!SimpleTableMorph methodsFor: 'private' stamp: 'HilaireFernandes 5/27/2014 14:55'!
updateLayout
"update: 
	cells in a row share the same height
	cells in a column share the same width
"
	| length |
	1 to: columns do: [:i | 
		length := cellGap * 2 + (self columnMinWidth: i).
		(self column: i) do: [:aCell | aCell width: length]].
	1 to: rows do: [ :i |
		length := cellGap * 2 + (self rowMinHeight: i).
		(self row: i) do: [:aCell | aCell height: length ] ]! !

Morph subclass: #StackMorph
	instanceVariableNames: 'form offset orientation quantity'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'iStoaXP-Core-Widget'!

!StackMorph commentStamp: 'HilaireFernandes 7/22/2014 20:51' prior: 0!
A StackMorph is a stack of an identical form, vertically or horizontally.

Instance Variables
	form:		Form
	items:		integer
	offset:		integer
	orentation:		symbol

form
	-  Form representing the item

items
	- the number of items stacked

offset
	- the offset (negative or positive) when 
	stacking subsequent items items

orentation
	- the direction of stacking: #leftToRight 
	#rightToLeft #topToBottom #bottomToTop
!

!StackMorph class methodsFor: 'instance creation' stamp: 'HilaireFernandes 7/23/2014 11:39'!
form: aForm offset: anOffset quantity: integer orientation: symbol
	^ self basicNew 
		form: aForm ;
		offset: anOffset ;
		quantity: integer;
		orientation: symbol ;
		initialize ! !

!StackMorph methodsFor: 'drawing' stamp: 'HilaireFernandes 7/23/2014 11:39'!
drawBottomToTopOn: aCanvas
	| position |
	position := bounds bottomLeft - (0 @ form height).
	quantity timesRepeat: [ 
		aCanvas translucentImage: form at: position.
		position := position - (0 @ (form height + offset))]! !

!StackMorph methodsFor: 'drawing' stamp: 'HilaireFernandes 7/23/2014 11:39'!
drawLeftToRightOn: aCanvas
	| position |
	position := bounds topLeft.
	quantity timesRepeat: [ 
		aCanvas translucentImage: form at: position.
		position := position + ((form width + offset) @ 0)]! !

!StackMorph methodsFor: 'drawing' stamp: 'HilaireFernandes 7/22/2014 22:48'!
drawOn: aCanvas
	super drawOn: aCanvas.
	orientation == #leftToRight ifTrue: [self drawLeftToRightOn: aCanvas].
	orientation == #rightToLeft ifTrue: [self drawRightToLeftOn: aCanvas].
	orientation == #topToBottom ifTrue: [self drawTopToBottomOn: aCanvas].
	orientation == #bottomToTop ifTrue: [self drawBottomToTopOn: aCanvas]! !

!StackMorph methodsFor: 'drawing' stamp: 'HilaireFernandes 7/23/2014 11:39'!
drawRightToLeftOn: aCanvas
	| position |
	position := bounds topRight - (form width @ 0).
	quantity timesRepeat: [ 
		aCanvas translucentImage: form at: position.
		position := position - ((form width + offset) @ 0)]! !

!StackMorph methodsFor: 'drawing' stamp: 'HilaireFernandes 7/23/2014 11:39'!
drawTopToBottomOn: aCanvas
	| position |
	position := bounds topLeft.
	quantity timesRepeat: [ 
		aCanvas translucentImage: form at: position.
		position := position + (0 @ (form height + offset))]! !

!StackMorph methodsFor: 'accessing' stamp: 'HilaireFernandes 7/22/2014 20:52'!
form: anObject
	form := anObject! !

!StackMorph methodsFor: 'initialization' stamp: 'HilaireFernandes 7/22/2014 22:49'!
initialize
	super initialize.
	self color: Color transparent.
	self extent: self minExtent! !

!StackMorph methodsFor: 'accessing' stamp: 'HilaireFernandes 7/23/2014 11:39'!
minExtent
	^ (orientation == #leftToRight or: [ orientation == #rightToLeft ])
		ifTrue: [(quantity * (form width + offset) - offset) @ form height ]
		ifFalse: [form width @ (quantity * (form height + offset) - offset )]! !

!StackMorph methodsFor: 'accessing' stamp: 'HilaireFernandes 7/22/2014 20:52'!
offset: anObject
	offset := anObject! !

!StackMorph methodsFor: 'accessing' stamp: 'HilaireFernandes 7/22/2014 20:59'!
orientation: anObject
	orientation := anObject! !

!StackMorph methodsFor: 'accessing' stamp: 'HilaireFernandes 7/23/2014 11:39'!
quantity
	^ quantity! !

!StackMorph methodsFor: 'accessing' stamp: 'HilaireFernandes 7/23/2014 11:39'!
quantity: anObject
	quantity := anObject! !

Morph subclass: #StoaAbstractView
	uses: TEasilyThemed
	instanceVariableNames: 'controls background dominantColor topbar username mascotDialogOk'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'iStoaXP-Core'!

StoaAbstractView class
	uses: TEasilyThemed classTrait
	instanceVariableNames: 'UniqueInstance'!

!StoaAbstractView commentStamp: '<historical>' prior: 0!
A StoaAbstractView is an abstract view for the screen layout.
Subclasses are StoaApp for the main screen to browse the activities and StoaPlayMorph the view to play an activity.

It contains a background picture, a side toolbar with buttons and a main view for the activity board.

Its extent is set by StoaSettings>>myDisplayExtent.
The application view is designed for a native resolution of 2560x1600.
Of course the resolution on the current display may be different, in that case all graphics are scaled, see StoaSettings>>scale.
!

StoaAbstractView class
	uses: TEasilyThemed classTrait
	instanceVariableNames: 'UniqueInstance'!

!StoaAbstractView class methodsFor: 'accessing' stamp: 'HilaireFernandes 3/8/2014 12:03'!
current
	^ UniqueInstance ifNil: [ UniqueInstance := self basicNew.
		UniqueInstance initialize.
		UniqueInstance]! !

!StoaAbstractView class methodsFor: 'initialize-destroy' stamp: 'HilaireFernandes 4/2/2014 20:35'!
delete
	UniqueInstance ifNotNil: [ 
		UniqueInstance delete.
		UniqueInstance := nil]! !

!StoaAbstractView class methodsFor: 'initialize-destroy' stamp: 'HilaireFernandes 12/30/2013 21:41'!
new
	self error: 'You can not create several instances of ', self printString.! !

!StoaAbstractView class methodsFor: 'initialize-destroy' stamp: 'HilaireFernandes 4/3/2014 21:03'!
rescale
	"do the necessary to rescale my self"
	self delete.
	self current openInWorld! !

!StoaAbstractView methodsFor: 'accessing' stamp: 'HilaireFernandes 12/27/2013 17:03'!
alphaDominantColor
	^ self dominantColor muchLighter alpha: 0.5
! !

!StoaAbstractView methodsFor: 'accessing' stamp: 'HilaireFernandes 4/27/2014 15:08'!
background
	^ background
		ifNil: [background := Stoa getBackground: 'background4.jpg']! !

!StoaAbstractView methodsFor: 'controls bar' stamp: 'HilaireFernandes 12/18/2013 20:28'!
controls
	controls ifNotNil: [^ controls].
	controls := Morph new	color: self alphaDominantColor.
	self setControlsLayout.
	self controlsWidget do: [:aSymbol | | ret |
		ret := self perform: aSymbol.
		ret ifNotNil: [ controls addMorphBack: ret ]].
	^ controls! !

!StoaAbstractView methodsFor: 'controls bar' stamp: 'HilaireFernandes 12/18/2013 20:30'!
controlsWidget
"Method name returning widget morph for the control bar"
	self subclassResponsibility ! !

!StoaAbstractView methodsFor: 'accessing' stamp: 'HilaireFernandes 5/20/2014 14:58'!
dominantColor
	^ dominantColor ifNil: [dominantColor := self background dominantColor ] ! !

!StoaAbstractView methodsFor: 'drawing' stamp: 'HilaireFernandes 5/19/2014 22:38'!
drawOn: aCanvas
	self background ifNotNilDo: [:aForm |
		aCanvas drawImage: aForm at: self innerBounds origin].
	super drawOn: aCanvas ! !

!StoaAbstractView methodsFor: 'controls bar' stamp: 'HilaireFernandes 2/16/2014 11:41'!
homeBtn
	^ self newButtonFor: self
		getState: nil
		action: #homeCallback
		arguments: nil
		getEnabled: nil
		labelForm: (Stoa getClipart: Path * 'buttons' / 'home.png')
		help: nil! !

!StoaAbstractView methodsFor: 'controls bar' stamp: 'HilaireFernandes 2/16/2014 11:41'!
homeCallback
	self subclassResponsibility! !

!StoaAbstractView methodsFor: 'initialize' stamp: 'HilaireFeranndes 3/23/2014 10:53'!
initialize
	super initialize.
	self color: Color transparent.
	self layoutPolicy: ProportionalLayout new.
	self extent: Stoa myDisplayExtent.
	self installControls.
	self beSticky! !

!StoaAbstractView methodsFor: 'initialize' stamp: 'HilaireFernandes 4/25/2014 18:50'!
installControls
"This bar occupies 
	- 10% in width (of 2560) = 256  
	- 100% in height (of 1600) = 1600"
	controls = nil ifFalse: [
		controls delete.
		controls := nil].
	self addMorph: self controls fullFrame: (LayoutFrame fractions: (0.9 @ 0 corner: 1 @ 1))! !

!StoaAbstractView methodsFor: 'initialize' stamp: 'HilaireFernandes 4/25/2014 18:46'!
installTopbar
"This bar occupies 
	- 90% in width (of 2560) = 2304  
	- 10% in height (of 1600) = 160"
	self 
		addMorph: self topbar 
		fullFrame: (LayoutFrame fractions: (0@0 corner: 1@0.10)).! !

!StoaAbstractView methodsFor: 'dialog' stamp: 'HilaireFernandes 4/24/2014 09:54'!
ok
	mascotDialogOk := true! !

!StoaAbstractView methodsFor: 'controls bar' stamp: 'HilaireFernandes 2/15/2014 14:49'!
okBtn
	^ self newButtonFor: self
		getState: nil
		action: #okCallback
		arguments: nil
		getEnabled: nil
		labelForm: (Stoa getClipart: Path * 'buttons' / 'ok.png')
		help: nil! !

!StoaAbstractView methodsFor: 'controls bar' stamp: 'HilaireFernandes 2/15/2014 14:50'!
okCallback
	self subclassResponsibility ! !

!StoaAbstractView methodsFor: 'controls bar' stamp: 'HilaireFernandes 12/18/2013 20:29'!
setControlsLayout
"set the layout of the controls bar"
	self subclassResponsibility ! !

!StoaAbstractView methodsFor: 'dialog' stamp: 'HilaireFernandes 4/22/2014 18:35'!
showDialog: aMascotMessage! !

!StoaAbstractView methodsFor: 'controls bar' stamp: 'HilaireFernandes 2/15/2014 14:53'!
stopBtn
	^ self newButtonFor: self
		getState: nil
		action: #stopCallback
		arguments: nil
		getEnabled: nil
		labelForm: (Stoa getClipart: Path * 'buttons' / 'stop.png')
		help: nil! !

!StoaAbstractView methodsFor: 'controls bar' stamp: 'HilaireFernandes 2/15/2014 14:51'!
stopCallback
	self subclassResponsibility ! !

!StoaAbstractView methodsFor: 'accessing' stamp: 'HilaireFernandes 4/25/2014 15:47'!
topbar
	^ topbar ifNil: [ 
		topbar := Morph new color: Color transparent.
		topbar layoutPolicy: TableLayout new;
			listDirection: #leftToRight;
			wrapCentering: #center;
			layoutInset: (Stoa scale: 10).
		topbar 
			addMorph: Stoa userAvatar;
			addMorph: self username;
			addMorph: Stoa hfill;
			addMorph: (ScoreMiniMorph new hResizing: #shrinkWrap; vResizing: #shrinkWrap).
		topbar]! !

!StoaAbstractView methodsFor: 'as yet unclassified' stamp: 'HilaireFernandes 3/26/2014 10:49'!
updateUser
	self username contents: Stoa username! !

!StoaAbstractView methodsFor: 'accessing' stamp: 'HilaireFernandes 3/26/2014 10:53'!
username
	^ username ifNil: [ 
		username := (Stoa stringMorph: Stoa username)
			color: Color white;
			yourself]! !

!StoaAbstractView methodsFor: 'dialog' stamp: 'HilaireFernandes 4/24/2014 09:54'!
waitForTuxDialog
	[ mascotDialogOk ] whileFalse: [ World doOneCycle ]! !

StoaAbstractView subclass: #StoaApp
	instanceVariableNames: 'etayagePage etayagePreviews etayagesByCategory maximumScore'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'iStoaXP-Core'!

StoaApp class
	instanceVariableNames: 'EtayageCategoryIndex'!

!StoaApp commentStamp: '<historical>' prior: 0!
StoaApp is the welcome view in the iStoa application. From here, the user can: 
- quit the application
- adjust the settings
- access to the board categories.
!

StoaApp class
	instanceVariableNames: 'EtayageCategoryIndex'!

!StoaApp class methodsFor: 'accesing' stamp: 'HilaireFernandes 5/17/2014 17:04'!
EtayageCategoryIndex
	^ EtayageCategoryIndex ifNil: [ EtayageCategoryIndex := 1 ]! !

!StoaApp class methodsFor: 'accesing' stamp: 'HilaireFernandes 5/17/2014 17:04'!
EtayageCategoryIndex: index
	 EtayageCategoryIndex := index! !

!StoaApp methodsFor: 'controls action' stamp: 'HilaireFernandes 12/20/2013 23:42'!
categoryChanged
	|   originalForm destForm |
	originalForm := Form fromDisplay: etayagePreviews  bounds.
	self updateEtayagePreviews.
	destForm := self imageForm copy: (etayagePreviews position - self position extent: originalForm extent).
	originalForm displayAt: etayagePreviews  position.
	Display 
		fadeImageHor: destForm 
		at: etayagePreviews  position.
! !

!StoaApp methodsFor: 'controls bar' stamp: 'HilaireFernandes 3/23/2014 14:32'!
controlsWidget
	^ #(#prevPageBtn #etayageCategoriesList #nextPageBtn "#hFill" #scoreBtn #settingsBtn #quitBtn)! !

!StoaApp methodsFor: 'accessing' stamp: 'HilaireFernandes 7/28/2014 10:32'!
etayageCategories
"Categories of etayage in the application, organised in a dictionary:
 - key --  localised string representing the category names
 - value -- collection of Etayage classes in the given category"
	etayagesByCategory ifNil: [ 
		etayagesByCategory := Dictionary new.
		Etayage allSubclasses do: [ :aClass | | sortedCollection |
			sortedCollection := etayagesByCategory 
				at: aClass myCategory 
				ifAbsentPut: [SortedCollection sortBlock: [:class1 :class2 |
						class1 printString numericSuffix < class2 printString numericSuffix]].
			 sortedCollection add: aClass].
		etayagesByCategory].
	^ etayagesByCategory keys sorted.! !

!StoaApp methodsFor: 'controls bar' stamp: 'HilaireFernandes 5/28/2014 17:03'!
etayageCategoriesList
	^ self
		newDropListFor: self
		list: #etayageCategories
		getSelected: #etayageCategoryIndex
		setSelected: #etayageCategoryIndex:
		help: 'These are the categories you can select to show the activities above. Each category proposes several activities on the same topic.' translated! !

!StoaApp methodsFor: 'controls action' stamp: 'HilaireFernandes 5/17/2014 17:05'!
etayageCategoryIndex
	^ self class EtayageCategoryIndex! !

!StoaApp methodsFor: 'controls action' stamp: 'HilaireFernandes 5/17/2014 17:06'!
etayageCategoryIndex: integer
	self class EtayageCategoryIndex = integer ifTrue: [ ^ self ].
	self class EtayageCategoryIndex: integer.
	etayagePage := 1.
	self categoryChanged  ! !

!StoaApp methodsFor: 'accessing' stamp: 'HilaireFernandes 12/20/2013 20:55'!
etayagePage
"The current preview page we are looking at"
	^ etayagePage ifNil: [ etayagePage := 1 ]! !

!StoaApp methodsFor: 'accessing' stamp: 'HilaireFernandes 12/20/2013 21:08'!
etayagePages
"How manty pages of Etayage do we have in the preview"
	^ (self etayagesInCurrentCategory size - 1) // self etayagesPerPage + 1 ! !

!StoaApp methodsFor: 'accessing' stamp: 'HilaireFernandes 5/17/2014 17:06'!
etayagesInCurrentCategory
"Collection of the etayages in the user selected category"
	^ etayagesByCategory at: (self etayageCategories at: self etayageCategoryIndex).
	! !

!StoaApp methodsFor: 'accessing' stamp: 'HilaireFernandes 5/28/2014 17:33'!
etayagesPerPage
	^ 6! !

!StoaApp methodsFor: 'controls bar' stamp: 'HilaireFernandes 12/26/2013 12:23'!
hFill
	^ Stoa hfill! !

!StoaApp methodsFor: 'initialize' stamp: 'HilaireFernandes 3/26/2014 09:15'!
initialize
	super initialize.
	self installUser;
		installEtayagePreviews.
	self updateEtayagePreviews; 
		updateUser;
		installTopbar.
! !

!StoaApp methodsFor: 'controls bar' stamp: 'HilaireFernandes 12/23/2013 10:05'!
installControls
"This bar occupies 
	- 100% in width (of 2560) = 2560  
	- 13% in height (of 1600) = 208"
	self addMorph: self controls fullFrame: (LayoutFrame fractions: (0 @ 0.87 corner: 1 @ 1))! !

!StoaApp methodsFor: 'initialize' stamp: 'HilaireFernandes 7/22/2014 12:07'!
installEtayagePreviews
	etayagePreviews := Morph new color: self alphaDominantColor.
	etayagePreviews layoutPolicy: TableLayout new;
		useRoundedCorners;
		listDirection: #leftToRight;
		cellInset: (10 * Stoa scale);
		layoutInset: (10 * Stoa scale);
		wrapCentering: #topLeft;
		wrapDirection: #topToBottom;
		hResizing: #shrinkWrap;
		vResizing: #shrinkWrap.
	self 
		addMorph: etayagePreviews 
		fullFrame: (LayoutFrame fractions: (0.05@0.10 corner: 0.95@0.82))! !

!StoaApp methodsFor: 'initialize' stamp: 'HilaireFernandes 2/20/2014 18:32'!
installUser
"Depending on the state, it will:
1. Create a new user instance if none, 
2. Or, load an user instance if there is only one
3. Or, ask the user in a modal dialog to choose an user"
	Stoa setUser.
	! !

!StoaApp methodsFor: 'score' stamp: 'HilaireFernandes 12/21/2013 12:37'!
maximumScore
"The maximum score for this application"
	^ maximumScore ifNil: [ 
		maximumScore := Etayage allSubclasses inject: 0 into: [ :subScore :aClass |
			subScore + aClass score ] ]! !

!StoaApp methodsFor: 'controls action' stamp: 'HilaireFernandes 12/20/2013 22:29'!
nextPage
	etayagePage := etayagePage + 1.
	self nextPageChanged .! !

!StoaApp methodsFor: 'controls bar' stamp: 'HilaireFernandes 1/18/2014 10:11'!
nextPageBtn
	^ self newButtonFor: self
		getState: nil
		action: #nextPage
		arguments: nil
		getEnabled: #nextPageEnabled
		labelForm: (Stoa getClipart: Path * 'buttons' / 'nextPage.png')
		help: nil! !

!StoaApp methodsFor: 'controls action' stamp: 'HilaireFernandes 12/20/2013 23:41'!
nextPageChanged
	|   originalForm destForm|
	originalForm := Form fromDisplay:  etayagePreviews bounds.
	self updateEtayagePreviews.
	destForm := self imageForm copy: (etayagePreviews position - self position extent: originalForm extent).
	originalForm displayAt: etayagePreviews position.
	Display 
		slideImage: destForm
		at: etayagePreviews position delta: -100@0.
! !

!StoaApp methodsFor: 'controls action' stamp: 'HilaireFernandes 12/20/2013 21:25'!
nextPageEnabled
	^ self etayagePage  ~= self etayagePages ! !

!StoaApp methodsFor: 'controls action' stamp: 'HilaireFernandes 12/20/2013 22:29'!
prevPage
	etayagePage := etayagePage - 1.
	self prevPageChanged .! !

!StoaApp methodsFor: 'controls bar' stamp: 'HilaireFernandes 1/18/2014 10:11'!
prevPageBtn
	^ self newButtonFor: self
		getState: nil
		action: #prevPage
		arguments: nil
		getEnabled: #prevPageEnabled
		labelForm: (Stoa getClipart: Path * 'buttons' / 'prevPage.png')
		help: nil! !

!StoaApp methodsFor: 'controls action' stamp: 'HilaireFernandes 12/20/2013 23:41'!
prevPageChanged
	|   originalForm destForm|
	originalForm := Form fromDisplay:  etayagePreviews bounds.
	self updateEtayagePreviews.
	destForm := self imageForm copy: (etayagePreviews position - self position extent: originalForm extent).
	originalForm displayAt: etayagePreviews position.
	Display 
		slideImage: destForm
		at: etayagePreviews position delta: 100@0.
! !

!StoaApp methodsFor: 'controls action' stamp: 'HilaireFernandes 12/20/2013 21:25'!
prevPageEnabled
	^ self etayagePage  ~= 1! !

!StoaApp methodsFor: 'controls action' stamp: 'HilaireFernandes 4/5/2014 08:59'!
quit
	Stoa develMode 
		ifTrue: [Stoa deleteViews]
		ifFalse: [(self questionWithoutCancel: 'Really quit iStoa?' translated title: 'Question' translated) ifTrue: 
			[Smalltalk snapshot: false andQuit: true]]! !

!StoaApp methodsFor: 'controls bar' stamp: 'HilaireFernandes 3/26/2014 14:34'!
quitBtn
	^ self newButtonFor: self
		getState: nil
		action: #quit
		arguments: nil
		getEnabled: nil
		labelForm: (Stoa getClipart: Path * 'buttons' / 'quit.png')
		help: nil! !

!StoaApp methodsFor: 'controls bar' stamp: 'HilaireFernandes 2/2/2014 11:10'!
scoreBtn
	^ self newButtonFor: self
		getState: nil
		action: #showScore
		arguments: nil
		getEnabled: nil
		labelForm: (Stoa getClipart: Path * 'buttons' / 'score.png')
		help: nil! !

!StoaApp methodsFor: 'controls bar' stamp: 'HilaireFernandes 5/21/2014 11:30'!
setControlsLayout
	controls layoutPolicy: TableLayout new;
		 listDirection: #leftToRight;
		 cellInset: (Stoa scale: 40) @ 0;
		 wrapCentering: #center;
		 hResizing: #rigid;
		 vResizing: #spaceFill.! !

!StoaApp methodsFor: 'controls bar' stamp: 'HilaireFernandes 2/16/2014 18:32'!
settingsBtn
	^ self newButtonFor: self
		getState: nil
		action: #showSettings
		arguments: nil
		getEnabled: nil
		labelForm: (Stoa getClipart: Path * 'buttons' / 'settings.png')
		help: nil! !

!StoaApp methodsFor: 'controls action' stamp: 'HilaireFernandes 4/2/2014 20:56'!
showScore
"Switch to the ScorePageStoaPlayMorph"
	Stoa openScoreView.! !

!StoaApp methodsFor: 'controls action' stamp: 'HilaireFernandes 4/2/2014 20:54'!
showSettings
"Switch to the Settings view"
	Stoa openSettingsView! !

!StoaApp methodsFor: 'controls action' stamp: 'HilaireFernandes 5/20/2014 14:56'!
startEtayage: aClass event: event in: preview
"Switch to the StoaPlayMorph"
	Stoa openPlayViewOn: aClass.
	StoaPlay current 
		etayage: aClass;
		layoutChanged;
		extent: Stoa myDisplayExtent +(1@0).
	World doOneCycle.
	StoaPlay current extent: Stoa myDisplayExtent.! !

!StoaApp methodsFor: 'controls action' stamp: 'HilaireFernandes 3/23/2014 14:37'!
updateEtayagePreviews
	| startIndex etayages |
	etayagePreviews removeAllMorphs.
	startIndex := (self etayagePage - 1) * self etayagesPerPage + 1.
	etayages := self etayagesInCurrentCategory.
	startIndex to: (startIndex + self etayagesPerPage - 1 min: etayages size) do: [:anIndex | | preview|
		preview := (etayages at: anIndex) preview.
		preview on: #click send: #startEtayage:event:in: to: self withValue: (etayages at: anIndex).
		etayagePreviews addMorphBack: preview].
	self changed: #nextPageEnabled.
	self changed: #prevPageEnabled! !

StoaAbstractView subclass: #StoaPlay
	instanceVariableNames: 'board etayage'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'iStoaXP-Core'!

!StoaPlay commentStamp: '<historical>' prior: 0!
I am the view to plug and play the board.!

!StoaPlay class methodsFor: 'as yet unclassified' stamp: 'HilaireFernandes 5/20/2014 14:55'!
currentOn: etayageClass
	^ UniqueInstance ifNil: [ UniqueInstance := self basicNew.
		UniqueInstance etayageClass: etayageClass.
		UniqueInstance initialize.
		UniqueInstance]! !

!StoaPlay class methodsFor: 'initialize-destroy' stamp: 'HilaireFernandes 5/20/2014 15:10'!
rescale
	|oldView|
	oldView := UniqueInstance.
	self delete.
	(self currentOn: oldView etayage class) openInWorld.
	UniqueInstance 
		etayage: oldView etayage class;
		layoutChanged;
		extent: Stoa myDisplayExtent +(1@0).
	World doOneCycle.
	UniqueInstance extent: Stoa myDisplayExtent.! !

!StoaPlay methodsFor: 'accessing' stamp: 'HilaireFernandes 5/20/2014 14:58'!
background
	^ background ifNil: [
		background := etayage ifNotNil: [etayage background]]! !

!StoaPlay methodsFor: 'controls bar' stamp: 'HilaireFernandes 4/25/2014 18:52'!
controlsWidget
	^ #( #okBtn #nextBtn #stopBtn)! !

!StoaPlay methodsFor: 'submorphs-add/remove' stamp: 'HilaireFernandes 4/3/2014 09:21'!
delete
	self releaseEtayage.
	super delete! !

!StoaPlay methodsFor: 'accessing' stamp: 'HilaireFernandes 12/30/2013 16:55'!
etayage
"The etayage the user is learning with"
	^ etayage ! !

!StoaPlay methodsFor: 'accessing' stamp: 'HilaireFernandes 1/15/2014 11:28'!
etayage: aClassOrInstance 
"Pass an Etayage class or an instance, the lateone for debugging purpose"
	self releaseEtayage.
	aClassOrInstance isBehavior 
		ifTrue: [etayage := aClassOrInstance new] 
		ifFalse: [etayage := aClassOrInstance].
	etayage when: #etayageStepReadyToStartEvt	send: #installBoard to: self.
	etayage when: #finishedEtayageEvt send: #etayageFinished to: self.
	etayage playStep.! !

!StoaPlay methodsFor: 'accessing' stamp: 'HilaireFernandes 5/20/2014 14:55'!
etayageClass: aClass
"Pass an Etayage class or an instance, the lateone for debugging purpose"
	etayage := aClass
! !

!StoaPlay methodsFor: 'update' stamp: 'HilaireFernandes 4/2/2014 21:00'!
etayageFinished
"The user went to the end or quit before finishing"
	self showGameScore. 
	self etayage release.
	Stoa openAppView.! !

!StoaPlay methodsFor: 'initialize' stamp: 'HilaireFernandes 3/26/2014 10:59'!
initialize
	super initialize.
	self installTopbar.! !

!StoaPlay methodsFor: 'initialize' stamp: 'HilaireFernandes 1/20/2014 10:51'!
installBoard
"The board occupies 
	- 90% in width (of 2560) = 2304  
	- 90% in height (of 1600) = 1440"
	board ifNotNil: [board delete].
	board := self etayage board color: self alphaDominantColor.
	self addMorph: board fullFrame: (LayoutFrame fractions: (0 @ 0.1 corner: 0.9 @ 1)).
	self layoutChanged.
	"A bit hackish to ensure a proper layout"
	1000 to: 0 by: -200 do: [ :y | 
		self extent: Stoa myDisplayExtent + (0 @ y).
		World doOneCycle].
	self extent: Stoa myDisplayExtent.! !

!StoaPlay methodsFor: 'testing' stamp: 'HilaireFernandes 12/30/2013 22:31'!
isTimeLimitReached
<todo>
	^ false! !

!StoaPlay methodsFor: 'controls bar' stamp: 'HilaireFernandes 2/15/2014 14:53'!
nextBtn
	^ self newButtonFor: self
		getState: nil
		action: #nextCallback
		arguments: nil
		getEnabled: nil
		labelForm: (Stoa getClipart: Path * 'buttons' / 'next.png')
		help: nil! !

!StoaPlay methodsFor: 'controls bar' stamp: 'HilaireFernandes 2/15/2014 14:53'!
nextCallback
"The user requests to move to the next step and abort the current one"
	self etayage nextStep! !

!StoaPlay methodsFor: 'dialog' stamp: 'HilaireFernandes 4/24/2014 10:24'!
ok
"We pressed ok on a mascot message dialog, replace the dialog by the original message "
	self okFinal.
	board getBackQuestion.
! !

!StoaPlay methodsFor: 'controls bar' stamp: 'HilaireFernandes 2/15/2014 14:51'!
okCallback
<todo>
"The user informs he's done with the current step"
	self etayage stepPreAnalysis.! !

!StoaPlay methodsFor: 'dialog' stamp: 'HilaireFernandes 4/24/2014 10:24'!
okFinal
"We pressed ok on a mascot message dialog, we DON'T replace the dialog by the original message "
	super ok.
	self unlock; unlockContents.
	board unlockContents.! !

!StoaPlay methodsFor: 'controls bar' stamp: 'HilaireFernandes 1/18/2014 10:11'!
pauseBtn
	^ self newButtonFor: self
		getState: nil
		action: #pauseStep
		arguments: nil
		getEnabled: nil
		labelForm: (Stoa getClipart: Path * 'buttons' / 'pause.png')
		help: nil! !

!StoaPlay methodsFor: 'action' stamp: 'HilaireFernandes 12/30/2013 12:46'!
pauseStep
"The user informs he want to pause the step"! !

!StoaPlay methodsFor: 'controls bar' stamp: 'HilaireFernandes 1/18/2014 10:11'!
questionBtn
	^ self newButtonFor: self
		getState: nil
		action: #requestHelp
		arguments: nil
		getEnabled: nil
		labelForm: (Stoa getClipart: Path * 'buttons' / 'question.png')
		help: nil! !

!StoaPlay methodsFor: 'update' stamp: 'HilaireFernandes 12/30/2013 17:04'!
releaseEtayage
	etayage ifNotNil: [ etayage release ].
! !

!StoaPlay methodsFor: 'action' stamp: 'HilaireFernandes 12/30/2013 12:47'!
requestHelp
"The user requests help on the current step"! !

!StoaPlay methodsFor: 'controls bar' stamp: 'HilaireFernandes 4/25/2014 18:51'!
setControlsLayout
	controls layoutPolicy: TableLayout new;
		 listDirection: #bottomToTop;
		 cellInset: 0 @ (Stoa scale: 40) asInteger;
		 wrapCentering: #center;
		 hResizing: #spaceFill;
		 vResizing: #rigid.
! !

!StoaPlay methodsFor: 'dialog' stamp: 'HilaireFernandes 4/24/2014 09:54'!
showDialog: aMascotDialog
	board dialog: aMascotDialog.
	self extent: Stoa myDisplayExtent + (0@1).
	World doOneCycle.
	self extent: Stoa myDisplayExtent.
	self submorphsDo: [:m | m lock].
	board unlock.
	board submorphsDo: [ :m | m lock ].
	aMascotDialog unlock.
	mascotDialogOk := false.
	self waitForTuxDialog! !

!StoaPlay methodsFor: 'update' stamp: 'HilaireFernandes 3/26/2014 10:42'!
showGameScore
	|showScoreMorph|
	etayage ifNil: [ ^self ].
	showScoreMorph := ScoreGainMorph gain: self etayage score maxGain: self etayage maxScore.
	showScoreMorph center: board center.
	showScoreMorph openInWorld; startStepping. 
	[showScoreMorph isStepping] whileTrue: [World doOneCycle ].
	(Delay forMilliseconds: 500) wait.
	showScoreMorph delete.
	Stoa score: self etayage score at: self etayage class.! !

!StoaPlay methodsFor: 'controls bar' stamp: 'HilaireFernandes 2/15/2014 14:52'!
stopCallback
"The user stops the Etatage, save the score and get back to StoaApp"
	self etayageFinished ! !

StoaAbstractView subclass: #StoaScore
	instanceVariableNames: 'scoreLines totalScore'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'iStoaXP-Core'!

!StoaScore commentStamp: '<historical>' prior: 0!
I am view to show the detail of my score.!

!StoaScore class methodsFor: 'initialize-destroy' stamp: 'HilaireFernandes 4/4/2014 17:18'!
rescale
	super rescale.
	self current 
		stopSteppingSelfAndSubmorphs;
		startStepping.! !

!StoaScore methodsFor: 'controls bar' stamp: 'HilaireFernandes 2/1/2014 14:59'!
controlsWidget 
	^# (#homeBtn)! !

!StoaScore methodsFor: 'controls bar' stamp: 'HilaireFernandes 4/2/2014 20:46'!
homeCallback
	Stoa openAppView.! !

!StoaScore methodsFor: 'initialize' stamp: 'HilaireFernandes 2/16/2014 11:49'!
initialize 
	super initialize.
	self installScore.! !

!StoaScore methodsFor: 'initialize' stamp: 'HilaireFernandes 4/27/2014 12:43'!
installScore
	| morph awardScore|
	morph := Morph new color: self alphaDominantColor;
		useRoundedCorners;
		layoutPolicy: TableLayout new;
		layoutInset: (Stoa scale: 40);
		cellInset: (Stoa scale: 0);
		cellPositioning: #rightCenter;
		listDirection: #topToBottom.
	scoreLines := OrderedCollection new.
	awardScore := (Stoa score printPaddedWith: $0 to: 5 base: 5) reverse.
	awardScore doWithIndex: [:awardQty : rank |
		scoreLines add: (ScoreLineMorph new 	
				awardQty: awardQty digitValue; 
				rank: rank - 1; 
				yourself).
		scoreLines last hResizing: #spaceFill; vResizing: #shrinkWrap.
		morph addMorphBack: scoreLines last].
	morph addMorphBack: (totalScore := (Stoa stringMorph: 0) font: (Stoa textFontOfSize: (Stoa largeFontSize * 1.5) asInteger)).
	self addMorph: morph fullFrame: (LayoutFrame fractions: (0.05 @ 0.05 corner: 0.85 @ 0.95))	! !

!StoaScore methodsFor: 'controls bar' stamp: 'HilaireFernandes 5/21/2014 13:25'!
setControlsLayout
	controls layoutPolicy: TableLayout new;
		listDirection: #bottomToTop;
		color: Color transparent;
		listCentering: #justified;
		wrapCentering: #center;
		hResizing: #spaceFill;
		vResizing: #rigid.
! !

!StoaScore methodsFor: 'stepping and presenter' stamp: 'HilaireFernandes 2/1/2014 17:05'!
startStepping 
	super startStepping.
	scoreLines do: [ :score | score stopStepping ].
	scoreLines first startStepping.! !

!StoaScore methodsFor: 'stepping and presenter' stamp: 'HilaireFernandes 2/1/2014 17:24'!
step
	1 to: scoreLines size - 1 do: [ :index |
		((scoreLines at: index) isAnimationDone and: [(scoreLines at: index + 1) isAnimationDone not]) 
			ifTrue: [ (scoreLines at: index +1) isStepping ifFalse: [ (scoreLines at: index + 1) startStepping]]].
	totalScore contents: (scoreLines inject: 0 into: [ :subtotal :each | subtotal + each score]) printString! !

!StoaScore methodsFor: 'stepping and presenter' stamp: 'HilaireFernandes 2/1/2014 17:15'!
stepTime 
	^ 500! !

StoaAbstractView subclass: #StoaSettings
	instanceVariableNames: 'users userIndex userHolder avatarFile avatarMorph'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'iStoaXP-Core'!

!StoaSettings commentStamp: '<historical>' prior: 0!
I am view to let the user adjust the settings.

- users: the list of StoaUser instances in the system!

!StoaSettings methodsFor: 'user' stamp: 'HilaireFernandes 4/25/2014 18:41'!
addUser
	userHolder contents ifEmpty: [ ^self ].
	Stoa createUser: userHolder contents.
	users := nil.
	self changed: #userList.
	self selectedUserIndex: (self userList indexOf: userHolder contents).
	self changed: #selectedUserIndex.
	userHolder contents: ''.! !

!StoaSettings methodsFor: 'user' stamp: 'HilaireFernandes 4/25/2014 15:25'!
avatarMorph
	^ avatarMorph ifNil: [ avatarMorph := Stoa avatars collect: [ :form | form asMorph ] ]! !

!StoaSettings methodsFor: 'controls bar' stamp: 'HilaireFernandes 2/16/2014 18:17'!
controlsWidget 
	^# (#homeBtn)! !

!StoaSettings methodsFor: 'controls bar' stamp: 'HilaireFernandes 4/2/2014 20:59'!
homeCallback
	Stoa openAppView.! !

!StoaSettings methodsFor: 'initialize' stamp: 'HilaireFernandes 4/25/2014 18:38'!
initialize
	super initialize.
	avatarFile := Stoa avatarFile.
	self installSettings! !

!StoaSettings methodsFor: 'initialize' stamp: 'HilaireFernandes 5/28/2014 17:04'!
installSettings
	| morph |
	morph := self newLabelGroup: {
			'Player' translated -> (self 
				newDropListFor: (self selectedUserIndex: (self userList indexOf: Stoa username))
				list: #userList 
				getSelected: #selectedUserIndex 
				setSelected: #selectedUserIndex:
				help: 'Choose yourself from the list.' translated).
			'Avatar' translated -> (self 
				newMorphDropListFor: self
				list: #avatarMorph
				getSelected: #selectedAvatarIndex 
				setSelected: #selectedAvatarIndex:
				help: 'Choose your avatar' translated).
			'Score' translated -> (self newLabelFor: self getLabel: #score getEnabled: nil).
			'New player' translated -> self newUserMorph.
			'Voice' translated -> (self 
				newCheckboxFor: Stoa
				getSelected: #voice 
				setSelected: #voice: 
				label: '                          ' 
				help: 'Select it to activate reading of the instrusctions.' translated) }
		font:  Stoa textFont labelColor: Color black.
			
	morph color: self alphaDominantColor;
		useRoundedCorners;
		layoutInset: (Stoa scale: 40);
		cellInset: (Stoa scale: 40).						
	self addMorph: morph fullFrame: (LayoutFrame fractions: (0.05 @ 0.05 corner: 0.85 @ 0.95))	! !

!StoaSettings methodsFor: 'user' stamp: 'HilaireFernandes 5/28/2014 17:05'!
newUserMorph
	^ self newRow: { 
		(self newTextEntryFor: (userHolder := ValueHolder new contents: '') 
			getText: #contents  
			setText:  #contents:
			help: 'Write a new player name here, then press the button ''Add player''.' translated) autoAccept: true.
		self newButtonFor: self action: #addUser label: 'Add player' translated help: nil}! !

!StoaSettings methodsFor: 'user' stamp: 'HilaireFernandes 4/25/2014 15:56'!
score
	^ (self selectedUser score * 100) printString! !

!StoaSettings methodsFor: 'user' stamp: 'HilaireFernandes 4/25/2014 15:39'!
selectedAvatarIndex
	^ avatarFile indexOf: self selectedUser avatar! !

!StoaSettings methodsFor: 'user' stamp: 'HilaireFernandes 4/25/2014 18:42'!
selectedAvatarIndex: index
	self selectedUser avatar: (avatarFile at: index).
	Stoa saveUser: self selectedUser.
	Stoa loadUser: self selectedUser username.
! !

!StoaSettings methodsFor: 'user' stamp: 'HilaireFernandes 4/25/2014 18:41'!
selectedUser
	^ self users at: userIndex ! !

!StoaSettings methodsFor: 'user' stamp: 'HilaireFernandes 4/25/2014 15:03'!
selectedUserIndex
	^ userIndex! !

!StoaSettings methodsFor: 'user' stamp: 'HilaireFernandes 4/25/2014 15:37'!
selectedUserIndex: anIndex
	userIndex := anIndex.
	Stoa loadUser: (self userList at: anIndex).
	self changed: #score.
	self changed: #selectedAvatarIndex ! !

!StoaSettings methodsFor: 'controls bar' stamp: 'HilaireFernandes 5/21/2014 13:25'!
setControlsLayout
	controls layoutPolicy: TableLayout new;
		color: Color transparent;
		listDirection: #bottomToTop;
		listCentering: #justified;
		wrapCentering: #center;
		hResizing: #spaceFill;
		vResizing: #rigid.
! !

!StoaSettings methodsFor: 'user' stamp: 'HilaireFernandes 4/25/2014 18:38'!
userList
	^ self users collect: [:u | u username]! !

!StoaSettings methodsFor: 'user' stamp: 'HilaireFernandes 4/25/2014 18:40'!
users
	^ users ifNil: [ users := Stoa users sorted: [:userA :userB | userA username < userB username]]! !

!SmallInteger methodsFor: '*iStoaXP-Core' stamp: 'HilaireFernandes 1/15/2014 15:47'!
apple
	^ self token: Stoa apple! !

!SmallInteger methodsFor: '*iStoaXP-Core' stamp: 'HilaireFernandes 1/15/2014 15:47'!
banana
	^ self token: Stoa banana! !

!SmallInteger methodsFor: '*iStoaXP-Core' stamp: 'HilaireFernandes 10/24/2014 18:11'!
blackBall
	^ self token: Stoa blackBall! !

!SmallInteger methodsFor: '*iStoaXP-Core' stamp: 'HilaireFernandes 10/24/2014 16:01'!
blouse
	^ self token: Stoa blouse ! !

!SmallInteger methodsFor: '*iStoaXP-Core' stamp: 'HilaireFernandes 10/24/2014 18:12'!
blueBall
	^ self token: Stoa blueBall! !

!SmallInteger methodsFor: '*iStoaXP-Core' stamp: 'HilaireFernandes 8/11/2014 10:35'!
bread
	^ self token: Stoa bread! !

!SmallInteger methodsFor: '*iStoaXP-Core' stamp: 'HilaireFernandes 1/26/2014 16:24'!
cherry
	^ self token: Stoa cherry! !

!SmallInteger methodsFor: '*iStoaXP-Core' stamp: 'HilaireFernandes 5/27/2014 16:06'!
coffeeCup
	^ self token: Stoa coffeeCup ! !

!SmallInteger methodsFor: '*iStoaXP-Core' stamp: 'HilaireFernandes 8/11/2014 10:35'!
croissant
	^ self token: Stoa croissant! !

!SmallInteger methodsFor: '*iStoaXP-Core' stamp: 'HilaireFernandes 8/11/2014 10:36'!
cupcake
	^ self token: Stoa cupcake ! !

!SmallInteger methodsFor: '*iStoaXP-Core' stamp: 'HilaireFernandes 6/8/2014 14:55'!
fork
	^ self token: Stoa fork! !

!SmallInteger methodsFor: '*iStoaXP-Core' stamp: 'HilaireFernandes 8/11/2014 10:36'!
friedEgg
	^ self token: Stoa friedEgg! !

!SmallInteger methodsFor: '*iStoaXP-Core' stamp: 'HilaireFernandes 12/31/2013 12:27'!
glass
	^ self token: Stoa glass! !

!SmallInteger methodsFor: '*iStoaXP-Core' stamp: 'HilaireFernandes 10/24/2014 18:12'!
greenBall
	^ self token: Stoa greenBall! !

!SmallInteger methodsFor: '*iStoaXP-Core' stamp: 'HilaireFernandes 10/24/2014 16:01'!
hoodie
	^ self token: Stoa hoodie ! !

!SmallInteger methodsFor: '*iStoaXP-Core' stamp: 'HilaireFernandes 10/24/2014 16:01'!
jacket
	^ self token: Stoa jacket ! !

!SmallInteger methodsFor: '*iStoaXP-Core' stamp: 'HilaireFernandes 6/8/2014 14:55'!
knife
	^ self token: Stoa knife! !

!SmallInteger methodsFor: '*iStoaXP-Core' stamp: 'HilaireFernandes 8/11/2014 10:36'!
misoSoup
	^ self token: Stoa misoSoup! !

!SmallInteger methodsFor: '*iStoaXP-Core' stamp: 'HilaireFernandes 8/11/2014 10:36'!
muffin
	^ self token: Stoa muffin! !

!SmallInteger methodsFor: '*iStoaXP-Core' stamp: 'HilaireFernandes 1/15/2014 15:47'!
orange
	^ self token: Stoa orange! !

!SmallInteger methodsFor: '*iStoaXP-Core' stamp: 'HilaireFernandes 10/24/2014 18:12'!
orangeBall
	^ self token: Stoa orangeBall! !

!SmallInteger methodsFor: '*iStoaXP-Core' stamp: 'HilaireFernandes 1/15/2014 15:48'!
pear
	^ self token: Stoa pear! !

!SmallInteger methodsFor: '*iStoaXP-Core' stamp: 'HilaireFernandes 5/17/2014 16:16'!
person
	^ self token: Stoa person1sm! !

!SmallInteger methodsFor: '*iStoaXP-Core' stamp: 'HilaireFernandes 12/31/2013 12:27'!
plate
	^ self token: Stoa plate! !

!SmallInteger methodsFor: '*iStoaXP-Core' stamp: 'HilaireFernandes 10/24/2014 16:01'!
polo
	^ self token: Stoa polo ! !

!SmallInteger methodsFor: '*iStoaXP-Core' stamp: 'HilaireFernandes 10/24/2014 18:12'!
redBall
	^ self token: Stoa redBall! !

!SmallInteger methodsFor: '*iStoaXP-Core' stamp: 'HilaireFernandes 5/27/2014 16:06'!
spoon
	^ self token: Stoa spoon ! !

!SmallInteger methodsFor: '*iStoaXP-Core' stamp: 'HilaireFernandes 12/11/2013 16:16'!
token: species
	| tokens |
	self < 0 ifTrue: [ self error: 'I can''t get a negative number of token'].
	tokens := Array new: self.
	1 to: self do: [:i |
		tokens at: i put: species copy].
	^ tokens! !

!SmallInteger methodsFor: '*iStoaXP-Core' stamp: 'HilaireFernandes 10/24/2014 16:01'!
tshirt
	^ self token: Stoa tshirt ! !

!SmallInteger methodsFor: '*iStoaXP-Core' stamp: 'HilaireFernandes 1/18/2014 20:39'!
walnut
	^ self token: Stoa walnut! !

!SmallInteger methodsFor: '*iStoaXP-Core' stamp: 'HilaireFernandes 10/24/2014 18:13'!
yellowBall
	^ self token: Stoa yellowBall! !

IncrementalSliderMorph subclass: #NumberSliderMorph
	instanceVariableNames: 'min max quantum numberOfDecimal buttonWidth'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'iStoaXP-Core-Widget'!

!NumberSliderMorph commentStamp: '<historical>' prior: 0!
A NumberSliderMorph is a number with two buttons to adjust its value.

You define min, max value and quantum.
You change button position when sending to me the messages:
- updateOrientation: #bottomToTop.
- updateOrientation: #leftToRight.

Example:

(NumberSliderMorph
	on: (ValueHolder new contents: 1)
	getValue: #contents 
	setValue: #contents: 
	min: 1 
	max: 9 
	quantum: 1) openInWorld.
!

!NumberSliderMorph class methodsFor: 'examples' stamp: 'HilaireFernandes 1/22/2014 10:35'!
example1
"
	self example1 openInWorld
"
	^ NumberSliderMorph value: 4 min: 1 max: 10! !

!NumberSliderMorph class methodsFor: 'examples' stamp: 'HilaireFernandes 1/22/2014 10:36'!
example2
"
	self example2 openInWorld
"
	^ NumberSliderMorph
		on: (ValueHolder new contents: 1)
		getValue: #contents 
		setValue: #contents: 
		min: 1 
		max: 9 
		quantum: 0.1! !

!NumberSliderMorph class methodsFor: 'instance creation' stamp: 'HilaireFernandes 1/22/2014 10:32'!
value: aValue min: min max: max
	^ self new 
		value: aValue min: min max: max! !

!NumberSliderMorph methodsFor: 'accessing' stamp: 'HilaireFernandes 4/26/2014 19:20'!
buttonWidth
	^ buttonWidth ifNil: [buttonWidth := (Stoa scale: 100) max: 16]! !

!NumberSliderMorph methodsFor: 'as yet unclassified' stamp: 'HilaireFernandes 8/7/2014 11:33'!
decrement
	"Decrement the value. Handle empty content"
	self value: (self value ifNil: [min]) - self quantum! !

!NumberSliderMorph methodsFor: 'accessing' stamp: 'HilaireFernandes 1/21/2014 14:14'!
font: newFont
	self sliderMorph font: newFont.
	self updateSliderOrientation ! !

!NumberSliderMorph methodsFor: 'as yet unclassified' stamp: 'HilaireFernandes 8/7/2014 11:33'!
increment
	"Decrement the value. Handle empty content"
	self value: (self value ifNil: [min]) + self quantum! !

!NumberSliderMorph methodsFor: 'initialize-release' stamp: 'HilaireFernandes 5/27/2014 17:25'!
initialize
	min := 0.
	max := 1.
	quantum := 1.
	super initialize.
	self sliderOrientation: #leftToRight.
	self hResizing: #shrinkWrap;
		vResizing: #shrinkWrap.
	self extent: self minExtent! !

!NumberSliderMorph methodsFor: 'accessing' stamp: 'HilaireFernandes 1/4/2014 13:02'!
max
	^ max! !

!NumberSliderMorph methodsFor: 'accessing' stamp: 'HilaireFernandes 1/4/2014 14:57'!
max: anObject
	max := anObject.
	self value: self value.! !

!NumberSliderMorph methodsFor: 'accessing' stamp: 'HilaireFernandes 1/4/2014 13:04'!
min
	^ min! !

!NumberSliderMorph methodsFor: 'accessing' stamp: 'HilaireFernandes 1/4/2014 14:57'!
min: anObject
	min := anObject.
	self value: self value.! !

!NumberSliderMorph methodsFor: 'initialize-release' stamp: 'HilaireFernandes 1/4/2014 15:45'!
newSliderMorph
"A morph for out number slider morph"
	|slider|
	slider := self theme builder
		newString: '0'.
	^slider! !

!NumberSliderMorph methodsFor: 'accessing' stamp: 'HilaireFernandes 1/4/2014 13:05'!
quantum
	^ quantum ! !

!NumberSliderMorph methodsFor: 'accessing' stamp: 'HilaireFernandes 1/4/2014 16:03'!
quantum: anObject
	quantum := anObject.
	numberOfDecimal := (quantum floorLog: 10) abs.
	self value: self value.! !

!NumberSliderMorph methodsFor: 'geometry' stamp: 'HilaireFernandes 5/27/2014 17:11'!
sliderOrientation: aSymbol
	"Set the layout for the new extent with #leftToRight or #bottomToTop argument."

	| butts stringExtent |
	
	butts := self buttons.
	stringExtent := self sliderMorph extent.
	aSymbol = #leftToRight	
		ifTrue: [self listDirection: #leftToRight.
				butts first
					roundedCorners: #(1 2);
					hResizing: #rigid;
					vResizing: #spaceFill;
					width: self buttonWidth;
					height: stringExtent y;
					label: (self newButtonLabel: #left ofSize: self buttonWidth).
				butts last
					roundedCorners: #(3 4);
					hResizing: #rigid;
					vResizing: #spaceFill;
					width: self buttonWidth;
					height: stringExtent y;					
					label: (self newButtonLabel: #right ofSize: self buttonWidth)]
		ifFalse: [self listDirection: #bottomToTop.
				butts first
					roundedCorners: #(2 3);
					hResizing: #spaceFill;
					vResizing: #rigid;
					height: self buttonWidth;
					width: stringExtent x;
					label: (self newButtonLabel: #bottom ofSize: self buttonWidth).
				butts last
					roundedCorners: #(1 4);
					hResizing: #spaceFill;
					vResizing: #rigid;
					height: self buttonWidth;
					width: stringExtent x;
					label: (self newButtonLabel: #top ofSize: self buttonWidth)].! !

!NumberSliderMorph methodsFor: 'geometry' stamp: 'HilaireFernandes 1/21/2014 14:14'!
updateOrientation: aSymbol
	"No automatic orientation, does nothing"! !

!NumberSliderMorph methodsFor: 'geometry' stamp: 'HilaireFernandes 1/21/2014 14:14'!
updateSliderOrientation
"Update the layout for the new extent. for example if font was changed."
	buttonWidth := nil.
	self updateOrientation: self listDirection.	
	! !

!NumberSliderMorph methodsFor: 'as yet unclassified' stamp: 'HilaireFernandes 1/4/2014 15:21'!
updateValue
	self changed: #minEnabled; changed: #maxEnabled! !

!NumberSliderMorph methodsFor: 'accessing' stamp: 'HilaireFernandes 8/7/2014 11:14'!
value
	^self getValueSelector
		ifNil: [|content| 
			content := (self sliderMorph ifNil: [^0]) contents.
			content isEmpty
				ifTrue: [ nil ] ifFalse:  [content asNumber]]
		ifNotNil: [self model contents]! !

!NumberSliderMorph methodsFor: 'accessing' stamp: 'HilaireFernandes 8/7/2014 11:04'!
value: newValue
	| val |
	newValue ifNil: [ ^ self sliderMorph contents: '' ].
	val := newValue.
	quantum ifNotNil: [val := val roundTo: quantum].
	numberOfDecimal ifNotNil: [val := val round: numberOfDecimal].
	val := (max <= min)
		ifTrue: [0]
		ifFalse: [val min: max max: min].
	self sliderMorph contents: val asString.
	self model ifNotNil: [
		self setValueSelector ifNotNil: [
			self model perform: self setValueSelector with: val]].
	self updateValue! !

!NumberSliderMorph methodsFor: 'initialize-release' stamp: 'HilaireFernandes 1/22/2014 10:32'!
value: aValue min: minValue max: maxValue
	min := minValue.
	max := maxValue.
	self value: aValue! !

ImageMorph subclass: #DecoratedMorph
	instanceVariableNames: 'points positionning'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'iStoaXP-Core-Widget'!

!DecoratedMorph commentStamp: '<historical>' prior: 0!
I come with a form giving me my overall shape and a set of morphs posititionned at determined positions (my decoration). These positions are not distingished but considered equivalent.

- points: a dictionary, keys are positions of my decoration relatively to my top left corner, values are boolean indicating with true when the position is free!

!DecoratedMorph methodsFor: 'accessing' stamp: 'HilaireFernandes 1/29/2014 10:22'!
add: aMorph 
	| position |
	[position := self freePositions first]
		on: Error
		do: [position := nil].
	position ifNotNil: [
		self position: position free: false.
		self addMorph: aMorph at: self topLeft + position].
	^ position! !

!DecoratedMorph methodsFor: 'accessing' stamp: 'HilaireFernandes 1/18/2014 11:42'!
addAll: aMorphCollection
	aMorphCollection do: [:aMorph | self add: aMorph ]
	! !

!DecoratedMorph methodsFor: 'private' stamp: 'HilaireFernandes 9/1/2015 08:28'!
addMorph: aMorph at: position
	self addMorph: aMorph.
	aMorph beSticky.
	positionning == #center ifTrue: [^ aMorph center: position].
	positionning == #topLeft ifTrue: [^ aMorph topLeft: position].
	positionning == #topRight ifTrue: [^ aMorph topRight: position].
	positionning == #bottomLeft ifTrue: [^ aMorph bottomLeft: position].
	positionning == #bottomRight ifTrue: [^ aMorph bottomRight: position].
	self computeFullBounds! !

!DecoratedMorph methodsFor: 'accessing' stamp: 'HilaireFernandes 6/7/2014 12:03'!
decoratePositions: aPointCollection 
"here we define the decorations position plots"
	aPointCollection asSet size ~= aPointCollection size
		ifTrue: [self error: 'Duplicate point position.'].
	points := Dictionary new.
	aPointCollection
		do: [:aPoint | points at: aPoint put: true]! !

!DecoratedMorph methodsFor: 'as yet unclassified' stamp: 'HilaireFernandes 9/1/2015 08:27'!
extent
"some child moprh may be partially outside of the parent morphs bounds"
	^ self fullBounds extent! !

!DecoratedMorph methodsFor: 'private' stamp: 'HilaireFernandes 1/29/2014 10:11'!
freePositions
	^ (points
		select: [:assoc | assoc value]) keys shuffled ! !

!DecoratedMorph methodsFor: 'initialization' stamp: 'HilaireFernandes 1/29/2014 09:54'!
initialize 
	super initialize.
	positionning := #center! !

!DecoratedMorph methodsFor: 'testing' stamp: 'HilaireFernandes 12/1/2013 12:13'!
isFull
	^ self freePositions size = 0! !

!DecoratedMorph methodsFor: 'private' stamp: 'HilaireFernandes 1/29/2014 10:22'!
position: aPosition free: boolean
	points at: aPosition put: boolean.! !

!DecoratedMorph methodsFor: 'accessing' stamp: 'HilaireFernandes 1/29/2014 09:45'!
positionning
	^ positionning ifNil: [ positionning := #center ]! !

!DecoratedMorph methodsFor: 'accessing' stamp: 'HilaireFernandes 6/7/2014 12:02'!
positionning: aSymbol 
" #center #topLeft #topRight #bottomRight #bottomLeft"
"Here we tell how we want the decoration to positionned to its position plot"


	positionning := aSymbol! !

!DecoratedMorph methodsFor: 'accessing' stamp: 'HilaireFernandes 1/29/2014 11:35'!
removeAllDecorations
	points associationsDo: [:assoc | assoc value: true ].
	self removeAllMorphs.! !

DecoratedMorph subclass: #DecoratedSymbolicMorph
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'iStoaXP-Core-Widget'!

!DecoratedSymbolicMorph commentStamp: 'HilaireFernandes 6/7/2014 12:03' prior: 0!
As my parent I am decorated with morphs, at given positions relative to me.
However, each of these positions are associated with a symbol so I can decide exactly where a given decoration should fit.

- points: a dictionary, keys are symbols and values are associations between decoration positions and booleans indicating with true when the position is free!

!DecoratedSymbolicMorph methodsFor: 'accessing' stamp: 'HilaireFernandes 1/29/2014 14:45'!
addMorph: aMorph at: symbolOrPoint
	| positionAssoc |
	symbolOrPoint isPoint ifTrue: [^ super addMorph: aMorph at: symbolOrPoint].
	positionAssoc := points at: symbolOrPoint.
	positionAssoc value ifFalse: [ self error: 'The position ', symbolOrPoint, ' is busy.'].
	positionAssoc value: false. "position is now occupied"
	super addMorph: aMorph at: self topLeft + positionAssoc key.
	^ positionAssoc key! !

!DecoratedSymbolicMorph methodsFor: 'accessing' stamp: 'HilaireFernandes 1/29/2014 10:07'!
decoratePositions: assocCollection 
	points := Dictionary new.
	assocCollection 
		do: [:assoc | points at: assoc key put: (assoc value ->true)]! !

!DecoratedSymbolicMorph methodsFor: 'accessing' stamp: 'HilaireFernandes 1/29/2014 10:13'!
freePositions
	^ (points select: [:assoc |
		assoc value value]) values asDictionary keys shuffled! !

!DecoratedSymbolicMorph methodsFor: 'accessing' stamp: 'HilaireFernandes 1/29/2014 10:21'!
position: aPosition free: boolean
	points do: [ :assoc |
		assoc key = aPosition ifTrue: [ assoc value: boolean ] ]! !

!DecoratedSymbolicMorph methodsFor: 'as yet unclassified' stamp: 'HilaireFernandes 1/29/2014 11:36'!
removeAllDecorations
	points associationsDo: [:assoc | assoc value value: true ].
	self removeAllMorphs.! !

ImageMorph subclass: #DecoratedNumberSliderMorph
	instanceVariableNames: 'slider'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'iStoaXP-Core-Widget'!

!DecoratedNumberSliderMorph commentStamp: '<historical>' prior: 0!
A number slider morph decorated above with a bitmap.!

!DecoratedNumberSliderMorph class methodsFor: 'as yet unclassified' stamp: 'HilaireFeranndes 3/23/2014 09:56'!
value: aValue min: min max: max withForm: aForm
	^ self new form: aForm;
		value: aValue
		min: min
		max: max! !

!DecoratedNumberSliderMorph methodsFor: 'as yet unclassified' stamp: 'HilaireFernandes 1/21/2014 18:12'!
hideButtons
	slider buttons do: [ :b | b hide ]! !

!DecoratedNumberSliderMorph methodsFor: 'as yet unclassified' stamp: 'HilaireFernandes 1/22/2014 14:37'!
numberSlider
	^ slider! !

!DecoratedNumberSliderMorph methodsFor: 'as yet unclassified' stamp: 'HilaireFernandes 1/21/2014 18:12'!
showButtons
	slider buttons do: [ :b | b show ]! !

!DecoratedNumberSliderMorph methodsFor: 'evaluating' stamp: 'HilaireFernandes 1/21/2014 15:14'!
value
	^ slider value! !

!DecoratedNumberSliderMorph methodsFor: 'evaluating' stamp: 'HilaireFernandes 1/22/2014 11:14'!
value: aValue
	^ slider value: aValue! !

!DecoratedNumberSliderMorph methodsFor: 'initialization' stamp: 'HilaireFernandes 1/21/2014 20:58'!
value: aValue min: min max: max
	slider := NumberSliderMorph value: aValue min: min max: max.
	slider font: Stoa textFont;
		sliderOrientation: #topToBottom.
	slider extent: slider minExtent;
		width: (image width * 0.8) rounded.
	self addMorph: slider.
	slider center: self center.! !

!ImageMorph methodsFor: '*iStoaXP-Core-Widget' stamp: 'HilaireFeranndes 3/23/2014 09:51'!
asStateMorph
	^ ImageStateMorph new form: self form! !

ImageMorph subclass: #ImageStateMorph
	instanceVariableNames: 'states'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'iStoaXP-Core-Widget'!

!ImageStateMorph commentStamp: 'HilaireFeranndes 3/23/2014 10:01' prior: 0!
An image optionally illusatrated in its whole extent with: 
 - a surrounding border O (#selected)
 - a crossed X on the whole extent (#removed)
 - none (#none)

(ImageStateMorph new form: ThemeIcons current volumeIcon) openInWorld

ThemeIcons current warningIcon asStateMorph openInWorld !

!ImageStateMorph methodsFor: 'event handling' stamp: 'HilaireFernandes 1/5/2014 15:21'!
clicked: event in: me
	self nextState.
	self changed! !

!ImageStateMorph methodsFor: 'drawing' stamp: 'HilaireFernandes 1/26/2014 15:09'!
drawOn: aCanvas
	| state |
	super drawOn: aCanvas.
	state := self state.
	state == #bordered ifTrue: [
		aCanvas frameRectangle: (self bounds insetBy: 2) width: 3 color: Stoa stateColor].
	state == #crossed ifTrue: [
		aCanvas line: self topLeft + 8 to: self bottomRight - 8 width: 3 color: Stoa stateColor.
		aCanvas line: self topRight - (8@ -8) to: self bottomLeft - (-8@8) width: 3 color: Stoa stateColor].! !

!ImageStateMorph methodsFor: 'initialize' stamp: 'HilaireFernandes 1/5/2014 15:35'!
initialize 
	super initialize.
	self setProperty: #state toValue: #none.
	self borderWidth: 4.
	self on: #click send: #clicked:in: to: self! !

!ImageStateMorph methodsFor: 'accessing' stamp: 'HilaireFernandes 1/5/2014 15:27'!
nextState
"Switch to next state"
	| index |
	index := (self states indexOf: self state) + 1.
	index > self states size ifTrue: [ index := 1 ].
	self setProperty: #state toValue: (self states at: index).
	self changed! !

!ImageStateMorph methodsFor: 'accessing' stamp: 'HilaireFernandes 1/5/2014 15:30'!
setState: aSymbol
	(self states includes: aSymbol) ifFalse: [ 
		self error: 'Unknown state: ', aSymbol].
	self setProperty: #state toValue: aSymbol.
	self changed
! !

!ImageStateMorph methodsFor: 'accessing' stamp: 'HilaireFernandes 1/5/2014 15:08'!
state
	^  self valueOfProperty: #state
! !

!ImageStateMorph methodsFor: 'accessing' stamp: 'HilaireFernandes 7/24/2014 17:59'!
states
	^ states ifNil: [states := #(#none #bordered #crossed)]! !

!ImageStateMorph methodsFor: 'accessing' stamp: 'HilaireFernandes 7/24/2014 18:03'!
states: collectionOfStates
	states := collectionOfStates ! !

BorderedMorph subclass: #DiceMorph
	instanceVariableNames: 'face dotColor diceValue'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'iStoaXP-Core-Widget'!

!DiceMorph commentStamp: '<historical>' prior: 0!
Widget pour simuler un dé à jouer.!

!DiceMorph class methodsFor: 'instance creation' stamp: 'HilaireFernandes 7/24/2014 16:07'!
example
	"
	DiceMorph example openInWorld
	"
	^ (self face: 6) diceValue: 6 atRandom! !

!DiceMorph class methodsFor: 'instance creation' stamp: 'HilaireFernandes 8/7/2006 15:54'!
face: aNumber
	^self new face: aNumber! !

!DiceMorph methodsFor: 'accessing' stamp: 'HilaireFernandes 5/26/2010 18:23'!
color: aColor
	color := aColor.
	self fillStyle 
		colorRamp: {0.0->color darker darker darker darker darker. 1.0->color}.
	self changed
! !

!DiceMorph methodsFor: 'accessing' stamp: 'HilaireFernandes 7/24/2014 16:06'!
diceValue
	"Answer the value of diceValue"

	^ diceValue! !

!DiceMorph methodsFor: 'accessing' stamp: 'HilaireFernandes 7/24/2014 16:06'!
diceValue: anObject 
	(anObject isInteger and: [anObject >= 1 and: [anObject <= face]]) 
		ifTrue: 
			[diceValue := anObject.
			self changed]! !

!DiceMorph methodsFor: 'accessing' stamp: 'HilaireFernandes 5/26/2010 18:24'!
dotColor
	^dotColor! !

!DiceMorph methodsFor: 'accessing' stamp: 'HilaireFernandes 5/26/2010 18:27'!
dotColor: aColor
	dotColor := aColor.
	self changed! !

!DiceMorph methodsFor: 'drawing' stamp: 'HilaireFernandes 7/24/2014 13:05'!
drawDotOn: aCanvas at: aPoint
	aCanvas 
		fillOval: (Rectangle 
			center: (self position + (self extent * aPoint)) 
			extent: self extent / 6) 
		color: self dotColor! !

!DiceMorph methodsFor: 'drawing' stamp: 'HilaireFernandes 7/24/2014 16:06'!
drawOn: aCanvas 
	| aaCanvas |
	aaCanvas := aCanvas asBalloonCanvas aaLevel: 4.
	super drawOn: aCanvas.
	(self perform: ('face' , diceValue asString) asSymbol) 
		do: [:aPoint | self drawDotOn: aCanvas at: aPoint]! !

!DiceMorph methodsFor: 'accessing' stamp: 'HilaireFernandes 8/7/2006 15:54'!
face
	"Answer the value of face"

	^ face! !

!DiceMorph methodsFor: 'dot-position' stamp: 'HilaireFernandes 8/7/2006 16:35'!
face1
	^{0.5@0.5}! !

!DiceMorph methodsFor: 'dot-position' stamp: 'HilaireFernandes 8/7/2006 17:00'!
face2
	^{0.25@0.25 . 0.75@0.75}! !

!DiceMorph methodsFor: 'dot-position' stamp: 'HilaireFernandes 8/7/2006 16:59'!
face3
	^{0.25@0.25 . 0.75@0.75 . 0.5@0.5}! !

!DiceMorph methodsFor: 'dot-position' stamp: 'HilaireFernandes 8/7/2006 17:00'!
face4
	^{0.25@0.25 . 0.75@0.25 . 0.75@0.75 . 0.25@0.75}! !

!DiceMorph methodsFor: 'dot-position' stamp: 'HilaireFernandes 8/7/2006 17:00'!
face5
	^{0.25@0.25 . 0.75@0.25 . 0.75@0.75 . 0.25@0.75 . 0.5@0.5}! !

!DiceMorph methodsFor: 'dot-position' stamp: 'HilaireFernandes 8/7/2006 17:01'!
face6
	^{0.25@0.25 . 0.75@0.25 . 0.75@0.75 . 0.25@0.75 . 0.25@0.5 . 0.75@0.5}! !

!DiceMorph methodsFor: 'dot-position' stamp: 'HilaireFernandes 8/7/2006 17:02'!
face7
	^{0.25@0.25 . 0.75@0.25 . 0.75@0.75 . 0.25@0.75 . 0.25@0.5 . 0.75@0.5 . 0.5@0.5}! !

!DiceMorph methodsFor: 'dot-position' stamp: 'HilaireFernandes 8/7/2006 17:03'!
face8
	^{0.25@0.25 . 0.75@0.25 . 0.75@0.75 . 0.25@0.75 . 0.25@0.5 . 0.75@0.5 . 0.5@0.5 . 0.5@0.25}! !

!DiceMorph methodsFor: 'dot-position' stamp: 'HilaireFernandes 8/7/2006 17:03'!
face9
	^{0.25@0.25 . 0.75@0.25 . 0.75@0.75 . 0.25@0.75 . 0.25@0.5 . 0.75@0.5 . 0.5@0.5 . 0.5@0.25 . 0.5@0.75}! !

!DiceMorph methodsFor: 'accessing' stamp: 'HilaireFernandes 7/24/2014 13:00'!
face: anObject
	"Set the number of face"
	(anObject isInteger and: [anObject >= 1and: [anObject <= 9]])
		ifTrue: [face := anObject asInteger]! !

!DiceMorph methodsFor: 'initialization' stamp: 'HilaireFernandes 7/24/2014 16:06'!
initialize
	super initialize.
	self extent: 50 @ 50.
	self
		useGradientFill;
		borderWidth: 2;
		useRoundedCorners.
	self setBorderStyle: #complexInset.
	self fillStyle direction: self extent.
	self color: Color green.
	self dotColor: Color black.
	self face: 6.
	self diceValue: 1.
! !

!MorphicUIManager methodsFor: '*iStoaXP-Core' stamp: 'HilaireFernandes 4/4/2014 17:34'!
checkForNewDisplaySize

	"Check whether the screen size has changed and if so take appropriate actions"

	Display extent = DisplayScreen actualScreenSize ifTrue: [^ Display].
	World restoreMorphicDisplay.
	Stoa rescaleView! !

!Form methodsFor: '*iStoaXP-Core-Widget' stamp: 'HilaireFeranndes 3/23/2014 09:55'!
asDecoratedMorph
	^DecoratedMorph new form: self! !

!Form methodsFor: '*iStoaXP-Core-Widget' stamp: 'HilaireFernandes 1/12/2014 19:11'!
asMorph: itemType
	^ self asMorph setProperty: #itemType toValue:  itemType! !

!Form methodsFor: '*iStoaXP-Core-Widget' stamp: 'HilaireFeranndes 3/23/2014 09:55'!
asStateMorph
	^ImageStateMorph new form: self! !

Object subclass: #Etayage
	instanceVariableNames: 'question steps currentStep'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'iStoaXP-Core'!

!Etayage commentStamp: '<historical>' prior: 0!
An Etayage is a set of board games arranged with a pedagogical intention (i.e. a set of EtayageStep)

We have the following models:
- An Etayage model to describe a set of activities in Etayage Steps.
- An Etayage Step model per Board model. The Etayage Step model hold the parameter for the Board model
- A Board model to describe one given board game




Instance Variables
- steps a collection of EtayageStep instances
- currentStep, the index in the steps collection of the played EtayageStep !

!Etayage class methodsFor: 'accessing' stamp: 'HilaireFernandes 5/19/2014 22:42'!
background
	^ Stoa getBackground: 'background1.jpg'! !

!Etayage class methodsFor: 'accessing' stamp: 'HilaireFernandes 12/15/2013 12:23'!
description
 ""
	self subclassResponsibility! !

!Etayage class methodsFor: 'accessing' stamp: 'HilaireFernandes 7/24/2014 14:03'!
icon
"Return an ImageMorph illustrating this Etayage.
It is search in the Cliparts fodler, then the etayage resources at path"
| icon |
	icon := self iconNameOrMorph.
	^ icon isMorph 
		ifTrue: [ | factor |
			factor := (Stoa scale: self iconHeight) / icon height. 
			icon extent: (icon extent * factor).
			icon imageForm asMorph "ensure it is not rescaled unproportionnally when fitted in the preview sticker"]
		ifFalse: [(Stoa getClipart: self iconNameOrMorph height: self iconHeight) asMorph]! !

!Etayage class methodsFor: 'accessing' stamp: 'HilaireFernandes 5/18/2014 11:54'!
iconHeight
"the prefered icon height"
	^ 200! !

!Etayage class methodsFor: 'accessing' stamp: 'HilaireFernandes 5/18/2014 11:55'!
iconNameOrMorph
"Return an ImageMorph illustrating this Etayage.
It is search in the Cliparts fodler, then the etayage resources at path"
	^ Path * 'misc' / 'exerciseBook.png'! !

!Etayage class methodsFor: 'accessing' stamp: 'HilaireFernandes 12/15/2013 12:25'!
myCategory
"A translated string, category the etayage belongs to"
	self subclassResponsibility! !

!Etayage class methodsFor: 'accessing' stamp: 'HilaireFernandes 12/18/2013 09:47'!
path
"A relative path (to Plateform etayagePath) to specific resources for this etayage"
	^ nil! !

!Etayage class methodsFor: 'debugging' stamp: 'HilaireFernandes 1/15/2014 11:15'!
playStep: integer
"An helper method to test etayage step"
	self new playStep: integer.
! !

!Etayage class methodsFor: 'accessing' stamp: 'HilaireFernandes 5/28/2014 17:32'!
preview
	| preview textMorph description score|
	score := self score.
	preview := Morph new extent: (Stoa scale: 700 @ 600); color: Color transparent; clipSubmorphs: true.
	preview layoutPolicy: ProportionalLayout new.
	preview 
		addMorph: self icon 
		fullFrame: (LayoutFrame fractions: (0 @ 0 corner: 1 @ 0.4)).
	description := ReadWriteStream on: String new.
	Stoa develMode ifTrue: [description nextPutAll: self name; nextPutAll: ' :: '].
	description	nextPutAll: self description;
		nextPut: Character cr;
		nextPutAll: 'Score: ' translated;
		nextPutAll: (Stoa scoreAt: self) printString;
		nextPut: $/;
		nextPutAll: score printString.
	textMorph := description contents asTextMorph width: preview width; wrapFlag: true; lock: true.
	preview 
		addMorph: textMorph
		fullFrame: (LayoutFrame fractions: (0 @ 0.4 corner: 1 @ 1)).
	score = (Stoa scoreAt: self) ifTrue: [ 
		preview addMorph: ((Stoa getClipart: Path * 'awards' / 'starWinner.png') asMorph position: 0@0)].
	^ preview! !

!Etayage class methodsFor: 'accessing' stamp: 'HilaireFernandes 12/21/2013 12:35'!
score
"Return the maximum gains for this etayage, simply based in the number of steps"
	| stepNumber |
	stepNumber := 1.
	[self canUnderstand: ('instanciateStep' , stepNumber asString) asSymbol]
		whileTrue: [	stepNumber := stepNumber + 1].
	^ stepNumber - 1! !

!Etayage class methodsFor: 'helpers' stamp: 'HilaireFernandes 8/7/2014 09:50'!
textPreview: aString
| morph string |
	string := Stoa stringMorphLarge: aString.
	morph := Morph new extent: string extent; color: Color transparent.
	morph addMorph: string.
	^ morph! !

!Etayage methodsFor: 'accessing' stamp: 'HilaireFernandes 12/3/2013 20:37'!
board
	^ self step board! !

!Etayage methodsFor: 'etayage step' stamp: 'HilaireFernandes 12/30/2013 22:36'!
continueStep
<todo> "Should be named tryAgainStep"
! !

!Etayage methodsFor: 'etayage step' stamp: 'HilaireFernandes 12/3/2013 20:29'!
currentStep
	^ currentStep ! !

!Etayage methodsFor: 'initialize-release' stamp: 'HilaireFernandes 12/30/2013 21:06'!
etayageFinished
	self triggerEvent: #finishedEtayageEvt.! !

!Etayage methodsFor: 'initialize-release' stamp: 'HilaireFernandes 12/12/2013 14:14'!
initialize
	super initialize.
	self instanciateSteps ! !

!Etayage methodsFor: 'etayage step' stamp: 'HilaireFernandes 12/3/2013 20:26'!
instanciateSteps
	| stepNumber |
	steps := OrderedCollection new.
	currentStep := 1.
	stepNumber := 1.
	[self class canUnderstand: ('instanciateStep' , stepNumber asString) asSymbol]
		whileTrue: [steps
				add: (self perform: ('instanciateStep' , stepNumber asString) asSymbol).
			stepNumber := stepNumber + 1]! !

!Etayage methodsFor: 'accessing' stamp: 'HilaireFernandes 12/30/2013 14:59'!
maxScore
	^ self class score! !

!Etayage methodsFor: 'helpers' stamp: 'HilaireFernandes 4/24/2014 10:00'!
message: aString
	"StoaPlay current message: aString"
	Stoa mascot: Stoa postman message: aString ! !

!Etayage methodsFor: 'etayage step' stamp: 'HilaireFernandes 12/30/2013 17:08'!
nextStep
	currentStep < self stepNumber 
		ifTrue: [
			currentStep := currentStep + 1.
			self playStep]
		ifFalse: [self etayageFinished]! !

!Etayage methodsFor: 'etayage step' stamp: 'HilaireFernandes 12/3/2013 20:54'!
pause
	"Make a pause on the board game"
	self board pause ! !

!Etayage methodsFor: 'etayage step' stamp: 'HilaireFernandes 12/30/2013 16:58'!
playStep
<todo>
	self releaseBoard.
	self triggerEvent: #etayageStepReadyToStartEvt.
	^ self step start! !

!Etayage methodsFor: 'debugging' stamp: 'HilaireFernandes 2/15/2014 14:44'!
playStep: integer
"An helper method to test etayage step"
	(integer between: 1 and: self stepNumber) ifFalse: 
		[self error: 'Number argument must belong to [1 ; ', self stepNumber printString, '].'].
	currentStep := integer asInteger.
	steps 
		at: currentStep 
		put: (self perform: ('instanciateStep' , currentStep asString) asSymbol).
	"Display the playMorph before plugging the etayage for a propery layout"
	StoaPlay current show; openInWorld.
	StoaPlay current etayage: self.
! !

!Etayage methodsFor: 'initialize-release' stamp: 'HilaireFernandes 12/12/2013 14:18'!
release
	super release.
	self releaseBoard ! !

!Etayage methodsFor: 'initialize-release' stamp: 'HilaireFernandes 12/30/2013 21:59'!
releaseBoard
	self step ifNotNilDo: [:aStep | aStep releaseBoard]! !

!Etayage methodsFor: 'etayage step' stamp: 'HilaireFernandes 12/3/2013 20:38'!
replayStep
	"regénérer les argument de l'exericce, sans reinitialiser le pas d'étayage
	courant"

	self releaseBoard.
	self step 
		arguments: (self 
				perform: ('instanciateStep' , currentStep asString) asSymbol) arguments.
	self playStep! !

!Etayage methodsFor: 'accessing' stamp: 'HilaireFernandes 12/30/2013 14:58'!
score
"Compute it"
	^ (self steps inject: 0 into: [ :subScore :step|
		subScore + step userAchievement ]) truncated! !

!Etayage methodsFor: 'etayage step' stamp: 'HilaireFernandes 12/30/2013 21:59'!
step
	"the played etayage step"
	^ steps at: currentStep ifAbsent: [ nil ]! !

!Etayage methodsFor: 'success-error' stamp: 'HilaireFernandes 7/28/2014 00:44'!
stepErrorAnalysis
	"By default only two failures before the system
	shows the solution to the exercise"
	self step failures < 3
		ifTrue: 
			[self board numberOfWrongAnswers > 1
				ifTrue: [Stoa mascotMessageOnErrors]
				ifFalse: [Stoa mascotMessageOnError].
			self continueStep]
		ifFalse: 
			[Stoa mascotMessageOnRepeatedErrors.
			self step showSolution.
			Stoa mascot: Stoa postman message: 'Ready to continue?' translated final: true.
			self nextStep]! !

!Etayage methodsFor: 'success-error' stamp: 'HilaireFernandes 5/28/2014 16:46'!
stepErrorAnalysisTimeOut
	"By default only two failures before the system
	shows the solution to the exercise"
	self message: 'You ran out of time!!' translated.
	self step failures < 3
		ifTrue: [Stoa mascot: Stoa mascotOrangeFlag message: 'I''ll give you a bit more time.
Have another go to make it!!' translated.
			self continueStep]
		
		ifFalse: [Stoa mascot: Stoa mascotRedFlag message: 'I highlighted your errors in red and
I will show your the right answers.' translated final: true.
			self step showSolution.
			self message: 'Ready to continue?' translated.
			self nextStep]! !

!Etayage methodsFor: 'success-error' stamp: 'HilaireFernandes 12/30/2013 16:40'!
stepFailedWhatNext
	self step fail.
	^ self step isTimeLimitReached
		ifTrue: [self stepErrorAnalysisTimeOut]
		ifFalse: [ self stepErrorAnalysis]! !

!Etayage methodsFor: 'etayage step' stamp: 'HilaireFernandes 12/3/2013 20:27'!
stepNumber
"The number of steps in this etayage"
	^steps size! !

!Etayage methodsFor: 'success-error' stamp: 'HilaireFernandes 12/30/2013 22:29'!
stepPreAnalysis
"Step was done, (user cklicked -done- or overtime, what do we do from now is:"
"Return true if the user continue with this etayage, false when it is done (all correct, too many mistakes,...)"
	self step showErrors.
	self step isSuccess 
		ifTrue: [self stepSucceedWhatNext]
		ifFalse: [self stepFailedWhatNext].! !

!Etayage methodsFor: 'success-error' stamp: 'HilaireFernandes 4/24/2014 10:10'!
stepSucceedWhatNext
<todo>
	"We should have rings and bells"
	"the learner succeed with the latest exercise, by default we just go
	to the next step"

	self step succeed.
	Stoa mascotMessageOnSuccess.
	self nextStep
	! !

!Etayage methodsFor: 'etayage step' stamp: 'HilaireFernandes 12/3/2013 20:27'!
steps
"access all the etayage steps of this etayage"
	^steps! !

!Etayage methodsFor: 'etayage step' stamp: 'HilaireFernandes 12/3/2013 20:55'!
unpause
	"Resume the activity on the board game"
	self board unpause ! !

Object subclass: #EtayageStep
	instanceVariableNames: 'boardClass board method arguments timeLimit successes failures'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'iStoaXP-Core'!

!EtayageStep commentStamp: '<historical>' prior: 0!
An EtayageStep is the parameterisation of a board game.

Instance Variables
 - method: message to instantiate the board game
 - arguments: a collection of arguments to pass to the board game method 
 - timeLimit: if <> 0 the board game is limited in time 
 - board: the instantiated board game!

!EtayageStep class methodsFor: 'instance creation' stamp: 'HilaireFernandes 12/30/2013 12:24'!
board: aClass method: aSymbol arguments: aCollection
	^ self board: aClass
		 method: aSymbol
		 arguments: aCollection
		 timeLimit: 0! !

!EtayageStep class methodsFor: 'instance creation' stamp: 'HilaireFernandes 12/15/2013 12:00'!
board: aClass method: aSymbol arguments: aCollection timeLimit: anInteger
	^ self basicNew boardClass: aClass;
		 method: aSymbol;
		 arguments: aCollection;
		 timeLimit: anInteger;
		 initialize! !

!EtayageStep methodsFor: 'accessing' stamp: 'HilaireFernandes 12/30/2013 22:18'!
arguments
	^ arguments! !

!EtayageStep methodsFor: 'accessing' stamp: 'HilaireFernandes 12/11/2013 22:06'!
arguments: aCollection
	arguments := aCollection! !

!EtayageStep methodsFor: 'accessing' stamp: 'HilaireFernandes 12/3/2013 20:49'!
board
	^ board ifNil: [ board := self instantiateBoard ]! !

!EtayageStep methodsFor: 'accessing' stamp: 'HilaireFernandes 12/3/2013 20:36'!
board: anObject
	board := anObject! !

!EtayageStep methodsFor: 'accessing' stamp: 'HilaireFernandes 12/3/2013 20:43'!
boardClass
	^ boardClass! !

!EtayageStep methodsFor: 'accessing' stamp: 'HilaireFernandes 12/3/2013 20:43'!
boardClass: anObject
	boardClass := anObject! !

!EtayageStep methodsFor: 'board manipulation' stamp: 'HilaireFernandes 12/13/2013 18:46'!
continueStep
	self board restartChronograph! !

!EtayageStep methodsFor: 'score' stamp: 'HilaireFernandes 12/13/2013 21:25'!
fail
	^ failures := failures + 1! !

!EtayageStep methodsFor: 'score' stamp: 'HilaireFernandes 12/13/2013 08:39'!
failures
	^ failures! !

!EtayageStep methodsFor: 'initialize-release' stamp: 'HilaireFernandes 12/13/2013 08:41'!
initialize 
	super initialize.
	successes := failures := 0 ! !

!EtayageStep methodsFor: 'board manipulation' stamp: 'HilaireFernandes 12/30/2013 22:27'!
instantiateBoard
	^ self boardClass perform: self method withArguments: self arguments! !

!EtayageStep methodsFor: 'testing' stamp: 'HilaireFernandes 12/3/2013 20:50'!
isSuccess
	^ self board isSuccess! !

!EtayageStep methodsFor: 'testing' stamp: 'HilaireFernandes 2/15/2014 14:44'!
isTimeLimitReached
	^ StoaPlay current isTimeLimitReached ! !

!EtayageStep methodsFor: 'testing' stamp: 'HilaireFernandes 12/30/2013 12:23'!
isTimeLimited
	^ timeLimit  ~= 0! !

!EtayageStep methodsFor: 'accessing' stamp: 'HilaireFernandes 12/30/2013 22:18'!
method
	^ method! !

!EtayageStep methodsFor: 'accessing' stamp: 'HilaireFernandes 12/11/2013 22:01'!
method: aSymbol
	method := aSymbol! !

!EtayageStep methodsFor: 'accessing' stamp: 'HilaireFernandes 1/15/2014 11:47'!
releaseBoard
	board ifNotNil: [
		board release.
		board :=nil ]! !

!EtayageStep methodsFor: 'board manipulation' stamp: 'HilaireFernandes 12/13/2013 21:11'!
showErrors
	^ self board showErrors! !

!EtayageStep methodsFor: 'board manipulation' stamp: 'HilaireFernandes 12/13/2013 21:12'!
showSolution
	^ self board showSolution! !

!EtayageStep methodsFor: 'board manipulation' stamp: 'HilaireFernandes 12/3/2013 20:49'!
start
	self board start! !

!EtayageStep methodsFor: 'score' stamp: 'HilaireFernandes 12/13/2013 21:26'!
succeed
	^ successes := successes + 1! !

!EtayageStep methodsFor: 'score' stamp: 'HilaireFernandes 1/11/2014 21:21'!
successModeration
"A value between 0 and 1 depending on the number of tries and success"
	^[successes asFloat / self tries] 
		on: ZeroDivide 
		do: [0].! !

!EtayageStep methodsFor: 'score' stamp: 'HilaireFernandes 12/13/2013 08:40'!
successes
	^ successes ! !

!EtayageStep methodsFor: 'accessing' stamp: 'HilaireFernandes 12/30/2013 22:26'!
timeLimit
	^ timeLimit! !

!EtayageStep methodsFor: 'accessing' stamp: 'HilaireFernandes 12/11/2013 22:07'!
timeLimit: integer 
	timeLimit := integer! !

!EtayageStep methodsFor: 'score' stamp: 'HilaireFernandes 12/13/2013 08:41'!
tries
	^ successes + failures! !

!EtayageStep methodsFor: 'score' stamp: 'HilaireFernandes 1/11/2014 21:21'!
userAchievement
"The success rate depend on the rationSuccess and the number of tries"
	^ self board ratioSuccess * self successModeration! !

!Object methodsFor: '*iStoaXP-Core-Match' stamp: 'HilaireFernandes 1/11/2014 18:41'!
asMatchItem
	^ MatchItemMorph from: self! !

!Object methodsFor: '*iStoaXP-Core-Match' stamp: 'HilaireFernandes 1/11/2014 18:41'!
asMatchItemBlock
	^ MatchItemBlockMorph from: self! !

!Object methodsFor: '*iStoaXP-Core-Match' stamp: 'HilaireFernandes 1/11/2014 18:41'!
asMatchTarget
	^ MatchTargetMorph from: self! !

!Object methodsFor: '*iStoaXP-Core-Match' stamp: 'HilaireFernandes 1/11/2014 18:41'!
asMatchTargetBlock
	^ MatchTargetBlockMorph from: self! !

Object subclass: #Stoa
	instanceVariableNames: ''
	classVariableNames: 'ActiveView DefaultFontSize DevelMode MyDisplayExtent Platform Scale User Voice'
	poolDictionaries: ''
	category: 'iStoaXP-Core'!

!Stoa commentStamp: 'HilaireFernandes 4/25/2014 15:09' prior: 0!
A place to hold the default settings and to access the data of the iStoa application, depending (or not) on the underneath system.!

!Stoa class methodsFor: 'system startup' stamp: 'HilaireFernandes 5/29/2014 09:03'!
addStartUp
	Smalltalk addToStartUpList: self! !

!Stoa class methodsFor: 'graphics - fruits' stamp: 'HilaireFernandes 1/18/2014 09:59'!
apple
	^ (Stoa getClipart: Path * 'fruits' / 'apple.png') asMorph: #apple.
! !

!Stoa class methodsFor: 'graphics' stamp: 'HilaireFernandes 1/29/2014 14:28'!
arrow
	^ (Stoa getClipart: Path * 'misc' / 'arrow.png') asMorph: #arrow.
! !

!Stoa class methodsFor: 'accessing' stamp: 'HilaireFernandes 4/25/2014 15:22'!
avatarFile
	^ (Platform avatarsPath filesMatching: '*.png') collect: [ :aFile |
		 aFile basename]! !

!Stoa class methodsFor: 'graphics' stamp: 'HilaireFernandes 4/25/2014 14:56'!
avatars
	^ (Platform avatarsPath filesMatching: '*.png') collect: [ :aFile |
		self getClipart: aFile ]! !

!Stoa class methodsFor: 'path' stamp: 'HilaireFernandes 5/5/2014 19:09'!
avatarsPath
	^ Platform avatarsPath ! !

!Stoa class methodsFor: 'graphics decorated' stamp: 'HilaireFernandes 5/17/2014 15:39'!
bag
"Decoration must be 64x64"
	| bag |
	bag := (Stoa getClipart: Path * 'misc' / 'bag.png') asDecoratedMorph.
	bag decoratePositions: (Stoa scale: {219@145. 292@145. 145@212. 219@212. 292@212. 77@282. 145@282. 219@282. 292@282. 348@282. 77@347. 145@347. 219@347. 292@347. 348@347. 180@405. 250@405}).
	^ bag! !

!Stoa class methodsFor: 'graphics - fruits' stamp: 'HilaireFernandes 1/18/2014 09:59'!
banana
	^ (Stoa getClipart: Path * 'fruits' / 'banana.png') asMorph: #banana.
! !

!Stoa class methodsFor: 'class initialization' stamp: 'HilaireFernandes 12/5/2013 14:06'!
beUnix
	" StoaSettings beUnix"
	Platform := StoaUnixPlatform new.! !

!Stoa class methodsFor: 'class initialization' stamp: 'HilaireFernandes 12/5/2013 14:06'!
beWin32
	" StoaSettings beWin32"
	Platform := StoaWin32Platform new.! !

!Stoa class methodsFor: 'graphics - token' stamp: 'HilaireFernandes 10/24/2014 18:11'!
blackBall
	^ (Stoa getClipart: Path * 'balls' / 'blackBall.png') asMorph: #blackBall! !

!Stoa class methodsFor: 'graphics - clothes' stamp: 'HilaireFernandes 10/24/2014 15:44'!
blouse
	^ (Stoa getClipart: Path * 'clothes' / 'blouse.png') asMorph: #blouse.! !

!Stoa class methodsFor: 'graphics - token' stamp: 'HilaireFernandes 10/24/2014 18:12'!
blueBall
	^ (Stoa getClipart: Path * 'balls' / 'blueBall.png') asMorph: #blueBall! !

!Stoa class methodsFor: 'graphics' stamp: 'HilaireFernandes 7/22/2014 22:58'!
blueCube
	^ (Stoa getClipart: Path * 'shapes' / 'blueCube.png')! !

!Stoa class methodsFor: 'graphics - breakfast' stamp: 'HilaireFernandes 8/11/2014 11:25'!
bread
	^ (Stoa getClipart: Path * 'breakfast' / 'bread.png' width: 64) asMorph: #bread.! !

!Stoa class methodsFor: 'graphics' stamp: 'HilaireFernandes 5/17/2014 15:55'!
cabriolet2places
	^ (Stoa getClipart: Path * 'vehicles' / 'cabriolet2places.png') asMorph! !

!Stoa class methodsFor: 'graphics decorated' stamp: 'HilaireFernandes 5/18/2014 11:45'!
cabriolet5places
"Decoration must be 64x64"
	| morph |
	morph := (Stoa getClipart: Path * 'vehicles' / 'cabriolet5places.png') asDecoratedMorph.
	morph decoratePositions: (Stoa scale: {98@134. 179@134. 79@233. 138@233. 199@233}).
	^ morph! !

!Stoa class methodsFor: 'graphics - fruits' stamp: 'HilaireFernandes 1/26/2014 16:24'!
cherry
	^ (Stoa getClipart: Path * 'fruits' / 'cherry.png') asMorph: #cherry.
! !

!Stoa class methodsFor: 'graphics decorated' stamp: 'HilaireFernandes 5/17/2014 16:52'!
cloud
"Decoration must be 64x64"
	| morph |
	morph := (Stoa getClipart: Path * 'misc' / 'decoratedCloud.png') asDecoratedMorph.
	morph decoratePositions: (Stoa scale: {249@71. 124@173. 208@173. 289@196. 84@300. 168@282. 249@300. 124@408. 208@408}).
	^ morph! !

!Stoa class methodsFor: 'graphics - breakfast' stamp: 'HilaireFernandes 5/27/2014 15:44'!
coffeeCup
	^ (Stoa getClipart: Path * 'furnitures' / 'coffeeCup.png') asMorph: #coffeeCup.
! !

!Stoa class methodsFor: 'text' stamp: 'HilaireFernandes 1/29/2014 08:12'!
color: aColor value: aValue min: min max: max
	^ DecoratedNumberSliderMorph 
		value: aValue min: min max: max
		withForm: ((Form extent: (Stoa scale: 150@253) depth: 32) fillColor: aColor)! !

!Stoa class methodsFor: 'user' stamp: 'HilaireFernandes 3/26/2014 10:49'!
createUser: aString
	User := StoaUser new username: aString.
	self saveUser! !

!Stoa class methodsFor: 'graphics - breakfast' stamp: 'HilaireFernandes 8/11/2014 11:25'!
croissant
	^ (Stoa getClipart: Path * 'breakfast' / 'croissant.png' width: 64) asMorph: #croissant.! !

!Stoa class methodsFor: 'graphics - breakfast' stamp: 'HilaireFernandes 8/11/2014 11:26'!
cupcake
	^ (Stoa getClipart: Path * 'breakfast' / 'cupcake.png' width: 64) asMorph: #cupcake.! !

!Stoa class methodsFor: 'graphics decorated' stamp: 'HilaireFernandes 8/11/2014 11:39'!
decoratedBlank10w
"Decorated blank morph with 10 decoration slots, extent is wide
 decoration extent max is 64@80
"
	| morph |
	morph := (Form extent: (Stoa scale: 320@160) depth: 32) asDecoratedMorph.
	morph decoratePositions: (Stoa scale: {32@40. 96@40. 160@40. 224@40. 288@40. 32@120. 96@120. 169@120. 224@120. 288@120}).
	^ morph! !

!Stoa class methodsFor: 'graphics decorated' stamp: 'HilaireFernandes 5/17/2014 15:16'!
decoratedBlank9
"Decorated blank morph with 9 decoration slots
 decoration must be 160 width
"
	| morph |
	morph := (Form extent: (Stoa scale: 480@600) depth: 32) asDecoratedMorph.
	morph decoratePositions: (Stoa scale: {80@100. 240@100. 400@100. 80@300. 240@300. 400@300. 80@500. 240@500. 400@500}).
	^ morph! !

!Stoa class methodsFor: 'geometry' stamp: 'HilaireFernandes 11/27/2013 18:02'!
defaultDisplayExtent
	^ 1024@768! !

!Stoa class methodsFor: 'text' stamp: 'HilaireFernandes 4/5/2014 08:57'!
defaultFontSize 
	 ^ DefaultFontSize ifNil: [ DefaultFontSize := (self scale: 36) min: 60 max: 9]! !

!Stoa class methodsFor: 'view' stamp: 'HilaireFernandes 4/4/2014 17:36'!
deleteViews
	StoaPlay delete.
	StoaSettings delete.
	StoaApp delete.
	StoaScore delete.
	ActiveView := nil.! !

!Stoa class methodsFor: 'accessing' stamp: 'HilaireFernandes 3/26/2014 09:59'!
develMode
	^ DevelMode ifNil: [ DevelMode := true ]! !

!Stoa class methodsFor: 'accessing' stamp: 'HilaireFernandes 3/26/2014 09:59'!
develMode: boolean
	DevelMode := boolean! !

!Stoa class methodsFor: 'graphics' stamp: 'HilaireFernandes 7/24/2014 16:58'!
dice
"a 6 faces dice"
	^ DiceMorph new extent: (Stoa scale: 160@160)! !

!Stoa class methodsFor: 'graphics' stamp: 'HilaireFernandes 7/24/2014 16:08'!
dice: value
"a 6 faces dice"
	^ self dice diceValue: value! !

!Stoa class methodsFor: 'graphics decorated' stamp: 'HilaireFernandes 10/25/2014 11:14'!
downArrow
"Decoration must be 64x64"
	| morph |
	morph := (Stoa getClipart: Path * 'misc' / 'downArrow.png') asDecoratedMorph.
	morph decoratePositions: (Stoa scale: {77@248. 77@178. 77@108. 77@38}).
	^ morph! !

!Stoa class methodsFor: 'graphics - breakfast' stamp: 'HilaireFernandes 6/8/2014 14:37'!
fork
	^ (Stoa getClipart: Path * 'furnitures' / 'fork.png') asMorph: #fork
! !

!Stoa class methodsFor: 'graphics - breakfast' stamp: 'HilaireFernandes 8/11/2014 11:25'!
friedEgg
	^ (Stoa getClipart: Path * 'breakfast' / 'friedEgg.png' width: 64) asMorph: #friedEgg.! !

!Stoa class methodsFor: 'accessing' stamp: 'HilaireFernandes 1/18/2014 09:54'!
getBackground: aName
"Load background and scale it accordingly to the display extent. 
For background, we scale a bit differently: we scale to fill the space, then chop the extra area, therefore we don't use the Scale attribute of the settings"
	| form factor |
	form := ImageReadWriter formFromFileNamed: (Platform backgroundsPath resolve: aName) fullName.
	factor := self myDisplayExtent x / form width max: self myDisplayExtent  y / form height.
	form := form
			magnify: form boundingBox
			by: factor
			smoothing: 1.
	form := form contentsOfArea: (0 @ 0 corner: self myDisplayExtent).
	^ form! !

!Stoa class methodsFor: 'accessing' stamp: 'HilaireFernandes 1/26/2014 16:21'!
getClipart: aRelativePathName
"Load the clipart and scale it"
	| form |
	form := self getClipartForm: aRelativePathName.
	^ form magnify: form boundingBox by: Stoa scale smoothing: 3! !

!Stoa class methodsFor: 'accessing' stamp: 'HilaireFernandes 8/11/2014 11:24'!
getClipart: aRelativePathName height: thisHeight 
"Rescale as if clipart height was thisHeight"
	| form realHeight reScale|
	form := self getClipartForm: aRelativePathName.
	realHeight := Stoa scale: thisHeight .
	reScale := realHeight / form height.
	^ form magnifyBy: reScale! !

!Stoa class methodsFor: 'accessing' stamp: 'HilaireFernandes 8/11/2014 11:24'!
getClipart: aRelativePathName width: thisWidth 
"Rescale as if clipart width was thisWidth"
	| form realWidth reScale|
	form := self getClipartForm: aRelativePathName.
	realWidth := Stoa scale: thisWidth .
	reScale := realWidth / form width.
	^ form magnifyBy: reScale! !

!Stoa class methodsFor: 'accessing' stamp: 'HilaireFernandes 5/5/2014 13:38'!
getClipartForm: aRelativeOrNotPathName
	| file |
	file := aRelativeOrNotPathName isRelative 
		ifTrue: [Platform clipartsPath resolve: aRelativeOrNotPathName]
		ifFalse: [aRelativeOrNotPathName].
	file exists ifFalse: [self error: file fullName , ' not found in library.'].
	^ ImageReadWriter formFromFileNamed: file fullName! !

!Stoa class methodsFor: 'graphics - breakfast' stamp: 'HilaireFernandes 1/18/2014 10:02'!
glass
	^ (Stoa getClipart: Path * 'furnitures' / 'glassJuice.png') asMorph: #glass.
! !

!Stoa class methodsFor: 'graphics - token' stamp: 'HilaireFernandes 10/24/2014 18:12'!
greenBall
	^ (Stoa getClipart: Path * 'balls' / 'greenBall.png') asMorph: #greenBall.
! !

!Stoa class methodsFor: 'helpers' stamp: 'HilaireFernandes 12/26/2013 12:20'!
hfill
	^ Morph new color: Color transparent;
		 height: 0;
		 hResizing: #spaceFill;
		 vResizing: #rigid! !

!Stoa class methodsFor: 'graphics - clothes' stamp: 'HilaireFernandes 10/24/2014 15:45'!
hoodie
	^ (Stoa getClipart: Path * 'clothes' / 'hoodie.png') asMorph: #hoodie.! !

!Stoa class methodsFor: 'colors' stamp: 'HilaireFernandes 1/8/2014 10:34'!
illustrationColor
	^ Color green muchLighter ! !

!Stoa class methodsFor: 'graphics - clothes' stamp: 'HilaireFernandes 10/24/2014 15:45'!
jacket
	^ (Stoa getClipart: Path * 'clothes' / 'jacket.png') asMorph: #jacket.! !

!Stoa class methodsFor: 'graphics - breakfast' stamp: 'HilaireFernandes 6/8/2014 14:38'!
knife
	^ (Stoa getClipart: Path * 'furnitures' / 'knife.png') asMorph: #knife
! !

!Stoa class methodsFor: 'graphics decorated' stamp: 'HilaireFernandes 6/7/2014 16:42'!
largeCloud
"31 decorations with size 64x64"
	| morph |
	morph := (Stoa getClipart: Path * 'misc' / 'decoratedLargeCloud.png') asDecoratedMorph.
	morph decoratePositions: (Stoa scale: {434@75. 521@89. 266@124. 357@144. 448@196. 533@196. 610@196.  288@243.  216@275. 377@267.  145@309. 487@309. 565@324. 651@344. 72@351. 319@357. 417@381. 728@381. 247@391. 179@423.  502@423. 597@436.  338@473.  412@497.  256@516. 179@541. 487@541. 632@560. 330@582. 425@616. 549@616}).
	^ morph! !

!Stoa class methodsFor: 'text' stamp: 'HilaireFernandes 11/3/2014 21:40'!
largeFontSize 
	 ^ (self defaultFontSize * 2) asInteger! !

!Stoa class methodsFor: 'text' stamp: 'HilaireFernandes 8/6/2014 23:21'!
largeTextFont
	^(Stoa textFontOfSize: self largeFontSize) ! !

!Stoa class methodsFor: 'user' stamp: 'HilaireFernandes 3/26/2014 10:49'!
loadUser: aString
	self loadUserFile: Platform usersPath / (StoaUser new username: aString) asFilename.
! !

!Stoa class methodsFor: 'user' stamp: 'HilaireFernandes 3/26/2014 09:12'!
loadUserFile: aFileEntry
	User := FLMaterializer materializeFromFileNamed: aFileEntry fullName.! !

!Stoa class methodsFor: 'dialog' stamp: 'HilaireFernandes 4/24/2014 10:20'!
mascot: mascotMorph message: aString
	self mascot: mascotMorph message: aString final: false! !

!Stoa class methodsFor: 'dialog' stamp: 'HilaireFernandes 4/24/2014 10:25'!
mascot: mascotMorph message: aString final: isFinal
"A modal morph with the mascotMorph delivering a message and waiting for the user acknowledge
isFinal = true means once the ok button is pressed we do not need to put back the original message where this dialog was plugged
"
| morph verticalMorph button|
	morph := Morph new color: Color transparent.
	morph layoutPolicy: TableLayout new;
		listDirection: #leftToRight;
		cellInset: (Stoa scale: 10@0);
		cellPositioning: #topLeft;
		hResizing: #spaceFill; vResizing: #shrinkWrap.
	verticalMorph := Morph new color: Color transparent.
	verticalMorph layoutPolicy: TableLayout new;
		listDirection: #topToBottom;
		cellInset: (Stoa scale: 0@10);
		cellPositioning: #topRight;
		hResizing: #spaceFill; vResizing: #shrinkWrap.
		
	verticalMorph 
		addMorphBack:  (Stoa textBubble: aString);
		addMorphBack: (button := ActiveView current newOKButton).
	isFinal ifTrue: [ button actionSelector: #okFinal].
		
	morph 
		addMorphBack: mascotMorph; 
		addMorphBack: verticalMorph.
	ActiveView current showDialog: morph.! !

!Stoa class methodsFor: 'graphics mascot' stamp: 'HilaireFernandes 4/24/2014 10:05'!
mascotGreenFlag
	^ (Stoa getClipart: Path * 'mascots' / 'tux-green-flag.png') asMorph: #mascotGreen.
! !

!Stoa class methodsFor: 'dialog' stamp: 'HilaireFernandes 5/17/2014 15:19'!
mascotMessageOnError
	self 
		mascot: Stoa mascotOrangeFlag 
		message: self messagesOnError atRandom! !

!Stoa class methodsFor: 'dialog' stamp: 'HilaireFernandes 7/28/2014 00:42'!
mascotMessageOnErrors
	self 
		mascot: Stoa mascotOrangeFlag 
		message: self messagesOnErrors atRandom! !

!Stoa class methodsFor: 'dialog' stamp: 'HilaireFernandes 7/28/2014 00:40'!
mascotMessageOnRepeatedErrors
	self 
		mascot: Stoa mascotRedFlag 
		message: self messagesOnRepeatedErrors atRandom 
		final: true! !

!Stoa class methodsFor: 'dialog' stamp: 'HilaireFernandes 5/17/2014 15:19'!
mascotMessageOnSuccess
	self 
		mascot: Stoa mascotGreenFlag 
		message: self messagesOnSuccess atRandom 
		final: true! !

!Stoa class methodsFor: 'graphics mascot' stamp: 'HilaireFernandes 4/24/2014 10:05'!
mascotOrangeFlag
	^ (Stoa getClipart: Path * 'mascots' / 'tux-orange-flag.png') asMorph: #mascotOrange.
! !

!Stoa class methodsFor: 'graphics mascot' stamp: 'HilaireFernandes 4/24/2014 10:04'!
mascotRedFlag
	^ (Stoa getClipart: Path * 'mascots' / 'tux-red-flag.png') asMorph: #mascotRed.
! !

!Stoa class methodsFor: 'dialog' stamp: 'HilaireFernandes 4/22/2014 18:08'!
mascotSays: aString
"A morph with the mascot delivering an information"
| morph |
	morph := Morph new color: Color transparent.
	morph layoutPolicy: TableLayout new;
		listDirection: #leftToRight;
		cellInset: (Stoa scale: 10@0);
		cellPositioning: #topLeft;
		hResizing: #spaceFill;
		vResizing: #shrinkWrap;
		addMorphBack: Stoa postman;
		addMorphBack: (Stoa textBubble: aString).
	^ morph
		! !

!Stoa class methodsFor: 'text' stamp: 'HilaireFernandes 1/21/2014 15:56'!
memoValue: aValue min: min max: max
	^ DecoratedNumberSliderMorph 
		value: aValue min: min max: max
		withForm: (self getClipart: (Path * 'decorations' / 'paper-frame2.png'))! !

!Stoa class methodsFor: 'dialog' stamp: 'HilaireFernandes 5/28/2014 16:50'!
messagesOnError
	^ Array 
		with: 'You made a mistake. I highlighted it in red.
Don''t worry. Try again to make it right!!' translated
		with: 'Oh!! You have an error shown in red.
It is ok you can do it again!!' translated
		with: 'I see an error. It is coloured in red so you can change it!!' translated! !

!Stoa class methodsFor: 'dialog' stamp: 'HilaireFernandes 8/11/2014 10:02'!
messagesOnErrors
	^ Array 
		with: 'You made some mistakes. I highlighted the errors in red.
Don''t worry. Try again to make it right!!' translated
		with: 'Oh!! You have a few errors shown in red.
It is ok you can do it again!!' translated
		with: 'I see some errors. It is coloured in red so you can change it!!' translated! !

!Stoa class methodsFor: 'dialog' stamp: 'HilaireFernandes 7/28/2014 00:40'!
messagesOnRepeatedErrors
	^ Array 
		with: 'It looks too difficult for you. You can try again later!!
Now I will show you the answers.'  translated
		with: 'This one is resisting you. Don''t worry, you can come back later.
Ready to see the answers?' translated
		with: 'Oh!! Wrong again. Never mind, you will do it right next time.
Be ready to look at the answers.' translated! !

!Stoa class methodsFor: 'dialog' stamp: 'HilaireFernandes 5/28/2014 17:02'!
messagesOnSuccess
	^ Array 
		with: 'Well done!! It is all correct.'  translated
		with: 'Fantastic!! It is all good.' translated
		with: 'Good job!! All your answers are correct.' translated! !

!Stoa class methodsFor: 'graphics - breakfast' stamp: 'HilaireFernandes 8/11/2014 11:25'!
misoSoup
	^ (Stoa getClipart: Path * 'breakfast' / 'misoSoup.png' width: 64) asMorph: #misoSoup.! !

!Stoa class methodsFor: 'graphics - breakfast' stamp: 'HilaireFernandes 8/11/2014 11:26'!
muffin
	^ (Stoa getClipart: Path * 'breakfast' / 'muffin.png' width: 64) asMorph: #muffin.! !

!Stoa class methodsFor: 'geometry' stamp: 'HilaireFernandes 12/5/2013 14:06'!
myDisplayExtent
	^ MyDisplayExtent ifNil: [MyDisplayExtent := self defaultDisplayExtent]! !

!Stoa class methodsFor: 'geometry' stamp: 'HilaireFernandes 12/11/2013 11:54'!
myDisplayExtent: extent
	MyDisplayExtent := extent.
	self resetScale ! !

!Stoa class methodsFor: 'text' stamp: 'HilaireFernandes 1/21/2014 15:55'!
noteValue: aValue min: min max: max
	^ DecoratedNumberSliderMorph 
		value: aValue min: min max: max
		withForm: (self getClipart: (Path * 'decorations' / 'paper-frame1.png'))! !

!Stoa class methodsFor: 'view' stamp: 'HilaireFernandes 4/4/2014 21:31'!
openAppView
	StoaPlay delete.
	StoaScore delete.
	StoaSettings delete.
	ActiveView := StoaApp.
	StoaApp current openInWorld.
! !

!Stoa class methodsFor: 'view' stamp: 'HilaireFernandes 5/20/2014 14:56'!
openPlayViewOn: etayageClass
	StoaApp delete.
	StoaScore delete.
	StoaSettings delete.
	ActiveView := StoaPlay.
	(StoaPlay currentOn: etayageClass) openInWorld.
! !

!Stoa class methodsFor: 'view' stamp: 'HilaireFernandes 4/4/2014 21:32'!
openScoreView
	StoaPlay delete.
	StoaSettings delete.
	StoaApp delete.
	ActiveView := StoaScore.
	StoaScore current openInWorld;
		stopSteppingSelfAndSubmorphs;
		startStepping.
! !

!Stoa class methodsFor: 'view' stamp: 'HilaireFernandes 4/4/2014 21:32'!
openSettingsView
	StoaPlay delete.
	StoaScore delete.
	StoaApp delete.
	ActiveView := StoaSettings.
	StoaSettings current openInWorld.
! !

!Stoa class methodsFor: 'graphics - fruits' stamp: 'HilaireFernandes 1/18/2014 09:59'!
orange
	^ (Stoa getClipart: Path * 'fruits' / 'orange.png') asMorph: #orange.
! !

!Stoa class methodsFor: 'graphics - token' stamp: 'HilaireFernandes 10/24/2014 18:12'!
orangeBall
	^ (Stoa getClipart: Path * 'balls' / 'orangeBall.png') asMorph: #orangeBall! !

!Stoa class methodsFor: 'graphics - fruits' stamp: 'HilaireFernandes 1/18/2014 09:59'!
pear
	^ (Stoa getClipart: Path * 'fruits' / 'pear.png') asMorph: #pear.
! !

!Stoa class methodsFor: 'graphics' stamp: 'HilaireFernandes 5/17/2014 15:40'!
person1sm
	^ (Stoa getClipart: Path * 'persons' / 'person1sm.png') asMorph! !

!Stoa class methodsFor: 'graphics - breakfast' stamp: 'HilaireFernandes 1/18/2014 10:02'!
plate
	^ (Stoa getClipart: Path * 'furnitures' / 'plate.png') asMorph: #plate.
! !

!Stoa class methodsFor: 'graphics - clothes' stamp: 'HilaireFernandes 10/24/2014 15:45'!
polo
	^ (Stoa getClipart: Path * 'clothes' / 'polo.png') asMorph: #polo.! !

!Stoa class methodsFor: 'graphics mascot' stamp: 'HilaireFernandes 4/22/2014 16:04'!
postman
	^ (Stoa getClipart: Path * 'mascots' / 'tux-message.png') asMorph: #postman.
! !

!Stoa class methodsFor: 'graphics - token' stamp: 'HilaireFernandes 10/24/2014 18:12'!
redBall
	^ (Stoa getClipart: Path * 'balls' / 'redBall.png') asMorph: #redBall! !

!Stoa class methodsFor: 'graphics' stamp: 'HilaireFernandes 7/22/2014 22:53'!
redCube
	^ (Stoa getClipart: Path * 'shapes' / 'redCube.png')! !

!Stoa class methodsFor: 'geometry' stamp: 'HilaireFernandes 11/27/2013 18:02'!
referenceDisplayExtent
"This is the display extent we use to design the application"
	^ 2560 @ 1600! !

!Stoa class methodsFor: 'system startup' stamp: 'HilaireFernandes 3/19/2014 09:20'!
removeStartUp
"self removeStartUp"
	Smalltalk removeFromStartUpList: self.! !

!Stoa class methodsFor: 'view' stamp: 'HilaireFernandes 4/5/2014 08:57'!
rescaleView
	Stoa myDisplayExtent = DisplayScreen actualScreenSize ifTrue: [ ^self ].
	Stoa myDisplayExtent: DisplayScreen actualScreenSize.
	DevelMode ifFalse: [Platform setFontPreferences].
	ActiveView ifNotNil: [ActiveView rescale].
! !

!Stoa class methodsFor: 'scale' stamp: 'HilaireFernandes 4/5/2014 08:23'!
resetScale
	Scale := nil.
	DefaultFontSize := nil.! !

!Stoa class methodsFor: 'colors' stamp: 'HilaireFernandes 1/7/2014 22:18'!
rightColor
	^ Color yellow lighter! !

!Stoa class methodsFor: 'helpers' stamp: 'HilaireFernandes 8/7/2014 10:28'!
rowMorph
	| morph |
	morph := Morph new color: Color transparent.
	morph layoutPolicy: TableLayout new;
		listDirection: #leftToRight;
		hResizing: #shrinkWrap;
		vResizing: #shrinkWrap;
		layoutInset: 0;
		cellInset: 0.
	^ morph! !

!Stoa class methodsFor: 'user' stamp: 'HilaireFernandes 4/25/2014 15:42'!
saveUser
	self saveUser: User! !

!Stoa class methodsFor: 'user' stamp: 'HilaireFernandes 4/25/2014 15:42'!
saveUser: anUser
	FLSerializer serialize: anUser toFileNamed: (Platform usersPath / anUser asFilename) fullName.! !

!Stoa class methodsFor: 'scale' stamp: 'HilaireFernandes 12/5/2013 14:07'!
scale
	"Recompute the scale according to the current extent"
	^ Scale ifNil: [ 
		Scale := (self myDisplayExtent x / self referenceDisplayExtent x min: 
			self myDisplayExtent y / self referenceDisplayExtent y) asFloat]! !

!Stoa class methodsFor: 'scale' stamp: 'HilaireFernandes 12/24/2013 10:24'!
scale: aNumberOrPointOrCollection
	^ aNumberOrPointOrCollection isNumber 
		ifTrue: [(aNumberOrPointOrCollection * self scale) asInteger]
		ifFalse: [aNumberOrPointOrCollection isPoint 
			ifTrue: [(aNumberOrPointOrCollection * self scale) asIntegerPoint]
			ifFalse:	[aNumberOrPointOrCollection collect: [ :aPoint | (aPoint  * self scale) asIntegerPoint]]]! !

!Stoa class methodsFor: 'user' stamp: 'HilaireFernandes 12/30/2013 11:53'!
score
	^ User ifNotNil: [ User score ] ifNil: [ 0 ]! !

!Stoa class methodsFor: 'user' stamp: 'HilaireFernandes 12/31/2013 11:11'!
score: anInteger at: anEtayageClass
	(User score: anInteger at: anEtayageClass) ifTrue: [self saveUser]! !

!Stoa class methodsFor: 'user' stamp: 'HilaireFernandes 12/30/2013 11:40'!
scoreAt: anEtayageClass
	^ User scoreAt: anEtayageClass! !

!Stoa class methodsFor: 'colors' stamp: 'HilaireFernandes 1/8/2014 10:36'!
selectColor
	^ Color blue! !

!Stoa class methodsFor: 'user' stamp: 'HilaireFernandes 4/3/2014 09:28'!
setUser
"Set the current user, depending on different scenario (Read comment on StoaApp>>installUser)
Return the entries in usersPath, if one entry: it has been loaded already, otherwise the sender should select one"
	| entries |
	User ifNotNil: [ ^ self "already installed" ].
	entries := Platform usersPath entries.
	entries 
		ifEmpty: [self createUser: 'Tux']
		ifNotEmpty: [ self loadUserFile: entries first].
! !

!Stoa class methodsFor: 'text' stamp: 'HilaireFernandes 11/3/2014 21:40'!
smallFontSize 
	 ^ ((self defaultFontSize * 8) / 10) asInteger max: 9! !

!Stoa class methodsFor: 'graphics - breakfast' stamp: 'HilaireFernandes 5/27/2014 15:44'!
spoon
	^ (Stoa getClipart: Path * 'furnitures' / 'spoon.png') asMorph: #spoon.
! !

!Stoa class methodsFor: 'graphics' stamp: 'HilaireFernandes 7/23/2014 12:26'!
stack: quantity cube: symbol
	| morph |
	morph :=  StackMorph 
		form: (symbol == #redCube ifTrue: [Stoa redCube] ifFalse: [Stoa blueCube]) 
		offset: (Stoa scale: -29) 
		quantity: quantity 
		orientation: #bottomToTop.
	morph setProperty: #itemType toValue: (symbol, quantity asString) asSymbol. "for example #redCube2"
	^ morph! !

!Stoa class methodsFor: 'graphics' stamp: 'HilaireFernandes 7/23/2014 12:25'!
stackBlueCube: quantity
	^ self stack: quantity cube: #blueCube! !

!Stoa class methodsFor: 'graphics' stamp: 'HilaireFernandes 7/23/2014 12:25'!
stackRedCube: quantity
	^ self stack: quantity cube: #redCube! !

!Stoa class methodsFor: 'system startup' stamp: 'HilaireFernandes 3/19/2014 09:21'!
startUp
	Platform startUp! !

!Stoa class methodsFor: 'colors' stamp: 'HilaireFernandes 1/26/2014 15:10'!
stateColor
	^ Color blue! !

!Stoa class methodsFor: 'text' stamp: 'HilaireFernandes 4/4/2014 21:25'!
stringMorph: contents
	^ ActiveView current
		newString: contents asString
		font: self textFont 
		style: #inset! !

!Stoa class methodsFor: 'text' stamp: 'HilaireFernandes 8/6/2014 23:22'!
stringMorphLarge: contents
	^ ActiveView current
		newString: contents asString
		font: self largeTextFont 
		style: #inset! !

!Stoa class methodsFor: 'colors' stamp: 'HilaireFernandes 1/8/2014 10:23'!
subAreaColor
	^ Color paleTan lighter! !

!Stoa class methodsFor: 'graphics decorated' stamp: 'HilaireFernandes 5/17/2014 15:39'!
table
"Decoration must be 64x64"
	| morph |
	morph := (Stoa getClipart: Path * 'furnitures' / 'table.png') asDecoratedMorph.
	morph decoratePositions: (Stoa scale: {54@33. 102@32. 157@31. 210@32. 264@32. 102@80. 157@80. 210@80. 264@80. 316@80}).
	^ morph! !

!Stoa class methodsFor: 'graphics decorated' stamp: 'HilaireFeranndes 3/23/2014 09:55'!
target
	| morph positions associations |
	morph := DecoratedSymbolicMorph new form: (Stoa getClipart: Path * 'misc' / 'target.png').
	positions := Stoa scale: {158@18. 108@83. 250@95. 170@111. 153@243. 210@220}.
	associations := positions collectWithIndex: [ :point :index | ( 'point', (index - 1) asString) asSymbol -> point].
	morph decoratePositions: associations.
	^ morph! !

!Stoa class methodsFor: 'colors' stamp: 'HilaireFernandes 1/8/2014 10:45'!
targetColor
	^ Color paleRed! !

!Stoa class methodsFor: 'text' stamp: 'HilaireFernandes 4/22/2014 17:19'!
textBubble: string 
	| textMorph |
	textMorph := Stoa textMorph: string.
	textMorph backgroundColor: Color white;
		borderStyle: (BorderStyle complexFramed width: (Stoa scale: 8));
		margins: (Stoa scale: 20).
	^ textMorph
		! !

!Stoa class methodsFor: 'text' stamp: 'HilaireFernandes 3/16/2014 09:12'!
textFont
	^Platform defaultFont! !

!Stoa class methodsFor: 'text' stamp: 'HilaireFernandes 4/27/2014 12:39'!
textFontOfSize: integer
	^Platform fontSize: integer! !

!Stoa class methodsFor: 'text' stamp: 'HilaireFernandes 4/22/2014 16:19'!
textMorph: contents
	^ (ActiveView current
		newText: contents)
		font: self textFont;
		wrapFlag: true;
		hResizing: #spaceFill! !

!Stoa class methodsFor: 'text' stamp: 'HilaireFernandes 8/6/2014 23:22'!
textMorphLarge: contents
	^ (ActiveView current
		newText: contents)
		font: self largeTextFont ;
		wrapFlag: true;
		hResizing: #spaceFill! !

!Stoa class methodsFor: 'graphics - clothes' stamp: 'HilaireFernandes 10/24/2014 15:44'!
tshirt
	^ (Stoa getClipart: Path * 'clothes' / 'tshirt.png') asMorph: #tshirt.! !

!Stoa class methodsFor: 'user' stamp: 'HilaireFernandes 5/5/2014 19:13'!
userAvatar
	^ User avatarMorph ! !

!Stoa class methodsFor: 'user' stamp: 'HilaireFernandes 3/26/2014 10:49'!
username
	^ User username! !

!Stoa class methodsFor: 'user' stamp: 'HilaireFernandes 4/25/2014 14:47'!
users
	^ (Platform usersPath filesMatching: '*.fl') collect: [ :file |
		FLMaterializer materializeFromFileNamed: file fullName]! !

!Stoa class methodsFor: 'path' stamp: 'HilaireFernandes 5/5/2014 19:09'!
usersPath
	^ Platform usersPath ! !

!Stoa class methodsFor: 'helpers' stamp: 'HilaireFernandes 1/15/2014 20:42'!
vfill
	^ Morph new color: Color transparent;
		 height: 0;
		 hResizing: #rigid;
		 vResizing: #spaceFill! !

!Stoa class methodsFor: 'sound' stamp: 'HilaireFernandes 2/18/2014 10:32'!
voice
	^ Voice ifNil: [ Voice := false ]! !

!Stoa class methodsFor: 'sound' stamp: 'HilaireFernandes 2/18/2014 10:32'!
voice: aBoolean
	Voice := aBoolean! !

!Stoa class methodsFor: 'graphics - fruits' stamp: 'HilaireFernandes 1/18/2014 19:05'!
walnut
	^ (Stoa getClipart: Path * 'fruits' / 'walnut.png') asMorph: #walnut.
! !

!Stoa class methodsFor: 'colors' stamp: 'HilaireFernandes 1/8/2014 10:35'!
wrongColor
	^ Color red alpha: 0.8! !

!Stoa class methodsFor: 'graphics - token' stamp: 'HilaireFernandes 10/24/2014 18:13'!
yellowBall
	^ (Stoa getClipart: Path * 'balls' / 'yellowBall.png') asMorph: #yellowBall! !

Object subclass: #StoaPlatform
	instanceVariableNames: 'myFont'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'iStoaXP-Core'!

!StoaPlatform commentStamp: '<historical>' prior: 0!
A StoaPlateform is an abstraction to access underneath plateform information, settings and preferences.
!

StoaPlatform subclass: #StoaPcPlatform
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'iStoaXP-Core'!

!StoaPcPlatform commentStamp: '<historical>' prior: 0!
A StoaWorkstation to access common plateform settings to workstation.!

!StoaPcPlatform methodsFor: 'testing' stamp: 'HilaireFernandes 3/16/2014 09:19'!
isPC
	^ true! !

StoaPcPlatform subclass: #StoaUnixPlatform
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'iStoaXP-Core'!

!StoaUnixPlatform commentStamp: '<historical>' prior: 0!
A StoaUnixPlateform for specific settings to Unix plateform.!

!StoaUnixPlatform methodsFor: 'path' stamp: 'HilaireFernandes 12/5/2013 14:11'!
applicationRootPath
	^  FileSystem disk workingDirectory parent! !

!StoaUnixPlatform methodsFor: 'testing' stamp: 'HilaireFernandes 3/16/2014 09:19'!
isUnix
	^ true! !

StoaPcPlatform subclass: #StoaWin32Platform
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'iStoaXP-Core'!

!StoaWin32Platform commentStamp: '<historical>' prior: 0!
A StoaWin32Plateform for specific settings to Windows plateform.!

!StoaWin32Platform methodsFor: 'path' stamp: 'HilaireFernandes 12/5/2013 14:12'!
applicationRootPath
	^  FileSystem disk workingDirectory! !

!StoaWin32Platform methodsFor: 'path' stamp: 'HilaireFernandes 3/16/2014 09:19'!
isWin32
	^ true! !

!StoaPlatform methodsFor: 'path' stamp: 'HilaireFernandes 12/5/2013 14:14'!
applicationRootPath
"path to the root of the application, where all the resources are rooted"
	self subclassResponsibility ! !

!StoaPlatform methodsFor: 'path' stamp: 'HilaireFernandes 4/25/2014 14:44'!
avatarsPath
	^ self clipartsPath /  'avatars'.! !

!StoaPlatform methodsFor: 'path' stamp: 'HilaireFernandes 12/10/2013 21:45'!
backgroundsPath
	^ self applicationRootPath /  'Backgrounds'.! !

!StoaPlatform methodsFor: 'path' stamp: 'HilaireFernandes 3/16/2014 09:31'!
checkForPaths
	<todo>
	"We may need to create the users directory as it iw read and write"! !

!StoaPlatform methodsFor: 'path' stamp: 'HilaireFernandes 12/10/2013 21:47'!
clipartsPath
	^ self applicationRootPath /  'Cliparts'.! !

!StoaPlatform methodsFor: 'font' stamp: 'HilaireFernandes 4/5/2014 08:26'!
defaultFont
	^ LogicalFont familyName: self fontName pointSize: Stoa defaultFontSize! !

!StoaPlatform methodsFor: 'path' stamp: 'HilaireFernandes 12/15/2013 11:34'!
etayagesPath
	^ self applicationRootPath / 'Etayages'! !

!StoaPlatform methodsFor: 'font' stamp: 'HilaireFernandes 4/27/2014 16:26'!
fontFile
| currentLocale |
	currentLocale := Locale current isoLanguage.
	(#('ko' 'ko_KR') includes: currentLocale) ifTrue: [
		myFont := 'UnDotum'.
		^ 'UnDotum.ttf'].
	(#('zh_CN' 'zh_TW' 'zh_HK' 'zh_SG' 'ja' 'ja_JP') includes: currentLocale) ifTrue:  [
		myFont := 'WenQuanYi Micro Hei'.
		^ 'wqy-microhei.ttc'].
	myFont := 'Andika'.
	^'Andika-R.ttf'! !

!StoaPlatform methodsFor: 'font' stamp: 'HilaireFernandes 4/27/2014 16:26'!
fontName
	^ myFont ! !

!StoaPlatform methodsFor: 'path' stamp: 'HilaireFernandes 4/5/2014 09:42'!
fontPath
	^ self applicationRootPath / 'Fonts'! !

!StoaPlatform methodsFor: 'font' stamp: 'HilaireFernandes 4/27/2014 12:39'!
fontSize: integer
	^ LogicalFont familyName: self fontName pointSize: integer! !

!StoaPlatform methodsFor: 'font' stamp: 'HilaireFernandes 4/27/2014 16:20'!
installFont
	| font |
	myFont := nil.
	FreeTypeFontProvider unload.
	font := self fontPath resolve: self fontFile.
	font ifNotNil: [
		FreeTypeFontProvider current updateFromFile: font.
		FreeTypeFontProvider current buildFamilies ].! !

!StoaPlatform methodsFor: 'testing' stamp: 'HilaireFernandes 3/16/2014 09:18'!
isPC
	^ false! !

!StoaPlatform methodsFor: 'testing' stamp: 'HilaireFernandes 3/16/2014 09:18'!
isUnix
	^ false! !

!StoaPlatform methodsFor: 'testing' stamp: 'HilaireFernandes 3/16/2014 09:18'!
isWin32
	^ false! !

!StoaPlatform methodsFor: 'font' stamp: 'HilaireFernandes 4/27/2014 12:31'!
largeFont
	^ LogicalFont familyName: self fontName pointSize: Stoa largeFontSize! !

!StoaPlatform methodsFor: 'font' stamp: 'HilaireFernandes 4/5/2014 08:40'!
setFontPreferences
	StandardFonts codeFont: (LogicalFont familyName: self fontName pointSize: Stoa defaultFontSize).
	StandardFonts listFont: (LogicalFont familyName: self fontName pointSize: Stoa defaultFontSize).
	StandardFonts menuFont: (LogicalFont familyName: self fontName pointSize: Stoa defaultFontSize).
	StandardFonts windowTitleFont: (LogicalFont familyName: self fontName pointSize: Stoa defaultFontSize).
	StandardFonts balloonFont: (LogicalFont familyName: self fontName pointSize: Stoa smallFontSize).
	StandardFonts haloFont: (LogicalFont familyName: self fontName  pointSize: Stoa smallFontSize).
	StandardFonts buttonFont: (LogicalFont familyName: self fontName  pointSize: Stoa defaultFontSize).
	StandardFonts defaultFont: (LogicalFont familyName: self fontName  pointSize: Stoa defaultFontSize).
! !

!StoaPlatform methodsFor: 'system startup' stamp: 'HilaireFernandes 4/5/2014 12:17'!
startUp
	self installFont.
	self checkForPaths.
	Stoa rescaleView.
	Stoa openAppView! !

!StoaPlatform methodsFor: 'path' stamp: 'HilaireFernandes 12/5/2013 14:11'!
templatePath
	^ self applicationRootPath  / 'TemplateActivity'.
! !

!StoaPlatform methodsFor: 'path' stamp: 'HilaireFernandes 12/30/2013 10:46'!
usersPath
	^ self applicationRootPath /  'Users'.! !

Object subclass: #StoaUser
	instanceVariableNames: 'name scorePerEtayage score avatar'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'iStoaXP-Core'!

!StoaUser commentStamp: '<historical>' prior: 0!
A user model. For now we just want 

- the score per Etayage (in a dictionary)
- the total score!

!StoaUser methodsFor: 'accessing' stamp: 'HilaireFernandes 5/5/2014 14:00'!
asFilename
	^ self asLegalSelectorUsername, '.fl'! !

!StoaUser methodsFor: 'accessing' stamp: 'HilaireFernandes 5/5/2014 14:00'!
asLegalSelectorUsername
	^ self username asLegalSelector! !

!StoaUser methodsFor: 'avatar' stamp: 'HilaireFernandes 4/25/2014 15:39'!
avatar
	^ avatar ifNil: [ avatar := 'testdummy-tux.png' ]! !

!StoaUser methodsFor: 'avatar' stamp: 'HilaireFernandes 4/25/2014 15:29'!
avatar: aFilename
	avatar := aFilename ! !

!StoaUser methodsFor: 'avatar' stamp: 'HilaireFernandes 5/5/2014 19:14'!
avatarMorph
	^ (self personalizedAvatar 
		ifNotNilDo: [:filename| Stoa getClipart: filename]
		ifNil: [ Stoa getClipart: Stoa avatarsPath / self avatar]) asMorph! !

!StoaUser methodsFor: 'initialize-release' stamp: 'HilaireFernandes 12/26/2013 16:29'!
initialize
	super initialize.
	scorePerEtayage := Dictionary new.! !

!StoaUser methodsFor: 'avatar' stamp: 'HilaireFernandes 5/5/2014 20:21'!
personalizedAvatar
"answer a file reference if we can find a personalized avatar, otherwise nil"
	| fileReference searchedName|
	searchedName := self asLegalSelectorUsername.
	^ #('-avatar.png' '.png' '.jpg' '.jpeg' '.JPG') 
		detect: [:ext | (fileReference := Stoa usersPath resolve: searchedName, ext) exists ]
		ifFound: [:ext |
			ext = '-avatar.png' ifFalse: [	self rescalePersonalizedAvatar: fileReference].
			Stoa usersPath / (searchedName, '-avatar.png')]
		ifNone: [nil]! !

!StoaUser methodsFor: 'printing' stamp: 'HilaireFernandes 4/25/2014 18:26'!
printOn: aStream
	aStream nextPutAll: 'User: ';
		nextPutAll: name;
		nextPut: Character space;
		nextPut: $(;
		nextPutAll: avatar;
		nextPut: $)! !

!StoaUser methodsFor: 'avatar' stamp: 'HilaireFernandes 5/5/2014 20:34'!
rescalePersonalizedAvatar: fileReference
	| form |
	form := ImageReadWriter formFromFileNamed: fileReference fullName.
	form := form 
		magnify: form boundingBox
		by:  160 / form height
		smoothing: 3.
	PNGReadWriter 
		putForm: form
		onFileNamed: (Stoa usersPath / (self asLegalSelectorUsername, '-avatar.png')) fullName! !

!StoaUser methodsFor: 'accessing' stamp: 'HilaireFernandes 12/26/2013 16:37'!
score
"The complete user score"
	^ score ifNil: [ score := scorePerEtayage keys inject: 0 into: [:subScore :anEtayage |
			subScore + (self scoreAt: anEtayage) ] ]! !

!StoaUser methodsFor: 'accessing' stamp: 'HilaireFernandes 12/31/2013 11:17'!
score: anInteger at: anEtayageClass
"Return a boolean to indicate if the user score changed"
	^ (self scoreAt: anEtayageClass) <= anInteger 
		ifTrue: [  
			scorePerEtayage at: anEtayageClass put: anInteger.
			score := nil.  "reset it, it must be recomputed"
			true]
		ifFalse: [ false ]! !

!StoaUser methodsFor: 'accessing' stamp: 'HilaireFernandes 12/26/2013 16:30'!
scoreAt: anEtayageClass
	^ scorePerEtayage at: anEtayageClass ifAbsent: [ 0]! !

!StoaUser methodsFor: 'accessing' stamp: 'HilaireFernandes 3/26/2014 10:49'!
username
	^ name! !

!StoaUser methodsFor: 'accessing' stamp: 'HilaireFernandes 3/26/2014 10:49'!
username: aString
	name := aString! !
